@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: arg.c,v 1.14 98/03/09 10:49:58 abe Exp $";
+static char *rcsid = "$Id: arg.c,v 1.20 98/09/14 13:47:24 abe Exp $";
 #endif
 
 
@@ -40,6 +40,7 @@ static char *rcsid = "$Id: arg.c,v 1.14 98/03/09 10:49:58 abe Exp $";
 
 
 _PROTOTYPE(static int ckfd_range,(char *first, char *dash, char *last, int *lo, int *hi));
+_PROTOTYPE(static void enter_fd_lst,(char *nm, int lo, int hi));
 _PROTOTYPE(static int enter_nwad,(struct nwad *n, int sp, int ep, char *s, struct hostent *he));
 
 
@@ -142,7 +143,7 @@ ck_file_arg(i, ac, av)
 	 */
 	    if (path != av[i] && (j = strlen(path)) > 1 && path[j-1] == '/')
 		path[j-1] = '\0';
-	    for (ftype = 1, mp = Mtab, nm = 0;
+	    for (ftype = 1, mp = readmnt(), nm = 0;
 		 (Ffilesys != 1) && mp;
 		 mp = mp->next)
 	    {
@@ -226,6 +227,8 @@ ck_file_arg(i, ac, av)
 			(void) fprintf(stderr, "%s: status error on ", Pn);
 			safestrprt(fnm, stderr, 0);
 			(void) fprintf(stderr, ": %s\n", strerror(errno_save));
+			Sfile = sfp->next;
+			(void) free((FREE_P *)sfp);
 			err = 1;
 			break;
 		    }
@@ -235,8 +238,8 @@ ck_file_arg(i, ac, av)
 #if	defined(CKFA_EXPDEV)
 		/*
 		 * Expand device numbers before saving, so that they match the
-		 * already-expanded Mtab device numbers.  (This is an EP/IX
-		 * 2.1.1 and above artifact.)
+		 * already-expanded local mount info table device numbers.
+		 * (This is an EP/IX 2.1.1 and above artifact.)
 		 */
 		    dev = expdev(sb.st_dev);
 		    rdev = expdev(sb.st_rdev);
@@ -358,7 +361,7 @@ ck_file_arg(i, ac, av)
 
 		else {
 		    for (j = pfsnl+1; path[j]; j++) {
-			if (!isdigit(path[(int)j]))
+			if (!isdigit((unsigned char)path[j]))
 			    break;
 		    }
 		    if (path[j] || (j - pfsnl - 1) < 1
@@ -513,11 +516,10 @@ enter_fd(f)
 	char buf[32], c, *cp1, *cp2, *dash;
 	int err, hi, lo;
 	char *fc;
-	MALLOC_S len;
 /*
  *  Check for non-empty list and make a copy.
  */
-	if (!f || (len = strlen(f) + 1) < 2) {
+	if (!f || (strlen(f) + 1) < 2) {
 	    (void) fprintf(stderr, "%s: no file descriptor specified\n", Pn);
 	    return(1);
 	}
@@ -545,20 +547,10 @@ enter_fd(f)
 		if (dash) {
 		    if (ckfd_range(cp1, dash, cp2, &lo, &hi))
 			err = 1;
-		    else {
-			while (lo <= hi) {
-			    (void) sprintf(buf, "%d", lo);
-			    if (enter_str_lst("d", buf, &Fdl)) {
-				err = 1;
-				break;
-			    }
-			    lo++;
-			}
-		    }
-		} else {
-		    if (enter_str_lst("d", cp1, &Fdl))
-			err = 1;
-		}
+		    else
+			(void) enter_fd_lst((char *)NULL, lo, hi);
+		} else
+		    (void) enter_fd_lst(cp1, 0, 0);
 	    }
 	    if (c == '\0')
 		break;
@@ -569,6 +561,62 @@ enter_fd(f)
 }
 
 
+/*
+ * enter_fd_lst() - make an entry in the FD list, Fdl
+ */
+
+static void
+enter_fd_lst(nm, lo, hi)
+	char *nm;			/* FD name (none if NULL) */
+	int lo;				/* FD low boundary (if nm NULL) */
+	int hi;				/* FD high boundary (if nm NULL) */
+{
+	char *cp;
+	struct fd_lst *f;
+	int n;
+/*
+ * Allocate an fd_lst entry.
+ */
+	if (!(f = (struct fd_lst *)malloc((MALLOC_S)sizeof(struct fd_lst)))) {
+	   (void) fprintf(stderr, "%s: no space for FD list entry\n", Pn);
+	   Exit(1);
+	}
+	if (nm) {
+
+	/*
+	 * Process an FD name.  First see if it contains only digits; if it
+	 * does, convert them to an integer and set the low and high
+	 * boundaries to the result.
+	 *
+	 * If the name has a non-digit, store it as a string, and set the
+	 * boundaries to impossible values (i.e., low > high).
+	 */
+	    for (cp = nm, n = 0; *cp; cp++) {
+		if (!isdigit((unsigned char)*cp))
+		    break;
+		n = (n * 10) + (int)(*cp - '0');
+	    }
+	    if (*cp) {
+		if (!(f->nm = mkstrcpy(nm, (MALLOC_S *)NULL))) {
+		    (void) fprintf(stderr,
+			"%s: no space for copy of: %s\n", Pn, nm);
+		    Exit(1);
+		}
+		lo = 1;
+		hi = 0;
+	    } else {
+		f->nm = (char *)NULL;
+		lo = hi = n;
+	    }
+	} else
+	    f->nm = (char *)NULL;
+	f->hi = hi;
+	f->lo = lo;
+	f->next = Fdl;
+	Fdl = f;
+}
+
+
 /*
  * enter_id() - enter PGRP or PID for searching
  */
@@ -686,13 +734,13 @@ int
 enter_network_address(na)
 	char *na;			/* Internet address string pointer */
 {
+	int ae, ep, i, pr, sp;
 	unsigned char *ap;
-	int ep, pr, sp;
+	char *cp, *p, *wa;
 	struct hostent *he = (struct hostent *)NULL;
 	char *hn = (char *)NULL;
 	MALLOC_S l;
 	struct nwad n;
-	char *p, *wa;
 	int pt = 0;
 	int pu = 0;
 	struct servent *se, *se1;
@@ -707,7 +755,10 @@ enter_network_address(na)
  * Initialize network address structure.
  */
 	n.proto = (char *)NULL;
-	n.a[0] = n.a[1] = n.a[2] = n.a[3] = 0;
+	n.af = 0;
+	for (i = 0; i < MAX_AF_ADDR; i++) {
+	    n.a[i] = 0;
+	}
 /*
  * Process protocol name, optionally followed by a '@' and a host name or
  * Internet address, or a ':' and a service name or port number.
@@ -753,8 +804,9 @@ nwad_exit:
 	    }
 	}
 /*
- * Process an Internet address (1.2.3.4) or host name, preceded by a '@'
- * and optionally followed by a colon and a service name or port number.
+ * Process an IPv4 address (1.2.3.4), IPv6 address ([1:2:3:4:5:6:7:8]),
+ * or host name, preceded by a '@' and optionally followed by a colon
+ * and a service name or port number.
  */
 	if (*wa == '@') {
 	    wa++;
@@ -766,10 +818,11 @@ unacc_address:
 		safestrprt(na, stderr, 1);
 		goto nwad_exit;
 	    }
-	    if (*wa < '0' || *wa > '9') {
-	/*
-	 * Assemble host name.
-	 */
+	    if (*wa < '0' || *wa > '9' && *wa != '[') {
+
+	    /*
+	     * Assemble host name.
+	     */
 		for (p = wa; *p && *p != ':'; p++)
 		    ;
 		if ((l = p - wa)) {
@@ -781,41 +834,99 @@ unacc_address:
 			safestrprt(na, stderr, 1);
 			goto nwad_exit;
 		    }
-		    if ((he = gethostbyname(hn)) == NULL) {
+
+#if	defined(HASIPv6)
+		    if (!(he = gethostbyname2(hn, AF_INET))
+		    ||  he->h_addrtype != AF_INET) {
+			if (!(he = gethostbyname2(hn, AF_INET6))
+			||  he->h_addrtype != AF_INET6)
+			    he = (struct hostent *)NULL;
+		    }
+		    if (!he)
+#else	/* !defined(HASIPv6) */
+		    if (!(he = gethostbyname(hn)))
+#endif	/* defined(HASIPv6) */
+
+		    {
 			fprintf(stderr, "%s: unknown host name (%s) in ",
 			    Pn, hn);
 			safestrprt(na, stderr, 1);
 			goto nwad_exit;
 		    }
 		    ap = (unsigned char *)he->h_addr;
+		    n.af = he->h_addrtype;
+
+#if	defined(HASIPv6)
+		    for (i = 0;
+			 i < (he->h_length - 1) && i < (MAX_AF_ADDR - 1);
+			 i++)
+		    {
+			 n.a[i] = *ap++;
+		    }
+		    n.a[i] = *ap;
+#else	/* !defined(HASIPv6) */
 		    n.a[0] = *ap++;
 		    n.a[1] = *ap++;
 		    n.a[2] = *ap++;
 		    n.a[3] = *ap;
+#endif	/* defined(HASIPv6) */
+
 		}
 		wa = p;
+	    } else if (*wa == '[') {
+
+#if	defined(HASIPv6)
+	    /*
+	     * Assemble IPv6 address.
+	     */
+		if (!(cp = strrchr(++wa, ']')))
+		    goto unacc_address;
+		*cp = '\0';
+		i = inet_pton(AF_INET6, wa, (void *)&n.a);
+		*cp = ']';
+		if (i != 1)
+		    goto unacc_address;
+		for (ae = i = 0; i < MAX_AF_ADDR; i++) {
+		    if ((ae |= n.a[i]))
+			break;
+		}
+		if (!ae)
+		    goto unacc_address;
+		n.af = AF_INET6;
+		wa = cp + 1;
+#else	/* !defined(HASIPv6) */
+		(void) fprintf(stderr,
+		    "%s: unsupported IPv6 address in ", Pn);
+		safestrprt(na, stderr, 1);
+		goto nwad_exit;
+#endif
+
+
 	    } else {
-	/*
-	 * Assemble Internet address.
-	 */
-		for (l = 0; *wa; wa++) {
+
+	    /*
+	     * Assemble IPv4 address.
+	     */
+		for (i = 0; *wa; wa++) {
 		    if (*wa == ':')
 			break;
 		    if (*wa == '.') {
-			l++;
-			if (l > 3)
+			i++;
+			if (i >= MIN_AF_ADDR)
 			    break;
 			continue;
 		    }
 		    if (*wa < '0' || *wa > '9')
 			goto unacc_address;
-		    n.a[l] = (10 * n.a[l]) + *wa - '0';
-		    if (n.a[l] > 255)
+		    ae = (10 * n.a[i]) + *wa - '0';
+		    if (ae > 255)
 			goto unacc_address;
+		    n.a[i] = ae;
 		}
-		if (l != 3
-		||  (n.a[0] == 0 && n.a[1] == 0 && n.a[2] == 0 && n.a[3] == 0))
+		if (i != (MIN_AF_ADDR - 1)
+		||  (!n.a[0] && !n.a[1] && !n.a[2] && !n.a[3]))
 		    goto unacc_address;
+		n.af = AF_INET;
 	    }
 	}
 /*
@@ -1013,7 +1124,15 @@ enter_nwad(n, sp, ep, s, he)
 	 * list.
 	 */
 	    if (!nc.proto
-	    &&  nc.a[0] == 0 && nc.a[1] == 0 && nc.a[2] == 0 && nc.a[3] == 0
+	    &&  !nc.a[0] && !nc.a[1] && !nc.a[2] && !nc.a[3]
+
+#if	defined(HASIPv6)
+	    &&  (nc.af != AF_INET6
+	    ||   (!nc.a[4]  && !nc.a[5]  && !nc.a[6]  && !nc.a[7]
+	    &&    !nc.a[8]  && !nc.a[9]  && !nc.a[10] && !nc.a[11]
+	    &&    !nc.a[12] && !nc.a[13] && !nc.a[14] && !nc.a[15]))
+#endif	/* defined(HASIPv6) */
+
 	    &&  sp == -1) {
 		(void) fprintf(stderr,
 		    "%s: incomplete Internet address specification: ", Pn);
@@ -1058,10 +1177,26 @@ enter_nwad(n, sp, ep, s, he)
 		break;
 	    if (!(ap = (unsigned char *)he->h_addr_list[ac++]))
 		break;
+
+#if	defined(HASIPv6)
+	    {
+		int i;
+
+		for (i = 0;
+		     (i < (he->h_length - 1)) && (i < (MAX_AF_ADDR - 1));
+		     i++)
+		{
+		    nc.a[i] = *ap++;
+		}
+		nc.a[i] = *ap;
+	    }
+#else	/* !defined(HASIPv6) */
 	    nc.a[0] = *ap++;
 	    nc.a[1] = *ap++;
 	    nc.a[2] = *ap++;
 	    nc.a[3] = *ap;
+#endif	/* defined(HASIPv6) */
+
 	}
 	return(0);
 }
