@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dstore.c,v 1.5 96/05/13 08:03:28 abe Exp $";
+static char *rcsid = "$Id: dstore.c,v 1.2 98/03/06 08:04:40 abe Exp $";
 #endif
 
 
@@ -54,32 +54,32 @@ char *AFSApath = (char *)NULL;		/* alternate AFS name list path
 					 * (from -a) */
 # endif /* defined(HASAOPT) */
 
-struct vfs *AFSVfsp = (struct vfs *)NULL;
+KA_T AFSVfsp = (KA_T)NULL;
 				/* AFS vfs struct kernel address */
 #endif	/* defined(HAS_AFS) */
 
-# if	_AIXV>=4140
+# if	AIXV>=4140
 struct clone *Clone = (struct clone *)NULL;
 					/* local clone information */
 int CloneMaj = -1;			/* clone major device number */
 int ClonePtc = -1;			/* /dev/ptc minor device number */
-# endif	/* _AIXV>=4140 */
+# endif	/* AIXV>=4140 */
 
-int Kmem = -1;				/* /dev/kmem file descriptor */
+int Kd = -1;				/* /dev/kmem file descriptor */
 struct l_vfs *Lvfs = NULL;		/* local vfs structure table */
-int Mem = -1;				/* /dev/mem file descriptor */
+int Km = -1;				/* /dev/mem file descriptor */
 
 struct nlist Nl[] = {
 
-#if	_AIXV<4100
-	{ "u",			 0, 0, 0, NULL, NULL },
-#else	/* _AIXV>=4100 */
-	{ "__ublock",		 0, 0, 0, NULL, NULL },
-#endif	/* _AIXV<4100 */
+#if	AIXV<4100
+	{ "u",			 0, 0, 0, 0, 0 },
+#else	/* AIXV>=4100 */
+	{ "__ublock",		 0, 0, 0, 0, 0 },
+#endif	/* AIXV<4100 */
 
 };
 
-#if	_AIXV>=4110
+#if	AIXV>=4110
 struct ublock __ublock;			/* dummy so we can define _KERNEL
 					 * for <sys/user.h> */
-#endif	/* _AIXV>=4110 */
+#endif	/* AIXV>=4110 */
