@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.12 98/02/16 19:47:11 abe Exp $
+ * $Id: machine.h,v 1.17 98/06/29 15:41:23 abe Exp $
  */
 
 
@@ -39,15 +39,34 @@
 #define	MACHINE_H	1
 
 
+# if	AIXV<=3250
+/*
+ * The AIX 3.2.5 and below <netdb.h> requires that _SUN be defined to
+ * be able to get the rpcent structure definition.
+ */
+
+#define	_SUN
+#include <netdb.h>
+#undef	_SUN
+# endif	/* AIXV<=3250 */
+
+
 # if	AIXV>=4200
 /*
  * AIX 4.2 requires that <sys/mstsave.h> be #include'd early.  It needs
  * <sys/types.h> and _KERNEL.
  *
+ * AIX 4.3 requires that _KERNEL be defined before the #include of
+ * <sys/types.h>
+ *
  * For gcc's sake, some redefinitions after including <sys/types.h> insure
  * the off64_t and offset_t types are aligned on an 8 byte boundary.
  */
  
+#  if	AIXV>=4300
+#define	_KERNEL	1
+#  endif	/* AIXV>=4300 */
+
 #include <sys/types.h>
 
 # if	defined(__GNUC__)
@@ -57,7 +76,10 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
 #define	offset_t	aligned_offset_t
 # endif	/* defined(__GNUC__) */
 
+#  if	AIXV<4300
 #define	_KERNEL	1
+#  endif	/* AIXV<4300 */
+
 #include <sys/mstsave.h>
 #undef	_KERNEL
 # endif	/* AIXV>=4200 */
@@ -174,6 +196,14 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
 /* #define	HASINTSIGNAL	1 */
 
 
+/*
+ * HASKERNIDCK is defined for those dialects that support the comparison of
+ * the build to running kernel identity.
+ */
+
+#define	HASKERNIDCK	1
+
+
 /*
  * HASKOPT is defined for those dialects that support the -k option of
  * reading the kernel's name list from an optional file.
@@ -424,6 +454,7 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
  * header files.
  */
 
+/* #define	USE_LIB_CKKV			1	   ckkv.c */
 /* #define	USE_LIB_COMPLETEVFS		1	   cvfs.c */
 #define	USE_LIB_FIND_CH_INO			1	/* fino.c */
 /* #define	USE_LIB_IS_FILE_NAMED		1	   isfn.c */
@@ -433,7 +464,6 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
 #define	USE_LIB_PRINT_TCPTPI			1	/* ptti.c */
 /* #define	USE_LIB_READDEV			1	   rdev.c */
 /* #define	USE_LIB_READMNT			1	   rmnt.c */
-/* #define	USE_LIB_READVFS			1	   rvfs.c */
 /* #define	USE_LIB_RNAM			1	   rnam.c */
 /* #define	USE_LIB_RNCH			1	   rnch.c */
 /* #define	USE_LIB_RNMH			1	   rnmh.c */
