@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.9 98/03/06 08:07:36 abe Exp $
+ * $Id: dlsof.h,v 1.10 98/04/29 13:25:59 abe Exp $
  */
 
 
@@ -151,7 +151,7 @@
 
 #define	COMP_P		const void
 #define DEVINCR		1024	/* device table malloc() increment */
-#define	KA_T		u_long
+typedef	u_long		KA_T;
 #define	KMEM		"/dev/kmem"
 #define MALLOC_P	void
 #define FREE_P		MALLOC_P
