@@ -32,24 +32,14 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.2 96/04/05 09:54:47 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.4 98/06/29 15:54:16 abe Exp $";
 #endif
 
 #include "lsof.h"
 
-#if     defined(HASNCACHE)
-#include <sys/uio.h>
-#include <sys/namei.h>
-#endif  /* defined(HASNCACHE) */
-
 
 _PROTOTYPE(static void enter_vn_text,(KA_T va, int *n));
 _PROTOTYPE(static void get_kernel_access,(void));
-
-#if     defined(HASNCACHE)
-_PROTOTYPE(static void ncache_load,(void));
-#endif  /* defined(HASNCACHE) */
-
 _PROTOTYPE(static void process_text,(KA_T tv, KA_T vm));
 
 
@@ -58,7 +48,49 @@ _PROTOTYPE(static void process_text,(KA_T tv, KA_T vm));
  */
 
 static MALLOC_S Nv = 0;			/* allocated Vp[] entries */
-static KA_T *Vp = NULL;			/* vnode address cache */
+static KA_T *Vp = (KA_T *)NULL;		/* vnode address cache */
+
+
+/*
+ * ckkv - check kernel version
+ */
+
+void
+ckkv(d, er, ev, ea)
+	char *d;			/* dialect */
+	char *er;			/* expected release */
+	char *ev;			/* expected version */
+	char *ea;			/* expected architecture */
+{
+
+#if	defined(HASKERNIDCK)
+	size_t l;
+	int m[2];
+	char v[64];
+
+	if (Fwarn)
+	    return;
+/*
+ * Read kernel version.
+ */
+	m[0] = CTL_KERN;
+	m[1] = KERN_OSRELEASE;
+	l = sizeof(v);
+	if (sysctl(m, 2, v, &l, NULL, 0) < 0) {
+	    (void) fprintf(stderr, "%s: CTL_KERN, KERN_OSRELEASE: %s\n",
+		Pn, strerror(errno));
+	    Exit(1);
+	}
+/*
+ * Warn if the actual and expected releases don't match.
+ */
+	if (!ev || strcmp(v, ev))
+	    (void) fprintf(stderr,
+		"%s: WARNING: compiled for %s release %s; this is %s.\n",
+		Pn, d, ev ? ev : "UNKNOWN", v);
+#endif	/* defined(HASKERNIDCK) */
+
+}
 
 
 /*
@@ -75,42 +107,32 @@ enter_vn_text(va, n)
  * Ignore the request if the vnode has already been entered.
  */
 	for (i = 0; i < *n; i++) {
-		if (va == Vp[i])
-			return;
+	    if (va == Vp[i])
+		return;
 	}
 /*
  * Save the text file information.
  */
 	alloc_lfile(" txt", -1);
-	process_node((caddr_t)va);
+	Cfp = (KA_T)NULL;
+	process_node(va);
 	if (Lf->sf)
-		link_lfile();
+	    link_lfile();
 	if (i >= Nv) {
 
 	/*
 	 * Allocate space for remembering the vnode.
 	 */
-		if (Vp == NULL) {
-			if ((Vp = (KA_T *)malloc((MALLOC_S)
-				   (sizeof(struct vnode *) * 10)))
-			== NULL) {
-				(void) fprintf(stderr,
-					"%s: no txt ptr space, PID %d\n",
-					Pn, Lp->pid);
-				Exit(1);
-			}
-			Nv = 10;
-		} else {
-			Nv += 10;
-			if ((Vp = (KA_T *)realloc((MALLOC_P *)Vp,
-				   (MALLOC_S)(Nv * sizeof(struct vnode *))))
-			== NULL) {
-				(void) fprintf(stderr,
-					"%s: no more txt ptr space, PID %d\n",
-					Pn, Lp->pid);
-				Exit(1);
-			}
-		}
+	    Nv += 10;
+	    if (!Vp)
+		Vp=(KA_T *)malloc((MALLOC_S)(sizeof(KA_T) * 10));
+	    else
+		Vp=(KA_T *)realloc((MALLOC_P *)Vp,(MALLOC_S)(Nv*sizeof(KA_T)));
+	    if (!Vp) {
+		(void) fprintf(stderr, "%s: no txt ptr space, PID %d\n",
+			Pn, Lp->pid);
+		Exit(1);
+	    }
 	}
 /*
  * Remember the vnode.
@@ -132,22 +154,17 @@ gather_proc_info()
 	static int nofb = 0;
 	static struct file **ofb = NULL;
 	struct kinfo_proc *p;
-	int pgrp, pid;
 	short pss, sf;
 	int px;
 	uid_t uid;
 /*
  * Read the process table.
  */
-	if ((P = kvm_getprocs(Kd, KERN_PROC_ALL, 0, &Np)) == NULL) {
-		(void) fprintf(stderr, "%s: can't read process table: %s\n",
-			Pn, kvm_geterr(Kd));
-		Exit(1);
+	if (!(P = kvm_getprocs(Kd, KERN_PROC_ALL, 0, &Np))) {
+	    (void) fprintf(stderr, "%s: can't read process table: %s\n",
+		Pn, kvm_geterr(Kd));
+	    Exit(1);
 	}
-/*
- * Initialize UID cache.
- */
-	(void) iuidcache(Np);
 
 #if     defined(HASNCACHE)
 /*
@@ -161,7 +178,8 @@ gather_proc_info()
  */
 
 	for (p = P, px = 0; px < Np; px++, p++) {
-
+	    if (p->P_STAT == 0 || p->P_STAT == SZOMB)
+		continue;
 	/*
 	 * Read process information, process group structure (if
 	 * necessary),  and User ID (if necessary).
@@ -170,89 +188,94 @@ gather_proc_info()
 	 *
 	 * Read file structure pointers.
 	 */
-		pgrp = p->P_PGRP;
-		uid = p->kp_eproc.e_ucred.cr_uid;
-		if (is_proc_excl(p->P_PID, pgrp, (UID_ARG)uid, &pss, &sf))
-			continue;
-		if (p->P_FD == NULL
-		||  kread((KA_T)p->P_FD, (char *)&fd, sizeof(fd)))
-			continue;
+	    uid = p->kp_eproc.e_ucred.cr_uid;
+	    if (is_proc_excl((int)p->P_PID, (int)p->P_PGRP, (UID_ARG)uid,
+		 &pss, &sf))
+	    {
+		continue;
+	    }
+	    if (!p->P_FD
+	    ||  kread((KA_T)p->P_FD, (char *)&fd, sizeof(fd)))
+		continue;
+	    if (!fd.fd_refcnt || fd.fd_lastfile > fd.fd_nfiles)
+		continue;
 	/*
 	 * Allocate a local process structure.
 	 */
-		if (is_cmd_excl(p->P_COMM, &pss, &sf))
-			continue;
-		alloc_lproc(p->P_PID, pgrp, (UID_ARG)uid, p->P_COMM,
-			(int)pss, (int)sf);
-		Plf = NULL;
+	    if (is_cmd_excl(p->P_COMM, &pss, &sf))
+		continue;
+	    alloc_lproc((int)p->P_PID, (int)p->P_PGRP, (int)p->P_PPID,
+		(UID_ARG)uid, p->P_COMM, (int)pss, (int)sf);
+	    Plf = (struct lfile *)NULL;
+	    Kpa = (KA_T)p->P_ADDR;
 	/*
 	 * Save current working directory information.
 	 */
-		if (fd.fd_cdir) {
-			alloc_lfile(CWD, -1);
-			process_node((caddr_t)fd.fd_cdir);
-			if (Lf->sf)
-				link_lfile();
-		}
+	    if (fd.fd_cdir) {
+		alloc_lfile(CWD, -1);
+		Cfp = (KA_T)NULL;
+		process_node((KA_T)fd.fd_cdir);
+		if (Lf->sf)
+		    link_lfile();
+	    }
 	/*
 	 * Save root directory information.
 	 */
-		if (fd.fd_rdir != NULL) {
-			alloc_lfile(RTD, -1);
-			process_node((caddr_t)fd.fd_rdir);
-			if (Lf->sf)
-				link_lfile();
-		}
+	    if (fd.fd_rdir != NULL) {
+		alloc_lfile(RTD, -1);
+		Cfp = (KA_T)NULL;
+		process_node((KA_T)fd.fd_rdir);
+		if (Lf->sf)
+		    link_lfile();
+	    }
 	/*
 	 * Save information on the text file.
 	 */
-		if (p->P_VMSPACE)
-			process_text((KA_T)p->P_TEXTVP, (KA_T)p->P_VMSPACE);
+	    if (p->P_VMSPACE)
+		process_text((KA_T)p->P_TEXTVP, (KA_T)p->P_VMSPACE);
 	/*
 	 * Read open file structure pointers.
 	 */
-		if (fd.fd_ofiles == NULL || (nf = fd.fd_nfiles) <= 0)
-			continue;
-		if (ofb == NULL) {
-			nofb = nf;
-			if ((ofb = (struct file **)malloc((MALLOC_S)
-				    (nofb * sizeof(struct file *))))
-			== NULL) {
-				(void) fprintf(stderr,
-					"%s: PID %d, no file * space\n",
-					Pn, p->P_PID);
-				Exit(1);
-			}
-		} else if (nf > nofb) {
-			nofb = fd.fd_nfiles;
-			if ((ofb = (struct file **)realloc((MALLOC_P *)ofb,
-				    (nofb * sizeof(struct file *))))
-			== NULL) {
-				(void) fprintf(stderr,
-					"%s: PID %d, no realloc file * space\n",
-					Pn, p->P_PID);
-				Exit(1);
-			}
+	    if (!fd.fd_ofiles || (nf = fd.fd_nfiles) <= 0)
+		continue;
+	    if (!ofb) {
+		nofb = nf;
+		if (!(ofb = (struct file **)malloc((MALLOC_S)
+				(nofb * sizeof(struct file *)))))
+		{
+		    (void) fprintf(stderr, "%s: PID %d, no file * space\n",
+			Pn, p->P_PID);
+		    Exit(1);
 		}
-		if (kread((KA_T)fd.fd_ofiles, (char *)ofb,
-		    nf * sizeof(struct file *)))
-			continue;
+	    } else if (nf > nofb) {
+		nofb = fd.fd_nfiles;
+		if (!(ofb = (struct file **)realloc((MALLOC_P *)ofb,
+				(nofb * sizeof(struct file *)))))
+		{
+		    (void) fprintf(stderr,
+			"%s: PID %d, no realloc file * space\n",
+			Pn, p->P_PID);
+		    Exit(1);
+		}
+	    }
+	    if (kread((KA_T)fd.fd_ofiles,(char *)ofb,nf*sizeof(struct file *)))
+		continue;
 	/*
 	 * Save information on file descriptors.
 	 */
-		for (i = 0; i < nf; i++) {
-			if (ofb[i]) {
-				alloc_lfile(NULL, i);
-				process_file(ofb[i]);
-				if (Lf->sf)
-					link_lfile();
-			}
+	    for (i = 0; i < nf; i++) {
+		if (ofb[i]) {
+		    alloc_lfile(NULL, i);
+		    process_file((Cfp = (KA_T)ofb[i]));
+		    if (Lf->sf)
+			link_lfile();
 		}
+	    }
 	/*
 	 * Examine results.
 	 */
-		if (examine_lproc())
-			return;
+	    if (examine_lproc())
+		return;
 	}
 }
 
@@ -265,38 +288,47 @@ static void
 get_kernel_access()
 {
 
+/*
+ * Check kernel version.
+ */
+	(void) ckkv("BSDI", (char *)NULL, LSOF_VSTR, (char *)NULL);
+
 #if	defined(WILLDROPGID)
 /*
  * If kernel memory isn't coming from KMEM, drop setgid permission
  * before attempting to open the (Memory) file.
  */
 	if (Memory)
-		(void) dropgid();
+	    (void) dropgid();
 #else	/* !defined(WILLDROPGID) */
 /*
  * See if the non-KMEM memory and name list files are readable.
  */
 	if ((Memory && !is_readable(Memory, 1))
 	||  (Nmlst && !is_readable(Nmlst, 1)))
-		Exit(1);
+	    Exit(1);
 #endif	/* defined(WILLDROPGID) */
 
 /*
  * Open kernel memory access.
  */
 	if ((Kd = kvm_openfiles(Nmlst, Memory, NULL, O_RDONLY, NULL)) == NULL) {
-		(void) fprintf(stderr,
-			"%s: kvm_openfiles (namelist=%s, core=%s): %s\n",
-			Pn,
-			Nmlst ? Nmlst : "default",
-			Memory  ? Memory  : "default",
-			strerror(errno));
-		Exit(1);
+	    (void) fprintf(stderr,
+		"%s: kvm_openfiles (namelist=%s, core=%s): %s\n",
+		Pn,
+		Nmlst ? Nmlst : "default",
+		Memory  ? Memory  : "default",
+		strerror(errno));
+	    Exit(1);
 	}
+/*
+ * Build and fill kernel namelist table.
+ */
+	(void) build_Nl(Drive_Nl);
 	if (nlist(Nmlst ? Nmlst : N_UNIX, Nl) < 0) {
-		(void) fprintf(stderr, "%s: can't read namelist from %s\n",
-			Pn, Nmlst ? Nmlst : N_UNIX);
-		Exit(1);
+	    (void) fprintf(stderr, "%s: can't read namelist from %s\n",
+		Pn, Nmlst ? Nmlst : N_UNIX);
+	    Exit(1);
 	}
 
 #if	defined(WILLDROPGID)
@@ -304,7 +336,7 @@ get_kernel_access()
  * Drop setgid permission, if necessary.
  */
 	if (!Memory)
-		(void) dropgid();
+	    (void) dropgid();
 #endif	/* defined(WILLDROPGID) */
 
 }
@@ -358,68 +390,49 @@ process_text(tv, vm)
  * Enter the text vnode reference, if there is one.
  */
 	if (tv)
-		(void) enter_vn_text(tv, &n);
+	    (void) enter_vn_text(tv, &n);
 /*
  * Read the vmspace structure for the process.
  */
 	if (kread(vm, (char *)&vmsp, sizeof(vmsp)))
-		return;
+	    return;
 /*
  * Read the vm_map structure.  Search its vm_map_entry structure list.
  */
-	if ( ! vmsp.vm_map.is_main_map)
-		return;
+	if (!vmsp.vm_map.is_main_map)
+	    return;
 	for (i = 0; i < vmsp.vm_map.nentries; i++) {
 
 	/*
 	 * Read the next vm_map_entry.
 	 */
-		if (i == 0)
-			e = &vmsp.vm_map.header;
-		else {
-			if ((ka = (KA_T)e->next) == NULL)
-				return;
-			e = &vmme;
-			if (kread(ka, (char *)e, sizeof(vmme)))
-				return;
-		}
-		if (e->is_a_map || e->is_sub_map)
-			continue;
+	    if (i == 0)
+		e = &vmsp.vm_map.header;
+	    else {
+		if (!(ka = (KA_T)e->next))
+		    return;
+		e = &vmme;
+		if (kread(ka, (char *)e, sizeof(vmme)))
+		    return;
+	    }
+	    if (e->is_a_map || e->is_sub_map)
+		continue;
 	/*
 	 * Read the map entry's object and the object's shadow.
 	 * Look for a PG_VNODE pager handle.
 	 */
-		for (j = 0, ka = (KA_T)e->object.vm_object;
-		     j < 2 && ka;
-		     j++, ka = (KA_T)vmo.shadow)
-		{
-			if (kread(ka, (char *)&vmo, sizeof(vmo)))
-				break;
-			if ((ka = (KA_T)vmo.pager) == NULL
-			||   kread(ka, (char *)&pg, sizeof(pg)))
-				continue;
-			if (pg.pg_handle == NULL || pg.pg_type != PG_VNODE)
-				continue;
-			(void) enter_vn_text((KA_T)pg.pg_handle, &n);
-		}
+	    for (j = 0, ka = (KA_T)e->object.vm_object;
+		 j < 2 && ka;
+		 j++, ka = (KA_T)vmo.shadow)
+	    {
+		if (kread(ka, (char *)&vmo, sizeof(vmo)))
+		    break;
+		if (!(ka = (KA_T)vmo.pager)
+		||  kread(ka, (char *)&pg, sizeof(pg)))
+		    continue;
+		if (!pg.pg_handle || pg.pg_type != PG_VNODE)
+		    continue;
+		(void) enter_vn_text((KA_T)pg.pg_handle, &n);
+	    }
 	}
 }
-
-
-/*
- * The ncache_addr(), ncache_load(), and ncache_lookup() functions are
- * obtained from ../common/rnam.frag.
- */
-
-#if	defined(HASNCACHE)
-#define	NCACHE		namecache	/* kernel's structure name */
-#define	NCACHE_NM	nc_name		/* name in NCACHE */
-#define	NCACHE_NMLEN	nc_nlen		/* name length in NCACHE */
-#define	NCACHE_NODEADDR	nc_vp		/* node address in NCACHE */
-#define	NCACHE_NODEID	nc_vpid		/* node ID in NCACHE */
-#define	NCACHE_NXT	nc_nxt		/* link in NCACHE */
-#define	NCACHE_PARADDR	nc_dvp		/* parent node address in NCACHE */
-#define	NCACHE_PARID	nc_dvpid	/* parent node ID in NCACHE */
-#endif	/* defined(HASNCACHE) */
-
-
