@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.13 98/02/16 19:47:50 abe Exp $
+ * $Id: machine.h,v 1.16 98/06/29 15:41:24 abe Exp $
  */
 
 
@@ -151,6 +151,14 @@
 /* #define	HASINTSIGNAL	1 */
 
 
+/*
+ * HASKERNIDCK is defined for those dialects that support the comparison of
+ * the build to running kernel identity.
+ */
+
+#define	HASKERNIDCK	1
+
+
 /*
  * HASKOPT is defined for those dialects that support the -k option of
  * reading the kernel's name list from an optional file.
@@ -399,6 +407,7 @@
  * header files.
  */
 
+/* #define	USE_LIB_CKKV			1	   ckkv.c */
 /* #define	USE_LIB_COMPLETEVFS		1	   cvfs.c */
 #define	USE_LIB_FIND_CH_INO			1	/* fino.c */
 #define	USE_LIB_IS_FILE_NAMED			1	/* isfn.c */
@@ -408,7 +417,6 @@
 #define	USE_LIB_PRINT_TCPTPI			1	/* ptti.c */
 #define	USE_LIB_READDEV				1	/* rdev.c */
 /* #define	USE_LIB_READMNT			1	   rmnt.c */
-/* #define	USE_LIB_READVFS			1	   rvfs.c */
 
 #if	BSDIV<20100
 #define	USE_LIB_RNAM				1	/* rnam.c */
