@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.7 98/03/06 08:20:00 abe Exp $
+ * $Id: dlsof.h,v 1.8 98/04/29 13:39:20 abe Exp $
  */
 
 
@@ -192,9 +192,9 @@ struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
 #define DEVINCR		1024	/* device table malloc() increment */
 
 # if	FREEBSDV<200
-#define	KA_T		off_t
+typedef	off_t		KA_T;
 # else	/* FREEBSDV>=200 */
-#define	KA_T		u_long
+typedef	u_long		KA_T;
 # endif	/* FREEBSDV<200 */
 
 #define	KMEM		"/dev/kmem"
