@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dmnt.c,v 1.3 98/03/06 08:20:05 abe Exp $";
+static char *rcsid = "$Id: dmnt.c,v 1.6 98/07/17 08:49:40 abe Exp $";
 #endif
 
 
@@ -43,6 +43,8 @@ static char *rcsid = "$Id: dmnt.c,v 1.3 98/03/06 08:20:05 abe Exp $";
  * Local static information
  */
 
+static struct mounts *Lmi = (struct mounts *)NULL;	/* local mount info */
+static int Lmist = 0;					/* Lmi status */
 static char *mnt_names[] =
 
 #if	FREEBSDV<200
@@ -56,11 +58,10 @@ static char *mnt_names[] =
  * readmnt() - read mount table
  */
 
-int
+struct mounts *
 readmnt()
 {
 	char *dn = (char *)NULL;
-	int err = 0;
 	char *ln;
 	struct statfs *mb = (struct statfs *)NULL;
 	struct mounts *mtp;
@@ -71,6 +72,8 @@ readmnt()
 	unsigned char procfs = 0;
 #endif	/* defined(HASPROCFS) */
 
+	if (Lmi || Lmist)
+	    return(Lmi);
 /*
  * Access mount information.
  */
@@ -90,15 +93,21 @@ readmnt()
 	    if (dn)
 		(void) free((FREE_P *)dn);
 	    if (!(dn = mkstrcpy(mb->f_mntonname, (MALLOC_S *)NULL))) {
-		err = 1;
-		break;
+
+no_space_for_mount:
+
+		(void) fprintf(stderr, "%s: no space for mount at ", Pn);
+		safestrprt(mb->f_mntonname, stderr, 0);
+		(void) fprintf(stderr, " (");
+		safestrprt(mb->f_mntfromname, stderr, 0);
+		(void) fprintf(stderr, ")\n");
+		Exit(1);
 	    }
 	    if (!(ln = Readlink(dn))) {
 		if (!Fwarn) {
 		    (void) fprintf(stderr,
 			"      Output information may be incomplete.\n");
 		}
-		err = 2;
 		continue;
 	    }
 	    if (ln != dn) {
@@ -117,7 +126,6 @@ readmnt()
 		    (void) fprintf(stderr,
 			"      Output information may be incomplete.\n");
 		}
-		err = 2;
 		(void) bzero((char *)&sb, sizeof(sb));
 		sb.st_dev = (dev_t)mb->f_fsid.val[0];
 		sb.st_mode = S_IFDIR | 0777;
@@ -130,10 +138,8 @@ readmnt()
 	/*
 	 * Allocate and fill a local mount structure.
 	 */
-	    if (!(mtp = (struct mounts *)malloc(sizeof(struct mounts)))) {
-		err = 1;
-		break;
-	    }
+	    if (!(mtp = (struct mounts *)malloc(sizeof(struct mounts))))
+		goto no_space_for_mount;
 	    mtp->dir = dn;
 	    dn = (char *)NULL;
 
@@ -152,7 +158,7 @@ readmnt()
 	    }
 #endif	/* defined(HASPROCFS) */
 
-	    mtp->next = Mtab;
+	    mtp->next = Lmi;
 	    mtp->dev = sb.st_dev;
 	    mtp->rdev = sb.st_rdev;
 	    mtp->inode = sb.st_ino;
@@ -160,10 +166,8 @@ readmnt()
 	/*
 	 * Interpolate a possible file system (mounted-on) device name link.
 	 */
-	    if (!(dn = mkstrcpy(mb->f_mntfromname, (MALLOC_S *)NULL))) {
-		err = 1;
-		break;
-	    }
+	    if (!(dn = mkstrcpy(mb->f_mntfromname, (MALLOC_S *)NULL)))
+		goto no_space_for_mount;
 	    mtp->fsname = dn;
 	    ln = Readlink(dn);
 	    dn = (char *)NULL;
@@ -175,25 +179,15 @@ readmnt()
 		sb.st_mode = 0;
 	    mtp->fsnmres = ln;
 	    mtp->fs_mode = sb.st_mode;
-	    Mtab = mtp;
+	    Lmi = mtp;
 	}
 /*
- * Clean up and handle errors.
+ * Clean up and return the local mount info table address.
  */
 	if (dn)
 	    (void) free((FREE_P *)dn);
-	switch(err) {
-	case 1:
-	    (void) fprintf(stderr, "%s: no space for mount at ", Pn);
-	    safestrprt(mb->f_mntonname, stderr, 0);
-	    (void) fprintf(stderr, " (");
-	    safestrprt(mb->f_mntfromname, stderr, 0);
-	    (void) fprintf(stderr, ")\n");
-	    return(0);
-	case 2:
-	    return(1);
-	}
-	return(1);
+	Lmist = 1;
+	return(Lmi);
 }
 
 
