@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.4 98/03/06 08:20:08 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.5 98/06/25 07:17:33 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -306,6 +306,10 @@ static void
 get_kernel_access()
 {
 
+/*
+ * Check kernel version.
+ */
+	(void) ckkv("FreeBSD", LSOF_VSTR, (char *)NULL, (char *)NULL);
 /*
  * Set name list file path.
  */
