@@ -33,9 +33,21 @@
 
 
 /*
- * $Id: dproto.h,v 1.5 96/01/09 08:18:14 abe Exp $
+ * $Id: dproto.h,v 1.4 98/03/06 08:20:10 abe Exp $
  */
 
 
+#if	!defined(N_UNIXV)
+_PROTOTYPE(extern char *get_nlist_path,(int ap));
+#endif	/* !defined(N_UNIXV) */
+
 _PROTOTYPE(extern int is_file_named,(char *p, enum vtype ty));
-_PROTOTYPE(extern struct l_vfs *readvfs,(struct mount *vm));
+_PROTOTYPE(extern struct l_vfs *readvfs,(KA_T vm));
+
+#if	FREEBSDV>=220
+_PROTOTYPE(extern void process_pipe,(KA_T pa));
+#endif	/* FREEBSDV>=220 */
+
+#if	defined(HAS9660FS)
+_PROTOTYPE(extern int read_iso_node,(struct vnode *v, dev_t *d, unsigned long *ino, unsigned long *sz));
+#endif	/* defined(HAS9660FS) */
