@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.5 98/03/06 08:20:11 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.7 98/08/03 07:41:45 abe Exp $";
 #endif
 
 
@@ -48,16 +48,14 @@ process_socket(sa)
 	KA_T sa;			/* socket address in kernel */
 {
 	struct domain d;
-	struct in_addr *fa = (struct in_addr *)NULL;
+	unsigned char *fa = (unsigned char *)NULL;
 	int fam;
 	int fp, lp;
 	struct inpcb inp;
-	struct in_addr *la = (struct in_addr *)NULL;
+	unsigned char *la = (unsigned char *)NULL;
 	struct mbuf mb;
 	struct protosw p;
-	struct rawcb raw;
 	struct socket s;
-	struct sockaddr sad;
 	struct tcpcb t;
 	struct unpcb uc, unp;
 	struct sockaddr_un *ua = NULL;
@@ -83,12 +81,14 @@ process_socket(sa)
 	    return;
 	}
 	if (!s.so_proto
-	||  kread((KA_T) s.so_proto, (char *) &p, sizeof(p))) {
-	    (void) strcpy(Namech, "no protocol switch");
+	||  kread((KA_T)s.so_proto, (char *)&p, sizeof(p))) {
+	    (void) sprintf(Namech, "can't read protocol switch from %s",
+		print_kptr((KA_T)s.so_proto, (char *)NULL));
 	    enter_nm(Namech);
 	    return;
 	}
-	if (kread((KA_T)p.pr_domain, (char *) &d, sizeof(d))) {
+	if (!p.pr_domain
+	||  kread((KA_T)p.pr_domain, (char *)&d, sizeof(d))) {
 	    (void) sprintf(Namech, "can't read domain struct from %s",
 		print_kptr((KA_T)p.pr_domain, (char *)NULL));
 	    enter_nm(Namech);
@@ -122,81 +122,99 @@ process_socket(sa)
  * Process an Internet domain socket.
  */
 	case AF_INET:
-	case AF_ROUTE:
+
+#if	defined(HASIPv6)
+	case AF_INET6:
+#endif	/* defined(HASIPv6) */
+
 	    if (Fnet)
 		Lf->sf |= SELNET;
-	    (void) strcpy(Lf->type, (fam == AF_INET) ? "inet" : "rte");
 	    printiproto(p.pr_protocol);
+
+#if	defined(HASIPv6)
+	    (void) strcpy(Lf->type, (fam == AF_INET) ? "IPv4" : "IPv6");
+#else	/* !defined(HASIPv6) */
+	    (void) strcpy(Lf->type, "inet");
+#endif	/* defined(HASIPv6) */
+
 	/*
 	 * Read protocol control block.
 	 */
-	    if (!s.so_pcb) {
-		enter_nm("no protocol control block");
+	    if (!s.so_pcb
+	    ||  kread((KA_T) s.so_pcb, (char *) &inp, sizeof(inp))) {
+		(void) sprintf(Namech, "can't read inpcb at %s",
+		    print_kptr((KA_T)s.so_pcb, (char *)NULL));
+		enter_nm(Namech);
 		return;
 	    }
-	    if (s.so_type == SOCK_RAW) {
+	/*
+	 * Print Internet socket information.
+	 */
+	    enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
+							: s.so_pcb),
+				    (char *)NULL));
 
-	    /*
-	     * Print raw socket information.
-	     */
-		if (kread((KA_T) s.so_pcb, (char *)&raw, sizeof(raw))) {
-		    (void) sprintf(Namech, "can't read rawcb at %s",
-			print_kptr((KA_T)s.so_pcb, (char *)NULL));
-		    enter_nm(Namech);
-		    return;
+#if	defined(HASIPv6) && defined(HASINRIAIPv6)
+	    if (inp.inp_latype == IPATYPE_UNBD)
+		la = (unsigned char *)&in6addr_any;
+	    else if (inp.inp_latype == IPATYPE_IPV6) {
+		fam = AF_INET6;
+		Lf->type[3] = '6';
+		la = (unsigned char *)&inp.inp_laddr6;
+	    } else {
+		if (inp.inp_latype == IPATYPE_IPV4) {
+		    fam = AF_INET;
+		    Lf->type[3] = '4';
 		}
-		enter_dev_ch(print_kptr((KA_T)s.so_pcb, (char *)NULL));
-		if (raw.rcb_laddr
-		 && !kread((KA_T)raw.rcb_laddr, (char *)&sad, sizeof(sad))) {
-		    if (sad.sa_family == AF_INET || sad.sa_family == AF_ROUTE)
-			la = (struct in_addr *)&sad.sa_data[2];
-		    else if (sad.sa_family)
-			printrawaddr(&sad);
-		}
-		if (raw.rcb_faddr
-		&& !kread((KA_T)raw.rcb_faddr, (char *)&sad, sizeof(sad))) {
-		    if (sad.sa_family == AF_INET || sad.sa_family == AF_ROUTE)
-			fa = (struct in_addr *)&sad.sa_data[2];
-		    else if (sad.sa_family) {
-			(void) strcat(endnm(), "->");
-			printrawaddr(&sad);
+		la = (unsigned char *)&inp.inp_laddr;
+	    }
+#else	/* !defined(HASIPv6) || !defined(HASINRIAIPv6) */
+	    la = (unsigned char *)&inp.inp_laddr;
+#endif	/* defined(HASIPv6) && defined(HASINRIAIPv6) */
+
+	    lp = (int)ntohs(inp.inp_lport);
+
+#if	defined(HASIPv6) && defined(HASINRIAIPv6)
+	    if (inp.inp_fatype != IPATYPE_UNBD || inp.inp_fport) {
+		if (inp.inp_fatype == IPATYPE_IPV6) {
+		    fa = (unsigned char *)&inp.inp_faddr6;
+		    fam = AF_INET6;
+		    Lf->type[3] = '6';
+		} else {
+		    fa = (unsigned char *)&inp.inp_faddr;
+		    if (inp.inp_fatype == IPATYPE_IPV4) {
+			fam = AF_INET;
+			Lf->type[3] = '4';
 		    }
 		}
-		if (fa || la)
-		    (void) ent_inaddr(la, -1, fa, -1);
-	    } else {
-		if (fam != AF_INET) {
-		    (void) strcpy(Namech, "no further information");
-		    break;
-		}
-
-	    /*
-	     * Print Internet socket information.
-	     */
-		if (kread((KA_T) s.so_pcb, (char *) &inp, sizeof(inp))
-		||  (struct socket *)sa != inp.inp_socket) {
-		    (void) sprintf(Namech, "can't read inpcb at %s",
-			print_kptr((KA_T)s.so_pcb, (char *)NULL));
-		    enter_nm(Namech);
-		    return;
-		}
-		enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
-							    : s.so_pcb),
-					(char *)NULL));
-		la = &inp.inp_laddr;
-		lp = (int)ntohs(inp.inp_lport);
-		if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport != 0) {
-		    fa = &inp.inp_faddr;
-		    fp = (int)ntohs(inp.inp_fport);
-		}
-		if (fa || la)
-		    (void) ent_inaddr(la, lp, fa, fp);
-		if (p.pr_protocol == IPPROTO_TCP && inp.inp_ppcb
-		&&  kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t)) == 0) {
-		    Lf->lts.type = 0;
-		    Lf->lts.state.i = (int)t.t_state;
-		}
+		fp = (int)ntohs(inp.inp_fport);
 	    }
+#else	/* !defined(HASIPv6) || !defined(HASINRIAIPv6) */
+	    if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport) {
+		fa = (unsigned char *)&inp.inp_faddr;
+		fp = (int)ntohs(inp.inp_fport);
+	    }
+#endif	/* defined(HASIPv6) && defined(HASINRIAIPv6) */
+
+	    if (fa || la)
+		(void) ent_inaddr(la, lp, fa, fp, fam);
+	    if (p.pr_protocol == IPPROTO_TCP && inp.inp_ppcb
+	    &&  !kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t))) {
+		Lf->lts.type = 0;
+		Lf->lts.state.i = (int)t.t_state;
+	    }
+	    break;
+/*
+ * Process a ROUTE domain socket.
+ */
+	case AF_ROUTE:
+	    (void) strcpy(Lf->type, "rte");
+	    if (s.so_pcb)
+		enter_dev_ch(print_kptr((KA_T)(s.so_pcb), (char *)NULL));
+	    else
+		(void) strcpy(Namech, "no protocol control block");
+	    if (!Fsize)
+		Lf->off_def = 1;
 	    break;
 /*
  * Process a Unix domain socket.
