@@ -31,13 +31,17 @@
 
 
 /*
- * $Id: dlsof.h,v 1.12 96/04/26 08:08:53 abe Exp $
+ * $Id: dlsof.h,v 1.10 98/08/18 08:08:36 abe Exp $
  */
 
 
 #if	!defined(HPUX_LSOF_H)
 #define	HPUX_LSOF_H	1
 
+# if	HPUXV>=1030
+#include <fcntl.h>
+# endif	/* HPUXV>=1030 */
+
 #include <stdlib.h>
 #include <dirent.h>
 #include <mntent.h>
@@ -45,86 +49,133 @@
 #include <string.h>
 #include <nlist.h>
 #include <unistd.h>
+
+# if	HPUXV<1020
 #include <sys/vnode.h>
+# endif	/* HPUXV<1020 */
+
+# if	HPUXV>=1030
+/*
+ * Include header files for HP-UX 10.30 and up that have been
+ * manufactured with q4 and hand editing.
+ */
+
+#include "lla.h"
+#include "proc.h"
+#include "rnode.h"
+#include "nfs_clnt.h"
+#include "vnode.h"
+# endif	/* HPUXV>=1030 */
+
 #include <sys/domain.h>
+
+# if	HPUXV>=1020
+#define	_INCLUDE_STRUCT_FILE
+# endif	/* HPUXV>=1020 */
+
+# if	HPUXV>=1030
+struct uio {		/* to satisfy function prototypes in <sys/file.h> */
+	int dummy;
+};
+# endif	/* HPUXV>=1030 */
+
 #include <sys/file.h>
+
+# if	HPUXV>=1020
+#undef	_INCLUDE_STRUCT_FILE
+# endif	/* HPUXV>=1020 */
+
+# if	HPUXV>=1030
+#include <sys/stream.h>
+#include "sth.h"
+# endif	/* HPUXV>=1030 */
+
 #include <sys/mbuf.h>
 
-# if	_HPUXV>=800
+# if	HPUXV>=800
 #undef	_PROTOTYPES
 #include <sys/pstat.h>
-# endif	/* _HPUXV>=800 */
+# endif	/* HPUXV>=800 */
 
 #include <sys/resource.h>
 
-# if	_HPUXV<1010
+# if	HPUXV<1010
 #include <sys/proc.h>
-# else	/* _HPUXV>=1010 */
-/*
- * HP doesn't include a definition for the proc structure in HP-UX 10.10
- * or above in an attempt to force use of pstat(2).  Unfortunately, pstat(2)
- * doesn't return the information lsof needs.  Hence, this private proc
- * structure definition.
- */
-
-#include <sys/vas.h>
-
-#define	SZOMB	3
-
-struct proc {
-	caddr_t d1[7];			/* dummy to occupy space */
-	int p_stat;			/* process status */
-	caddr_t d2[9];			/* dummy to occupy space */
-	uid_t p_uid;			/* UID */
-	caddr_t d3[2];			/* dummy to occupy space */
-	gid_t p_pgrp;			/* process group ID */
-	pid_t p_pid;			/* PID */
-	pid_t p_ppid;			/* parent PID */
-	caddr_t d4[9];			/* dummy to occupy space */
-	vas_t *p_vas;			/* virtual address space */
-	caddr_t d5[16];			/* dummy to occupy space */
-	int p_maxof;			/* max open files allowed */
-	struct vnode *p_cdir;		/* current directory */
-	struct vnode *p_rdir;		/* root directory */
-	struct ofile_t **p_ofilep;     /* file descriptor chunks */
-	caddr_t d6[43];			/* dummy to occupy space */
-};
-# endif	/* _HPUXV<1010 */
+# endif	/* HPUXV<1010 */
 
 #include <sys/protosw.h>
 #include <sys/socket.h>
 #include <sys/socketvar.h>
 #include <netinet/in.h>
 #include <net/route.h>
+
+# if	HPUXV<1030
 #include <net/raw_cb.h>
 #include <netinet/in_pcb.h>
-#include <rpc/types.h>
+# endif	/* HPUXV<1030 */
 
-# if	_HPUXV>=1000
+#include <netinet/ip_var.h>
+#include <netinet/tcp.h>
+#include <netinet/tcpip.h>
+
+# if	HPUXV<1030
+#include <netinet/tcp_fsm.h>
+#include <netinet/tcp_timer.h>
+#include <netinet/tcp_var.h>
+# else	/* HPUXV>=1030 */
+#include <sys/tihdr.h>
+/*
+ * Include header files for HP-UX 10.30 and up that have been
+ * manufactured with q4 and hand editing.
+ */
+
+#include "ipc_s.h"
+#include "tcp_s.h"
+#include "udp_s.h"
+# endif	/* HPUXV<1030 */
+
+# if	HPUXV>=1030
+#undef	TCP_NODELAY
+#undef	TCP_MAXSEG
+# endif	/* HPUXV>=1030 */
+
+#include <rpc/types.h>
+#include <rpc/rpc.h>
+#include <rpc/pmap_prot.h>
+
+# if	HPUXV>=1030
+#include <rpc/clnt_soc.h>
+# endif	/* HPUXV>=1030 */
+
+# if	HPUXV>=1000
 #include <rpc/xdr.h>
 #include <rpc/auth.h>
 #include <rpc/clnt.h>
 #include <sys/cdfsdir.h>
 #include <sys/cdfs.h>
 #include <sys/cdnode.h>
-# endif	/* _HPUXV>=1000 */
+# endif	/* HPUXV>=1000 */
 
 #include <nfs/nfs.h>
+
+# if	HPUXV<1030
 #include <nfs/nfs_clnt.h>
 #include <nfs/rnode.h>
+# endif	/* HPUXV<1030 */
+
 #include <nfs/snode.h>
 
-# if	_HPUXV>=1000
+# if	HPUXV>=1000
 #define	_KERNEL
 #include <nfs/fifonode.h>
 #undef	_KERNEL
-# endif	/* _HPUXV>=1000 */
+# endif	/* HPUXV>=1000 */
 
-# if	defined(DTYPE_LLA)
+# if	defined(DTYPE_LLA) && HPUXV<1030
 #define	_KERNEL	1
 #include <sio/lla.h>
 #undef	_KERNEL
-# endif
+# endif	/* defined(DTYPE_LLA) && HPUXV<1030 */
 
 #include <sys/un.h>
 #include <sys/unpcb.h>
@@ -133,21 +184,15 @@ struct proc {
 #include <sys/user.h>
 
 /*
- * This is from <sys/mount.h>, defined under the _KERNEL symbol.
- * Unfortunately, defining _KERNEL causes <sys/mount.h> to include
- * other header files not in <sys>.
+ * The hpux_mount.h header file is manufactured from <sys/mount.h> by the
+ * Configure script to get the mount structure without needing to define
+ * _KERNEL when including <sys/mount.h>.  Defining _KERNEL causes unresolvable
+ * header file complications.
  */
 
-struct mount {
-	struct	mount *m_hforw;  /* forward hash pointer */
-	struct	mount *m_hback;  /* backward hash pointer */
-	struct	mount *m_rhforw; /* forward hash pointer for real device */
-	struct	mount *m_rhback; /* backward hash pointer for real device */
-	struct vfs	*m_vfsp; /* vfs structure for this filesystem */
-	dev_t	m_dev;		 /* device mounted */
-   };
+#include "hpux_mount.h"
 
-# if	_HPUXV>=800
+# if	HPUXV>=800
 /*
  * These definitions are from <sys/vfs.h>, defined under the _KERNEL symbol.
  * Unfortunately, defining _KERNEL causes <sys/vfs.h> to include other
@@ -156,7 +201,7 @@ struct mount {
 #define MOUNT_UFS 0
 #define MOUNT_NFS 1
 #define MOUNT_CDFS 2
-# endif	/* _HPUXV>=800 */
+# endif	/* HPUXV>=800 */
 
 # if	defined(HAS_CONST)
 #define	COMP_P		const void
@@ -164,46 +209,64 @@ struct mount {
 #define	COMP_P		void
 # endif	/* defined(HAS_CONST) */
 
-# if	_HPUXV>=800
+# if	HPUXV>=800
 #define	CURDIR	p->p_cdir
-#define	FILEPTR	oft.ofile[j-1]
 #define	ROOTDIR	p->p_rdir
-# else	/* _HPUXV<800 */
+# else	/* HPUXV<800 */
 #define CURDIR	u->u_cdir
-#define	FILEPTR	u->u_ofile.ofile[j-1]
 #define	ROOTDIR	u->u_rdir
-# endif	/* _HPUXV>=800 */
+# endif	/* HPUXV>=800 */
 
 #define DEVINCR		1024	/* device table malloc() increment */
-#define	KA_T		off_t
+
+# if	HPUXV<1030
+/*
+ * KA_T is defined in dialects/hpux/hpux11/kernbits.h for HP-UX 10.30
+ * and above.
+ */
+typedef	off_t		KA_T;
+# endif	/* HPUXV<1030 */
+
 #define	KMEM		"/dev/kmem"
 #define MALLOC_P	void
 #define FREE_P		void
 #define MALLOC_S	unsigned
 #define MOUNTED		MNT_MNTTAB
 
-# if	_HPUXV<1000
+# if	HPUXV<1000
 #define N_UNIX		"/hp-ux"
-# else	/* _HPUXV>=1000 */
+# else	/* HPUXV>=1000 */
 #define N_UNIX		"/stand/vmunix"
-# endif	/* _HPUXV<1000 */
+# endif	/* HPUXV<1000 */
 
 #define QSORT_P		void
 #define	READLEN_T	int
 #define STRNCPY_L	size_t
+
+# if	HPUXV>=1000
+#define	SZOFFTYPE	unsigned long long
+				/* type for size and offset */
+#define	SZOFFPSPEC	"ll"	/* SZOFFTYPE printf specification modifier */
+# endif	/* HPUXV>=1000 */
+
 #define SWAP		"/dev/swap"
 
-# if	_HPUXV<800
+# if	HPUXV<800
 #define unp_addr	unp_locaddr
 /*
  * HP-UX <8 SWAP must be read in DEV_BSIZE chunks.
  */
 #define U_SIZE		(((DEV_BSIZE+sizeof(struct user))/DEV_BSIZE)*DEV_BSIZE)
-# endif	/* _HPUXV<800 */
+# endif	/* HPUXV<800 */
 
-# if	_HPUXV>=800
+# if	HPUXV>=800
 #define	U_SIZE		sizeof(struct user)
-# endif	/* _HPUXV>=800 */
+# endif	/* HPUXV>=800 */
+
+# if	HPUXV>=1030
+#define	XDR_PMAPLIST	(xdrproc_t)xdr_pmaplist
+#define	XDR_VOID	(xdrproc_t)xdr_void
+# endif	/* HPUXV>=1030 */
 
 
 # if	defined(HAS_AFS)
@@ -227,10 +290,15 @@ extern struct vfs *AFSVfsp;		/* AFS struct vfs kernel pointer */
  * Global storage definitions (including their structure definitions)
  */
 
-extern int Kmem;
+extern int Kd;
+extern KA_T Kpa;
+
+# if	HPUXV>=1010
+extern KA_T Ktp;
+#endif	/* HPUXV>=1010 */
 
 struct l_vfs {
-	struct vfs *addr;		/* kernel address */
+	KA_T addr;			/* kernel address */
 	dev_t dev;			/* device */
 	char *dir;			/* mounted directory */
 	char *fsname;			/* file system name */
@@ -243,75 +311,32 @@ struct l_vfs {
 };
 extern struct l_vfs *Lvfs;
 
-# if	_HPUXV<800
+# if	HPUXV<800
 extern int Mem;
-# endif	/* _HPUXV<800 */
+# endif	/* HPUXV<800 */
 
 struct mounts {
-	dev_t dev;              	/* st_dev */
-	char *dir;              	/* directory */
-	char *fsname;           	/* file system */
-	ino_t inode;			/* st_ino */
-	mode_t mode;			/* st_mode */
+	char *dir;              	/* directory (mounted on) */
+	char *fsname;           	/* file system
+					 * (symbolic links unresolved) */
+	char *fsnmres;           	/* file system
+					 * (symbolic links resolved) */
+	dev_t dev;              	/* directory st_dev */
+	dev_t rdev;			/* directory st_rdev */
+	ino_t inode;			/* directory st_ino */
+	mode_t mode;			/* directory st_mode */
+	mode_t fs_mode;			/* file system st_mode */
 	struct mounts *next;    	/* forward link */
-	dev_t rdev;			/* st_rdev */
 };
 
-# if	defined(hp9000s300)
-#define	X_AFS_FID	0
-#define	X_AFS_OPS	1
-#define	X_AFS_VOL	2
-#define	X_NCACHE	3
-#define	X_NCSIZE	4
-#define X_PROC          5
-#define X_NFS_OPS	6
-#define X_NPROC         7
-#define X_UFS_OPS	8
-#define X_VN_FOPS	9
+#define	X_NCACHE	"ncache"
+#define	X_NCSIZE	"ncsize"
+#define	NL_NAME		n_name
 
-#  if	_HPUXV<800
-#define X_USRPTMAP	10
-#define X_USRPT		11
-#define	X_LAST		12
-#  else	/* _HPUXV>=800 */
-#define	X_LAST		10
-#  endif	/* _HPUXV<800 */
-# endif	/* defined(hp9000s300) */
-
-# if	defined(hp9000s800)
-#define	X_AFS_FID	0
-#define	X_AFS_OPS	1
-#define	X_AFS_VOL	2
-#define	X_NCACHE	3
-#define	X_NCSIZE	4
-#define X_PROC          5
-#define X_NFS_OPS	6
-#define X_NPROC         7
-#define X_UFS_OPS	8
-#define X_VN_FOPS	9
-
-#  if	_HPUXV<800
-#define X_UBASE		10
-#define X_NPIDS		11
-#define	X_LAST		12
-#  else	/* _HPUXV>=800 */
-#   if	_HPUXV>=1000
-#define	X_CDFS_OPS	10
-#define	X_FIFO_OPS	11
-#define	X_PIPE_OPS	12
-#define	X_SPEC_OPS	13
-#define X_VXFS_OPS	14
-#define	X_LAST		15
-#   else	/* _HPUXV<1000 */
-#define	X_LAST		10
-#   endif	/* _HPUXV>=1000 */
-#  endif	/* _HPUXV<800 */
-# endif	defined(hp9000s800)
-
-# if	_HPUXV<800 && defined(hp9000s800)
+# if	HPUXV<800 && defined(hp9000s800)
 extern int npids;
 extern struct proc *proc;
-# endif	/* _HPUXV<800 && defined(hp9000s800) */
+# endif	/* HPUXV<800 && defined(hp9000s800) */
 
 struct sfile {
 	char *aname;			/* argument file name */
@@ -326,19 +351,47 @@ struct sfile {
 	struct sfile *next;		/* forward link */
 };
 
-# if	_HPUXV<800
+# if	HPUXV<800
 extern int Swap;
-# endif	/* _HPUXV<800 */
+# endif	/* HPUXV<800 */
 
-# if	_HPUXV<800 && defined(hp9000s800)
+# if	HPUXV<800 && defined(hp9000s800)
 extern struct user *ubase;
-# endif	/* _HPUXV<800 && defined(hp9000s800) */
+# endif	/* HPUXV<800 && defined(hp9000s800) */
 
-# if	_HPUXV<800 && defined(hp9000s300)
+# if	HPUXV<800 && defined(hp9000s300)
 extern struct pte *Usrptmap;
 extern struct pte *usrpt;
-# endif	/* _HPUXV<800 && defined(hp9000s300) */
+# endif	/* HPUXV<800 && defined(hp9000s300) */
 
-extern struct fileops *Vnfops;
+extern KA_T Vnfops;
+
+
+/*
+ * Definitions for rdev.c
+ */
+
+#define	DIRTYPE	dirent
+#define HASDNAMLEN	1	/* struct DIRTYPE has d_namlen element */
+
+
+/*
+ * Definition for rmnt.c
+ */
+
+#define MNTSKIP \
+	{ if (strcmp(mp->mnt_type, MNTTYPE_IGNORE) == 0) \
+		continue; }
+
+/*
+ * Definitions for rnch.c
+ */
+
+# if     defined(HASNCACHE)
+#include <sys/dnlc.h>
+#  if	HPUXV<1000
+#define	ADDR_NCACHE	1
+#  endif	/* HPUXV<1000 */
+# endif  /* defined(HASNCACHE) */
 
 #endif	/* HPUX_LSOF_H */
