@@ -32,9 +32,15 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.3 98/03/06 08:21:26 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.6 98/08/18 08:08:44 abe Exp $";
 #endif
 
+#if	defined(HPUXKERNBITS) && HPUXKERNBITS>=64
+#define _INO_T
+typedef int ino_t;
+#define _TIME_T
+typedef int time_t;
+#endif	/* defined(HPUXKERNBITS) && HPUXKERNBITS>=64 */
 
 #include "lsof.h"
 #include <sys/inode.h>
@@ -98,7 +104,19 @@ islocked(lp)
 # endif	/* HPUXV<1010 */
 
 		continue;
-	    if (ll.ll_start == 0 && ll.ll_end == 0x7fffffff)
+	    if (ll.ll_start == 0
+
+# if	HPUXV<1030
+		&& ll.ll_end == 0x7fffffff
+# else	/* HPUXV>=1030 */
+#  if	defined(HPUXKERNBITS) && HPUXKERNBITS>=64
+		&& (ll.ll_end == 0x80000000 || ll.ll_end == 0x10000000000)
+#  else	/* !defined(HPUXKERNBITS) || HPUXKERNBITS<64 */
+		&& ll.ll_end == 0x80000000
+#  endif	/* defined(HPUXKERNBITS) && HPUXKERNBITS>=64 */
+# endif	/* HPUXV<1030 */
+
+	    )
 		l = 1;
 	    else
 		l = 0;
@@ -141,6 +159,7 @@ getnodety(v)
 	static KA_T cvops;
 	static KA_T fvops;
 	static KA_T nvops;
+	static KA_T nv3ops;
 	static KA_T pvops;
 	static KA_T svops;
 	static KA_T uvops;
@@ -157,6 +176,8 @@ getnodety(v)
 		fvops = (unsigned long)0;
 	    if (get_Nl_value("nvops", Drive_Nl, &nvops) < 0)
 		nvops = (unsigned long)0;
+	    if (get_Nl_value("nv3ops", Drive_Nl, &nv3ops) < 0)
+		nv3ops = (unsigned long)0;
 	    if (get_Nl_value("pvops", Drive_Nl, &pvops) < 0)
 		pvops = (unsigned long)0;
 	    if (get_Nl_value("svops", Drive_Nl, &svops) < 0)
@@ -174,6 +195,8 @@ getnodety(v)
 	    return(N_REGLR);
 	else if (nvops && nvops == (unsigned long)v->v_op)
 	    return(N_NFS);
+	else if (nv3ops && nv3ops == (unsigned long)v->v_op)
+	    return(N_NFS);
 
 #if	defined(HASVXFS)
 	else if (vvops && vvops == (unsigned long)v->v_op)
@@ -252,6 +275,7 @@ unknown_v_op:
 void
 process_node(va)
 	KA_T va;			/* vnode kernel space address */
+
 {
 
 #if	defined(HAS_AFS)
@@ -259,6 +283,7 @@ process_node(va)
 #endif	/* defined(HAS_AFS) */
 
 	dev_t dev;
+	int devs = 1;
 	struct inode i;
 	int ins = 0;
 	struct rnode r;
@@ -450,6 +475,8 @@ process_node(va)
 		    enter_nm(Namech);
 		    return;
 		}
+
+#if	defined(HASVXFS)
 		if (getnodety(&rv) == N_VXFS) {
 		    if (!rv.v_data || read_vxnode(&rv, vfs, &dev)) {
 			(void) sprintf(Namech,
@@ -462,6 +489,8 @@ process_node(va)
 		    Ntype = N_VXFS;
 		    break;
 		}
+#endif	/* defined(HASVXFS) */
+
 		if (!rv.v_data || readinode((KA_T)rv.v_data, &i)) {
 		    (void) sprintf(Namech,
 			"snode at %s: can't read inode (%s)",
@@ -551,22 +580,26 @@ process_node(va)
 #if	defined(HAS_AFS)
 	case N_AFS:
 	    dev = an.dev;
+	    devs = 1;
 	    break;
 #endif	/* defined(HAS_AFS) */
 
 	case N_NFS:
 	    dev = vfs ? vfs->dev : 0;
+	    devs = 1;
 	    break;
 
 #if	HPUXV>=1000
 	case N_CDFS:
 	    dev = c.cd_dev;
+	    devs = 1;
 	    break;
 	case N_FIFO:
 	case N_PIPE:
-	    if (vfs && vfs->fsname)
+	    if (vfs && vfs->fsname) {
 		dev = vfs->dev;
-	    else
+		devs = 1;
+	    } else
 		enter_dev_ch(print_kptr(va, (char *)NULL));
 	    break;
 #endif	/* _HPUX>=1000 */
@@ -581,19 +614,27 @@ process_node(va)
 	default:
 
 #if	HPUXV>=800
-	    if (v->v_type == VCHR || v->v_type == N_BLK)
+	    if (v->v_type == VCHR || v->v_type == N_BLK) {
 		dev = v->v_rdev;
-	    else if (vfs && vfs->fsname)
+		devs = 1;
+	    } else if (vfs && vfs->fsname) {
 		dev = vfs->dev;
-	    else
+		devs = 1;
+	    } else if (ins) {
 		dev = i.i_dev;
+		devs = 1;
+	    }
 #else	/* HPUXV<800 */
-	    dev = (v->v_type == VCHR || v->v_type == VBLK) ? v->v_rdev
-							   : i.i_dev;
+	    if (v->v_type == VCHR || v->v_type == VBLK) {
+		dev = v->v_rdev;
+		devs = 1;
+	    } else if (ins) {
+		dev = i.i_dev;
+		devs = 1;
+	    }
 #endif	/* HPUXV>=800 */
 
 	}
-
 	type = v->v_type;
 /*
  * Obtain the inode number.
@@ -664,6 +705,27 @@ process_node(va)
 		Lf->inp_ty = 1;
 	    }
 	}
+
+#if	HPUXV>=1030
+/*
+ * Check for an HP-UX 10.30 and above stream.
+ */
+	if (v->v_stream) {
+	    KA_T ip, pcb;
+	    char *pn;
+
+	    Lf->dev = dev;
+	    Lf->dev_def = devs;
+	    if (read_mi((KA_T)v->v_stream, &ip, &pcb, &pn))
+		return;
+	    if (ip && pcb) {
+		process_stream_sock(ip, pcb, pn, type);
+		return;
+	    }
+	    Lf->is_stream = 1;
+	}
+#endif	/* HPUXV>=1030 */
+
 /*
  * Obtain the file size.
  */
@@ -698,11 +760,11 @@ process_node(va)
 
 # if	HPUXV<1000
 		if (ins) {
-			    rp = i.i_frptr;
-			    sz = (int)i.i_fifosize;
-			    wp = i.i_fwptr;
+		    rp = i.i_frptr;
+		    sz = (int)i.i_fifosize;
+		    wp = i.i_fwptr;
 		} else if (rns)
-			    Lf->sz = (SZOFFTYPE)r.r_nfsattr.na_size;
+		    Lf->sz = (SZOFFTYPE)r.r_nfsattr.na_size;
 # else	/* HPUXV>=1000 */
 		if (fns) {
 		    rp = f.fn_rptr;
@@ -753,11 +815,12 @@ process_node(va)
 
 	    case N_SPEC:
 	    case N_REGLR:
-		if (type == VREG || type == VDIR) {
+		if ((type == VCHR || type == VBLK) && !Fsize)
+		    Lf->off_def = 1;
+		else if (ins) {
 		    Lf->sz = (SZOFFTYPE)i.i_size;
 		    Lf->sz_def = 1;
-		} else if ((type == VCHR || type == VBLK) && !Fsize)
-		    Lf->off_def = 1;
+		}
 		break;
 	    }
 	}
@@ -788,19 +851,19 @@ process_node(va)
 	case VREG:
 	case VDIR:
 	    ty = (type == VREG) ? "VREG" : "VDIR";
-	    Lf->dev_def = 1;
+	    Lf->dev_def = devs;
 	    Lf->dev = dev;
 	    break;
 	case VBLK:
 	    ty = "VBLK";
-	    Lf->dev_def = 1;
+	    Lf->dev_def = devs;
 	    Lf->dev = dev;
 	    Ntype = N_BLK;
 	    break;
 	case VCHR:
 	    ty = "VCHR";
 	    Lf->dev = dev;
-	    Lf->dev_def = 1;
+	    Lf->dev_def = devs;
 	    Ntype = N_CHR;
 	    break;
 	case VLNK:
@@ -823,14 +886,14 @@ process_node(va)
 	    case N_FIFO:
 		if (vfs && vfs->fsname) {
 		    Lf->dev = dev;
-		    Lf->dev_def = 1;
+		    Lf->dev_def = devs;
 		}
 		ty = "FIFO";
 		break;
 	    case N_PIPE:
 		if (vfs && vfs->fsname) {
 		    Lf->dev = dev;
-		    Lf->dev_def = 1;
+		    Lf->dev_def = devs;
 		}
 		ty = "PIPE";
 		break;
@@ -838,7 +901,7 @@ process_node(va)
 
 	    default:
 		Lf->dev = dev;
-		Lf->dev_def = 1;
+		Lf->dev_def = devs;
 		ty = "FIFO";
 	    }
 	    break;
