@@ -35,31 +35,69 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode1.c,v 1.3 96/03/13 07:33:33 abe Exp $";
+static char *rcsid = "$Id: dnode1.c,v 1.5 98/08/18 08:08:46 abe Exp $";
 #endif
 
 
+#if	defined(HASVXFS)
+
+# if	defined(HPUXKERNBITS) && HPUXKERNBITS>=64
+#define _INO_T
+typedef int ino_t;
+#define _TIME_T
+typedef int time_t;
+# endif	/* defined(HPUXKERNBITS) && HPUXKERNBITS>=64 */
+
 #include "lsof.h"
 
-#if	defined(HASVXFS)
+
 /*
- * The pool_id_t type does not seem to be defined in the header  files
- * distributed by HP.  However, <sys/fs/vx_hpux.h> requires  it when
- * _KERNEL is defined.  So we fake the pool_id_t definition.
+ * HP-UX versions below 10.20:
  *
- * <sys/fs/vx_hpux.h> also requires sv_sema_t.  It's defined in
- * <sys/sem_alpha.h> when _KERNEL is defined, but some other header file has
- * already included <sys/sem_alpha.h> with _KERNEL undefined.  So we fake the
- * sv_sema_t definition.
+ *    The pool_id_t type does not seem to be defined in the header  files
+ *    distributed by HP.  However, <sys/fs/vx_hpux.h> requires  it when
+ *    _KERNEL is defined.  So we fake the pool_id_t definition.
+ *
+ *    <sys/fs/vx_hpux.h> also requires sv_sema_t.  It's defined in
+ *    <sys/sem_alpha.h> when _KERNEL is defined, but some other header file has
+ *    already included <sys/sem_alpha.h> with _KERNEL undefined.  So we fake the
+ *    sv_sema_t definition.
+ *
+ * HP-UX version 10.20 and above:
+ *
+ *    The pool_id_t type is used by other header files for other purposes.
+ *    Redefine it for VXFS.  Delete some other conflicting definitions.
+ *    Don't #define _KERNEL.  Include a different set of VXFS header files.
  */
 
+
+# if	HPUXV>=1020
+#undef	te_offset
+#undef	i_size
+#undef	di_size
+#define	pool_id_t	vx_pool_id_t
+
+#  if	HPUXV>=1030
+#define	ulong	vx_ulong		/* avoid <sys/stream.h> conflict */
+#  endif	/* HPUXV>=1030 */
+
+#include <sys/fs/vx_hpux.h>
+#include <sys/fs/vx_port.h>
+#include <sys/fs/vx_inode.h>
+
+#  if	HPUXV>=1030
+#undef	ulong
+#  endif	/* HPUXV>=1030 */
+
+# else	/* HPUXV<1020 */
+
 #define	pool_id_t	caddr_t
 #define	sv_sema_t	caddr_t
-
 #define	_KERNEL
 #include <sys/fs/vx_hpux.h>
 #include <sys/fs/vx_inode.h>
 #undef	_KERNEL
+# endif	/* HPUXV>=1020 */
 
 
 /*
@@ -68,23 +106,23 @@ static char *rcsid = "$Id: dnode1.c,v 1.3 96/03/13 07:33:33 abe Exp $";
 
 int
 read_vxnode(v, vfs, dev)
-	struct vnode *v;		/* containing vnode */
+	struct vnode *v;		/* local containing vnode */
 	struct l_vfs *vfs;		/* local vfs structure */
 	dev_t *dev;			/* device number receiver */
 {
 	struct vx_inode i;
 
 	if (!v->v_data || kread((KA_T)v->v_data, (char *)&i, sizeof(i)))
-		return(1);
+	    return(1);
 /*
  * Return device number.
  */
-	if (v->v_type == VCHR)
-		*dev = v->v_rdev;
+	if (v->v_type == VCHR || v->v_type == VBLK)
+	    *dev = v->v_rdev;
 	else if (vfs && vfs->fsname)
-		*dev = vfs->dev;
+	    *dev = vfs->dev;
 	else
-		*dev = i.i_dev;
+	    *dev = i.i_dev;
 /*
  * Record inode number.
  */
@@ -93,9 +131,11 @@ read_vxnode(v, vfs, dev)
 /*
  * Record size.
  */
-	if (!Foffset) {
-		Lf->sz = (unsigned long)i.i_size;
-		Lf->sz_def = 1;
+	if (Foffset || ((v->v_type == VCHR || v->v_type == VBLK) && !Fsize))
+	    Lf->off_def = 1;
+	else {
+	    Lf->sz = (SZOFFTYPE)i.i_size;
+	    Lf->sz_def = 1;
 	}
 	return(0);
 }
