@@ -33,13 +33,19 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode2.c,v 1.2 98/03/06 08:21:29 abe Exp $";
+static char *rcsid = "$Id: dnode2.c,v 1.3 98/07/17 08:54:54 abe Exp $";
 #endif
 
-
 #if	defined(HAS_AFS)
-#include "lsof.h"
 
+# if	defined(HPUXKERNBITS) && HPUXKERNBITS>=64
+#define _INO_T
+typedef int ino_t;
+#define _TIME_T
+typedef int time_t;
+# endif	/* defined(HPUXKERNBITS) && HPUXKERNBITS>=64 */
+
+#include "lsof.h"
 #include <afs/stds.h>
 #include <afs/param.h>
 #undef	__dontcare__
