@@ -33,7 +33,7 @@
 
 
 /*
- * $Id: dproto.h,v 1.3 98/03/06 08:21:32 abe Exp $
+ * $Id: dproto.h,v 1.4 98/07/14 12:58:24 abe Exp $
  */
 
 
@@ -52,6 +52,11 @@ _PROTOTYPE(extern void process_lla,(KA_T la));
 
 _PROTOTYPE(extern struct l_vfs *readvfs,(struct vnode *lv));
 
+#if	HPUXV>=1030
+_PROTOTYPE(extern void process_stream_sock,(KA_T ip, KA_T pcb, char *pn, enum vtype vt));
+_PROTOTYPE(extern int read_mi,(KA_T sh, KA_T *ip, KA_T *pcb, char **pn));
+#endif	/* HPUXV>=1030 */
+
 #if     defined(HAS_AFS)
 _PROTOTYPE(extern struct vnode *alloc_vcache,(void));
 _PROTOTYPE(extern void ckAFSsym,(struct nlist *nl));
