@@ -32,13 +32,18 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.5 98/03/06 08:21:33 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.9 98/08/03 07:42:38 abe Exp $";
 #endif
 
+#if	defined(HPUXKERNBITS) && HPUXKERNBITS>=64
+#define _INO_T
+typedef int ino_t;
+#define _TIME_T
+typedef int time_t;
+#endif	/* defined(HPUXKERNBITS) && HPUXKERNBITS>=64 */
 
 #include "lsof.h"
 
-
 #if     HPUXV>=800 && defined(HPUX_CCITT)
 #include <x25/x25addrstr.h>
 #include <x25/x25stat.h>
@@ -314,10 +319,10 @@ void
 process_socket(sa)
 	KA_T sa;			/* socket address in kernel */
 {
-	struct in_addr *fa = (struct in_addr *)NULL;
+	unsigned char *fa = (unsigned char *)NULL;
 	int fam;
 	int fp, mbl, lp;
-	struct in_addr *la = (struct in_addr *)NULL;
+	unsigned char *la = (unsigned char *)NULL;
 	struct protosw p;
 	struct socket s;
 	char tbuf[32];
@@ -334,6 +339,7 @@ process_socket(sa)
 	struct x25pcb xp;
 	struct x25pcb_extension xpe;
 # endif	/* defined(HPUX_CCITT) */
+
 # if	HPUXV<1030
 	struct mbuf mb;
 	struct inpcb inp;
@@ -343,24 +349,8 @@ process_socket(sa)
 	struct datab db;
 	static char *dbf = (char *)NULL;
 	static int dbl = 0;
-	int i;
-	struct ipc_s ic;
-	int ics = 0;
-	KA_T pcb = (KA_T)NULL;
-	KA_T ka, qp;
 	struct msgb mb;
-	struct module_info mi;
-	u_short pt;
-	struct queue q;
-	struct qinit qi;
 	struct sockbuf rb, sb;
-	struct sth_s sh;
-	struct streams_queue sq;
-	struct tcp_s tc;
-	int tcs = 0;
-	tcph_t th;
-	int uds = 0;
-	struct udp_s ud;
 # endif	/* HPUXV<1030 */
 #endif	/* HPUXV>=800 */
 
@@ -449,11 +439,8 @@ process_socket(sa)
 	/*
 	 * Get the X25 PCB and its extension.
 	 */
-	    if (!s.so_pcb) {
-		enter_nm("no x.25 protocol control block");
-		return;
-	    }
-	    if (kread((KA_T)s.so_pcb, (char *)&xp, sizeof(xp))) {
+	    if (!s.so_pcb
+	    ||  kread((KA_T)s.so_pcb, (char *)&xp, sizeof(xp))) {
 		(void) sprintf(Namech, "can't read x.25 pcb at %s",
 		    print_kptr((KA_T)s.so_pcb, (char *)NULL));
 		enter_nm(Namech);
@@ -522,160 +509,48 @@ process_socket(sa)
 	/*
 	 * Handle HP-UX 10.30 and above socket streams.
 	 */
-	    if (!s.so_sth
-	    ||  kread((KA_T)s.so_sth, (char *)&sh, sizeof(sh))) {
+	    if (s.so_sth) {
 
-no_more_INET_info:
+		KA_T ip, pcb;
+		char *pn;
+	    /*
+	     * Read module information.
+	     */
+		if (read_mi((KA_T)s.so_sth, &ip, &pcb, &pn))
+		    return;
+		if (ip && pcb) {
 
-		enter_nm("no more information");
+		/*
+		 * If IP and TCP or UDP modules are present, process as a
+		 * stream socket.
+		 */
+		    process_stream_sock(ip, pcb, pn, VNON);
+		    return;
+		}
+	    /*
+	     * If an IP module's PCB address is present, print it as the
+	     * device characters.
+	     */
+
+		if (ip && !Lf->dev_def)
+		    enter_dev_ch(print_kptr(ip, (char *)NULL));
+		if (!strlen(Namech)) {
+
+		/*
+		 * If there are no NAME field characters, enter an error
+		 * message.
+		 */
+		    if (!ip) {
+			(void) strcpy(Namech,
+			    "no IP module for stream socket");
+		    } else {
+			(void) strcpy(Namech,
+			    "no TCP/UDP module for stream socket");
+		    }
+		}
+		enter_nm(Namech);
 		return;
 	    }
-	    for (i = 0, qp = (KA_T)sh.sth_wq;
-		 i < 20 && qp;
-		 i++, qp = (KA_T)q.q_next)
-	    {
-		if (kread(qp, (char *)&q, sizeof(q)))
-		    break;
-		if (!(ka = (KA_T)q.q_qinfo)
-		||  kread(ka, (char *)&qi, sizeof(qi)))
-		    continue;
-		if (!(ka = (KA_T)qi.qi_minfo)
-		||  kread(ka, (char *)&mi, sizeof(mi)))
-		    continue;
-		if (!(ka = (KA_T)mi.mi_idname)
-		||  kread(ka, (char *)tbuf, sizeof(tbuf) - 1))
-		    continue;
-		if (!(pcb = (KA_T)q.q_ptr))
-		    continue;
-		if (strncasecmp(tbuf, "IP", 2) == 0) {
-		    if (kread((KA_T)pcb, (char *)&ic, sizeof(ic)) == 0)
-			ics = 1;
-		    continue;
-		}
-		if (strncasecmp(tbuf, "TCP", 3) == 0) {
-		    if (kread((KA_T)pcb, (char *)&tc, sizeof(tc)) == 0)
-			tcs = 1;
-		    continue;
-		}
-		if (strncasecmp(tbuf, "UDP", 3) == 0) {
-		    if (kread((KA_T)pcb, (char *)&ud, sizeof(ud)) == 0)
-			uds = 1;
-		    continue;
-		}
-	    }
-	    if (ics) {
-
-	    /*
-	     * Print stream head's q_ptr address as protocol control block
-	     * address.
-	     */
-		if (pcb)
-		    enter_dev_ch(print_kptr((KA_T)pcb, (char *)NULL));
-		if (strcmp(Lf->iproto, "UDP") == 0) {
-
-		/*
-		 * Save UDP address and TPI state.
-		 */
-		    la = (struct in_addr *)&ic.ipc_udp_addr;
-		    pt = (u_short)ic.ipc_udp_port;
-		    if (la->s_addr == INADDR_ANY && pt == 0 && uds) {
-
-		    /*
-		     * If the ipc_s structure has no local address, use the
-		     * one in the udp_s structure.
-		     */
-			pt = (u_short)&ud.udp_port[0];
-		    }
-		    (void) ent_inaddr(la, (int)ntohs(pt),
-			(struct in_addr *)NULL, -1);
-		    if (!Fsize)
-			Lf->off_def = 1;
-		    if (uds) {
-			Lf->lts.type = 1;
-			Lf->lts.state.u = (unsigned int)ud.udp_state;
-		    }
-		} else if (strcmp(Lf->iproto, "TCP") == 0) {
-
-		/*
-		 * Save TCP address.
-		 */
-		    la = (struct in_addr *)&ic.ipc_tcp_laddr;
-		    pt = (u_short)ic.ipc_tcp_lport;
-		    if (la->s_addr == INADDR_ANY && pt == 0 && tcs) {
-
-		    /*
-		     * If the ipc_s structure has no local address, use the
-		     * local address in its tcp_iph structure, and the port
-		     * number in its tcph structure.
-		     */
-			la = (struct in_addr *)&tc.tcp_u.tcp_u_iph.iph_src[0];
-			if (tc.tcp_hdr_len && tc.tcp_tcph
-			&&  kread((KA_T)tc.tcp_tcph, (char *)&th, sizeof(th))
-			== 0)
-			    pt = (u_short)th.th_lport;
-		    }
-		    lp = (int)ntohs(pt);
-		    if ((int)ic.ipc_tcp_faddr != INADDR_ANY
-		    || (u_short)ic.ipc_tcp_fport != 0)
-		    {
-			fa = (struct in_addr *)&ic.ipc_tcp_faddr;
-			fp = (int)ntohs((u_short)ic.ipc_tcp_fport);
-		    }
-		    if (fa || la)
-			(void) ent_inaddr(la, lp, fa, fp);
-
-		/*
-		 * Save TCP state information.
-		 */
-		    if (tcs) {
-			Lf->lts.type = 0;
-			Lf->lts.state.i = (int)tc.tcp_state;
-		    }
-		/*
-		 * Save TCP size information.
-		 */
-
-# if	defined(HASTCPTPIQ) || defined(HASTCPTPIW)
-		    if (tcs) {
-
-			int rq, sq;
-
-#  if	defined(HASTCPTPIW)
-			Lf->lts.rw = (int)tc.tcp_rwnd;
-			Lf->lts.ww = (int)tc.tcp_swnd;
-			Lf->lts.rws = Lf->lts.wws = 1;
-#  endif	/* defined(HASTCPTPIW) */
-
-			if ((rq = (int)tc.tcp_rnxt - (int)tc.tcp_rack - 1) < 0)
-			    rq = 0;
-			if ((sq = (int)tc.tcp_snxt - (int)tc.tcp_suna) < 0)
-			    sq  = 0;
-
-#  if	defined(HASTCPTPIQ)
-			Lf->lts.rq = (unsigned long)rq;
-			Lf->lts.sq = (unsigned long)sq;
-			Lf->lts.rqs = Lf->lts.sqs = 1;
-#  endif	/* defined(HASTCPTPIQ) */
-
-			if (Fsize) {
-			    if (Lf->access == 'r')
-				Lf->sz = (SZOFFTYPE)rq;
-			    else if (Lf->access == 'w')
-				Lf->sz = (SZOFFTYPE)sq;
-			    else
-				Lf->sz = (SZOFFTYPE)(rq + sq);
-			    Lf->sz_def = 1;
-			} else
-			    Lf->off_def = 1;
-		    }
-# else	/* !defined(HASTCPTPIQ) && !defined(HASTCPTPIW) */
-		    Lf->off_def = 1;
-# endif	/* defined(HASTCPTPIQ) || defined(HASTCPTPIW) */
-
-		}
-	    } else
-		goto no_more_INET_info;
-	    return;
 #else	/* HPUXV<1030 */
 
 	/*
@@ -701,40 +576,39 @@ no_more_INET_info:
 							   : s.so_pcb),
 					(char *)NULL));
 		if (raw.rcb_laddr.sa_family == AF_INET)
-		    la = (struct in_addr *)&raw.rcb_laddr.sa_data[2];
+		    la = (unsigned char *)&raw.rcb_laddr.sa_data[2];
 		else if (raw.rcb_laddr.sa_family)
 		    printrawaddr(&raw.rcb_laddr);
 		if (raw.rcb_faddr.sa_family == AF_INET)
-		    fa = (struct in_addr *)&raw.rcb_faddr.sa_data[2];
+		    fa = (unsigned char *)&raw.rcb_faddr.sa_data[2];
 		else if (raw.rcb_faddr.sa_family) {
 		    (void) strcat(endnm(), "->");
 		    printrawaddr(&raw.rcb_faddr);
 		}
 		if (fa || la)
-		    (void) ent_inaddr(la, -1, fa, -1);
+		    (void) ent_inaddr(la, -1, fa, -1, AF_INET);
 	    } else {
 
 	    /*
 	     * Print Internet socket information.
 	     */
-		if (kread((KA_T)s.so_pcb, (char *) &inp, sizeof(inp))
-		||  (struct socket *)sa != inp.inp_socket) {
+		if (kread((KA_T)s.so_pcb, (char *)&inp, sizeof(inp))) {
 		    (void) sprintf(Namech, "can't read inpcb at %s",
-			print_kptr((KA_T)s.so_pcb));
+			print_kptr((KA_T)s.so_pcb, (char *)NULL));
 		    enter_nm(Namech);
 		    return;
 		}
 		enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
 							    : s.so_pcb),
 					(char *)NULL));
-		la = &inp.inp_laddr;
+		la = (unsigned char *)&inp.inp_laddr;
 		lp = (int)ntohs(inp.inp_lport);
 		if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport != 0) {
-		    fa = &inp.inp_faddr;
+		    fa = (unsigned char *)&inp.inp_faddr;
 		    fp = (int)ntohs(inp.inp_fport);
 		}
 		if (fa || la)
-		    (void) ent_inaddr(la, lp, fa, fp);
+		    (void) ent_inaddr(la, lp, fa, fp, AF_INET);
 		if (p.pr_protocol == IPPROTO_TCP && inp.inp_ppcb
 		&&  kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t)) == 0) {
 		    Lf->lts.type = 0;
@@ -756,22 +630,22 @@ no_more_INET_info:
 /*
  * Save size information for HP-UX 10.30 and above.
  */
- 	if (Fsize) {
-	    if (!s.so_rcv
-	    ||  kread((KA_T)s.so_rcv, (char *)&rb, sizeof(rb)))
-		rb.sb_cc = 0;
-	    if (!s.so_snd
-	    ||  kread((KA_T)s.so_snd, (char *)&sb, sizeof(sb)))
-		sb.sb_cc = 0;
-	    if (Lf->access == 'r')
-		Lf->sz = (SZOFFTYPE)rb.sb_cc;
-	    else if (Lf->access == 'w')
-		Lf->sz = (SZOFFTYPE)sb.sb_cc;
-	    else
-		Lf->sz = (SZOFFTYPE)(rb.sb_cc + sb.sb_cc);
-	    Lf->sz_def = 1;
-	} else
-	    Lf->off_def = 1;
+ 	    if (Fsize) {
+		if (!s.so_rcv
+		||  kread((KA_T)s.so_rcv, (char *)&rb, sizeof(rb)))
+		    rb.sb_cc = 0;
+		if (!s.so_snd
+		||  kread((KA_T)s.so_snd, (char *)&sb, sizeof(sb)))
+		    sb.sb_cc = 0;
+		if (Lf->access == 'r')
+		    Lf->sz = (SZOFFTYPE)rb.sb_cc;
+		else if (Lf->access == 'w')
+		    Lf->sz = (SZOFFTYPE)sb.sb_cc;
+		else
+		    Lf->sz = (SZOFFTYPE)(rb.sb_cc + sb.sb_cc);
+		Lf->sz_def = 1;
+	    } else
+		Lf->off_def = 1;
 #endif	/* HPUXV>=1030 */
 
 	/*
@@ -884,3 +758,192 @@ no_more_INET_info:
 	if (Namech[0])
 	    enter_nm(Namech);
 }
+
+
+#if	HPUXV>=1030
+/*
+ * process_stream_sock() - process stream socket
+ */
+
+void
+process_stream_sock(ip, pcb, pn, vt)
+	KA_T ip;			/* IP module's q_ptr */
+	KA_T pcb;			/* protocol's q_ptr */
+	char *pn;			/* protocol name */
+	enum vtype vt;			/* vnode type */
+{
+	unsigned char *fa = (unsigned char *)NULL;
+	int fp, lp, rq, sq;
+	struct ipc_s ic;
+	unsigned char *la = (unsigned char *)NULL;
+	size_t nl;
+	char *np;
+	u_short pt;
+	struct tcp_s tc;
+	tcph_t th;
+	struct udp_s ud;
+/*
+ * Set current Namech address, file type, and protocol.  If AF_INET selection
+ * is in effect, set its flag.
+ */
+	np = endnm();
+	nl = (size_t)(np - Namech);
+	if (Fnet)
+	    Lf->sf |= SELNET;
+	(void) strcpy(Lf->type, "inet");
+	if (Sfile && vt != VNON && Lf->dev_def && Lf->inp_ty == 1) {
+	    if (is_file_named((char *)NULL, vt))
+		Lf->sf |= SELNM;
+	}
+	if (pn) {
+	    (void) strcpy(Lf->iproto, pn);
+	    Lf->inp_ty = 2;
+	}
+/*
+ * Get IP structure.
+ */
+	if (!ip || kread(ip, (char *)&ic, sizeof(ic))) {
+	    (void) sprintf(np, "%scan't read IP control structure from %s",
+		nl ? " " : "", print_kptr(ip, (char *)NULL));
+	    enter_nm(Namech);
+	    return;
+	}
+	if (!Lf->dev_def)
+	    enter_dev_ch(print_kptr(ip, (char *)NULL));
+/*
+ * Check for protocol control block address.  Enter if non-NULL and clear
+ * device definition flag.
+ */
+	if (!pcb) {
+	    (void) sprintf(np, "%ssocket stream has no TCP or UDP module",
+		nl ? " " : "");
+	    enter_nm(Namech);
+	    return;
+	}
+/*
+ * Select processing by protocol name.
+ */
+	if (pn && !strcmp(pn, "TCP")) {
+
+	/*
+	 * Process TCP socket.
+	 */
+	    if (kread(pcb, (char *)&tc, sizeof(tc))) {
+		(void) sprintf(np, "%scan't read TCP PCB from %s",
+		    nl ? " " : "", print_kptr(pcb, (char *)NULL));
+		enter_nm(Namech);
+		return;
+	    }
+	/*
+	 * Save TCP address.
+	 */
+	    la = (unsigned char *)&ic.ipc_tcp_laddr;
+	    pt = (u_short)ic.ipc_tcp_lport;
+	    if (((struct in_addr *)la)->s_addr == INADDR_ANY && pt == 0) {
+
+	    /*
+	     * If the ipc_s structure has no local address, use the local
+	     * address in its tcp_iph structure, and the port number in its
+	     * tcph structure.
+	     */
+		la = (unsigned char *)&tc.tcp_u.tcp_u_iph.iph_src[0];
+		if (tc.tcp_hdr_len && tc.tcp_tcph
+		&&  kread((KA_T)tc.tcp_tcph, (char *)&th, sizeof(th))
+		== 0)
+		    pt = (u_short)th.th_lport;
+	    }
+	    lp = (int)ntohs(pt);
+	    if ((int)ic.ipc_tcp_faddr != INADDR_ANY
+	    || (u_short)ic.ipc_tcp_fport != 0)
+	    {
+		fa = (unsigned char *)&ic.ipc_tcp_faddr;
+		fp = (int)ntohs((u_short)ic.ipc_tcp_fport);
+	    }
+	    if (fa || la)
+		(void) ent_inaddr(la, lp, fa, fp, AF_INET);
+	/*
+	 * Save TCP state and size information.
+	 */
+	    Lf->lts.type = 0;
+	    Lf->lts.state.i = (int)tc.tcp_state;
+
+# if	defined(HASTCPTPIQ) || defined(HASTCPTPIW)
+#  if	defined(HASTCPTPIW)
+	    Lf->lts.rw = (int)tc.tcp_rwnd;
+	    Lf->lts.ww = (int)tc.tcp_swnd;
+	    Lf->lts.rws = Lf->lts.wws = 1;
+#  endif	/* defined(HASTCPTPIW) */
+
+	    if ((rq = (int)tc.tcp_rnxt - (int)tc.tcp_rack - 1) < 0)
+		rq = 0;
+	    if ((sq = (int)tc.tcp_snxt - (int)tc.tcp_suna) < 0)
+		sq  = 0;
+
+#  if	defined(HASTCPTPIQ)
+	    Lf->lts.rq = (unsigned long)rq;
+	    Lf->lts.sq = (unsigned long)sq;
+	    Lf->lts.rqs = Lf->lts.sqs = 1;
+#  endif	/* defined(HASTCPTPIQ) */
+
+	    if (Fsize) {
+		if (Lf->access == 'r')
+		    Lf->sz = (SZOFFTYPE)rq;
+		else if (Lf->access == 'w')
+		    Lf->sz = (SZOFFTYPE)sq;
+		else
+		    Lf->sz = (SZOFFTYPE)(rq + sq);
+		Lf->sz_def = 1;
+	    } else
+		Lf->off_def = 1;
+
+# else	/* !defined(HASTCPTPIQ) && !defined(HASTCPTPIW) */
+	    if (!Fsize)
+	        Lf->off_def = 1;
+# endif	/* defined(HASTCPTPIQ) || defined(HASTCPTPIW) */
+
+	    Namech[0] = '\0';
+	    return;
+	} else if (pn && !strcmp(pn, "UDP")) {
+
+	/*
+	 * Process UDP socket.
+	 */
+	    if (kread(pcb, (char *)&ud, sizeof(ud))) {
+		(void) sprintf(np, "%scan't read UDP PCB from %s",
+		    nl ? " " : "", print_kptr(pcb, (char *)NULL));
+		enter_nm(Namech);
+		return;
+	    }
+	/*
+	 * Save UDP address and TPI state.
+	 */
+	    la = (unsigned char *)&ic.ipc_udp_addr;
+	    pt = (u_short)ic.ipc_udp_port;
+	    if (((struct in_addr *)la)->s_addr == INADDR_ANY && pt == 0) {
+
+	    /*
+	     * If the ipc_s structure has no local address, use the one in the
+	     * udp_s structure.
+	     */
+		pt = (u_short)ud.udp_port[0];
+	    }
+	    (void) ent_inaddr(la, (int)ntohs(pt), (unsigned char *)NULL,
+		-1, AF_INET);
+	    if (!Fsize)
+		Lf->off_def = 1;
+	    Lf->lts.type = 1;
+	    Lf->lts.state.u = (unsigned int)ud.udp_state;
+	    Namech[0] = '\0';
+	    return;
+	} else {
+
+	/*
+	 * Record an unknown protocol.
+	 */
+	    (void) sprintf(np, "%sunknown stream protocol: %s",
+		nl ? " " : "", pn ? pn : "NUll");
+	}
+	if (Namech[0])
+	    enter_nm(Namech);
+}
+#endif	/* HPUXV>=1030 */
