@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.13 98/03/06 08:21:36 abe Exp $
+ * $Id: machine.h,v 1.18 98/07/14 12:58:48 abe Exp $
  */
 
 
@@ -39,7 +39,17 @@
 #define	MACHINE_H	1
 
 
+/*
+ * Do HP-UX 11 64 bit preparation.
+ */
+
+#if	defined(HPUXKERNBITS) && HPUXKERNBITS>=64
+#define	NLIST_TYPE	nlist64
+#endif	/* defined(HPUXKERNBITS) && HPUXKERNBITS>=64 */
+
+
 # if	HPUXV>=1030
+#include <sys/types.h>
 #include "kernbits.h"
 # else	/* HPUXV<1030 */
 #  if	HPUXV>=1020
@@ -181,6 +191,14 @@
 /* #define	HASINTSIGNAL	1 */
 
 
+/*
+ * HASKERNIDCK is defined for those dialects that support the comparison of
+ * the build to running kernel identity.
+ */
+
+#define	HASKERNIDCK	1
+
+
 /*
  * HASKOPT is defined for those dialects that support the -k option of
  * reading the kernel's name list from an optional file.
@@ -432,6 +450,7 @@
  * header files.
  */
 
+#define	USE_LIB_CKKV				1	/* ckkv.c */
 /* #define	USE_LIB_COMPLETEVFS		1	   cvfs.c */
 #define	USE_LIB_FIND_CH_INO			1	/* fino.c */
 #define	USE_LIB_IS_FILE_NAMED			1	/* isfn.c */
@@ -447,7 +466,6 @@
 
 #define	USE_LIB_READDEV				1	/* rdev.c */
 #define	USE_LIB_READMNT				1	/* rmnt.c */
-/* #define	USE_LIB_READVFS			1	   rvfs.c */
 /* #define	USE_LIB_RNAM			1	   rnam.c */
 #define	USE_LIB_RNCH				1	/* rnch.c */
 /* #define	USE_LIB_RNMH			1	   rnmh.c */
