@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.10 98/03/06 08:23:08 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.14 98/06/25 07:18:53 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -56,7 +56,7 @@ _PROTOTYPE(static void process_mapmem,(struct inode *x, struct vm_area_struct *m
 
 static int Coff = 0;			/* kernel loader format is COFF */
 static KA_T Kta;			/* kernel task[] address */
-static caddr_t *Nc = NULL;		/* node cache */
+static caddr_t *Nc = (caddr_t *)NULL;	/* node cache */
 static MALLOC_S Nn = 0;			/* number of Nc[] entries allocated */
 static char *MapNm[] = {		/* possible system map file paths */
 	(char *)NULL,			/* place marker for an optional path */
@@ -92,7 +92,7 @@ gather_proc_info()
 	int ppid = 0;
 	short pss, sf;
 	struct task_struct *t, ts;
-	static struct task_struct **tt = NULL;
+	static struct task_struct **tt = (struct task_struct **)NULL;
 	uid_t uid;
 
 #if	defined(HASHASHPID)
@@ -103,11 +103,11 @@ gather_proc_info()
 	int ttlm = NR_TASKS;
 #endif	/* defined(HASHASHPID) */
 
-#if	LINUXV>1321
+#if	LINUXV>13021
 	struct files_struct fis;
 	struct fs_struct fss;
 	struct mm_struct mms;
-#endif	/* LINUXV>1321 */
+#endif	/* LINUXV>13021 */
 
 #if	defined(HASPPID)
 	struct task_struct pt;
@@ -122,8 +122,8 @@ gather_proc_info()
  * Allocate space for task structure pointers and read them.
  */
 	len = (MALLOC_S)(ttlm * sizeof(struct task_struct *));
-	if (tt == (struct task_struct **)NULL) {
-	    if ((tt = (struct task_struct **)malloc(len)) == NULL) {
+	if (!tt) {
+	    if (!(tt = (struct task_struct **)malloc(len))) {
 		(void) fprintf(stderr,
 		    "%s: no space for %d task structure pointers\n", Pn, ttlm);
 		Exit(1);
@@ -158,8 +158,7 @@ gather_proc_info()
 		Tsp = thp;
 		thp = t->pidhash_next;
 #else	/* !defined(HASHASHPID) */
-		if ((Tsp = tt[nt]) == (struct task_struct *)NULL
-		||   kread((KA_T)Tsp, (char *)t, sizeof(ts)))
+		if (!(Tsp = tt[nt]) || kread((KA_T)Tsp, (char *)t, sizeof(ts)))
 		    continue;
 #endif	/* defined(HASHASHPID) */
 
@@ -191,15 +190,14 @@ gather_proc_info()
 		    (int)pss, (int)sf);
 		Plf = (struct lfile *)NULL;
 
-#if	LINUXV>1321
+#if	LINUXV>13021
 	    /*
 	     * Read file system structure for Linux version 1.3.22 and above.
 	     */
-		if (t->fs == (struct fs_struct *)NULL
-		||  kread((KA_T)t->fs, (char *)&fss, sizeof(fss)))
+		if (!t->fs || kread((KA_T)t->fs, (char *)&fss, sizeof(fss)))
 		    continue;
 		
-#endif	/* LINUXV>1321 */
+#endif	/* LINUXV>13021 */
 
 	    /*
 	     * Save current working directory information.
@@ -207,9 +205,9 @@ gather_proc_info()
 		Cfd = -1;
 		Cfp = (struct file *)NULL;
 
-#if	LINUXV<=1321
+#if	LINUXV<=13021
 		if ((ip = t->PWD))
-#else	/* LINUXV>1321 */
+#else	/* LINUXV>13021 */
 # if	defined(HASDENTRY)
 		if (fss.pwd
 		&&  kread((KA_T)fss.pwd, (char *)&de, sizeof(de)) == 0
@@ -217,7 +215,7 @@ gather_proc_info()
 # else	/* !defined(HASDENTRY) */
 		if ((ip = fss.pwd))
 # endif	/* defined(HASDENTRY) */
-#endif	/* LINUXV<=1321 */
+#endif	/* LINUXV<=13021 */
 
 		{
 		    alloc_lfile(CWD, -1);
@@ -229,9 +227,9 @@ gather_proc_info()
 	     * Save root directory information.
 	     */
 
-#if	LINUXV<=1321
+#if	LINUXV<=13021
 		if ((ip = t->ROOT))
-#else	/* LINUXV>1321 */
+#else	/* LINUXV>13021 */
 # if	defined(HASDENTRY)
 		if (fss.root
 		&&  kread((KA_T)fss.root, (char *)&de, sizeof(de)) == 0
@@ -239,7 +237,7 @@ gather_proc_info()
 # else	/* !defined(HASDENTRY) */
 		if ((ip = fss.root))
 # endif	/* defined(HASDENTRY) */
-#endif	/* LINUXV<=1321 */
+#endif	/* LINUXV<=13021 */
 
 		{
 		    alloc_lfile(RTD, -1);
@@ -251,47 +249,47 @@ gather_proc_info()
 	     * Save information on the executable file and mapped memory.
 	     */
 
-#if	LINUXV<1147
+#if	LINUXV<11047
 		if (t->executable || t->mmap)
 		    process_mapmem(t->executable, t->mmap);
-#else	/* LINUXV>=1147 */
-# if	LINUXV<=1321
+#else	/* LINUXV>=11047 */
+# if	LINUXV<=13021
 		if (t->mm->mmap)
-		    process_mapmem(NULL, t->mm->mmap);
-# else	/* LINUXV>1321 */
+		    process_mapmem((struct inode *)NULL, t->mm->mmap);
+# else	/* LINUXV>13021 */
 		if (t->mm
 		&&  kread((KA_T)t->mm, (char *)&mms, sizeof(mms)) == 0
 		&&  mms.mmap)
-		    process_mapmem(NULL, mms.mmap);
-# endif	/* LINUXV<=1321 */
-#endif	/* LINUXV<1147 */
+		    process_mapmem((struct inode *)NULL, mms.mmap);
+# endif	/* LINUXV<=13021 */
+#endif	/* LINUXV<11047 */
 
 	    /*
 	     * Save information on file descriptors.
 	     */
 
-#if	LINUXV>1321
-		if (t->files == (struct files_struct *)NULL
+#if	LINUXV>13021
+		if (!t->files
 		||  kread((KA_T)t->files, (char *)&fis, sizeof(fis)))
 		    continue;
-#endif	/* LINUXV>1321 */
+#endif	/* LINUXV>13021 */
 
 		for (Cfd = 0, f = &fs; Cfd < NR_OPEN; Cfd++) {
 
-#if	LINUXV<1147
+#if	LINUXV<11047
 		    if (!(Cfp = t->filp[Cfd]))
-#else	/* LINUXV>=1147 */
-# if	LINUXV<=1321
+#else	/* LINUXV>=11047 */
+# if	LINUXV<=13021
 		    if (!(Cfp = t->files->fd[Cfd]))
-# else	/* LINUXV>1321 */
+# else	/* LINUXV>13021 */
 		    if (!(Cfp = fis.fd[Cfd]))
-# endif	/* LINUXV<=1321 */
-#endif	/* LINUXV<1147 */
+# endif	/* LINUXV<=13021 */
+#endif	/* LINUXV<11047 */
 
 			continue;
 		    if (kread((KA_T)Cfp, (char *)f, sizeof(fs)))
 			continue;
-		    alloc_lfile(NULL, Cfd);
+		    alloc_lfile((char *)NULL, Cfd);
 		    process_file((KA_T)f);
 		    if (Lf->sf)
 			link_lfile();
@@ -317,6 +315,10 @@ gather_proc_info()
 static void
 get_kernel_access()
 {
+/*
+ * Check kernel version.
+ */
+	(void) ckkv("Linux", LSOF_VSTR, (char *)NULL, (char *)NULL);
 
 #if	defined(WILLDROPGID)
 /*
@@ -324,22 +326,22 @@ get_kernel_access()
  * before attempting to open the (Memory) file.
  */
 	if (Memory)
-		(void) dropgid();
+	    (void) dropgid();
 #else	/* !defined(WILLDROPGID) */
 /*
  * See if the non-KMEM memory file is readable.
  */
 	if (Memory && !is_readable(Memory, 1))
-		Exit(1);
+	    Exit(1);
 #endif	/* defined(WILLDROPGID) */
 
 /*
  * Open kernel memory access.
  */
 	if ((Kd = open(Memory ? Memory : KMEM, O_RDONLY, 0)) < 0) {
-		(void) fprintf(stderr, "%s: can't open %s: %s\n", Pn,
-			Memory ? Memory : KMEM, strerror(errno));
-		Exit(1);
+	    (void) fprintf(stderr, "%s: can't open %s: %s\n", Pn,
+		Memory ? Memory : KMEM, strerror(errno));
+	    Exit(1);
 	}
 
 #if	defined(WILLDROPGID)
@@ -347,13 +349,13 @@ get_kernel_access()
  * Drop setgid permission, if necessary.
  */
 	if (!Memory)
-		(void) dropgid();
+	    (void) dropgid();
 #else	/* !defined(WILLDROPGID) */
 /*
  * See if the name list file is readable.
  */
 	if (Nmlst && !is_readable(Nmlst, 1))
-		Exit(1);
+	    Exit(1);
 #endif	/* defined(WILLDROPGID) */
 
 /*
@@ -368,19 +370,18 @@ get_kernel_access()
 	    }
 	}
 	if (nlist_private(Nmlst, Nl) < 0) {
-		(void) fprintf(stderr,
-		    "%s: can't read kernel name list from %s\n", Pn, Nmlst);
-		Exit(1);
+	    (void) fprintf(stderr,
+		"%s: can't read kernel name list from %s\n", Pn, Nmlst);
+	    Exit(1);
 	}
 /*
  * Make sure important kernel symbols are defined.
  */
 	if (get_Nl_value("task", Drive_Nl, (unsigned long *)&Kta) < 0 || !Kta)
 	{
-		(void) fprintf(stderr,
-		    "%s: no %stask kernel definition\n",
-		    Pn, Coff ? "_" : "");
-		Exit(1);
+	    (void) fprintf(stderr, "%s: no %stask kernel definition\n",
+		Pn, Coff ? "_" : "");
+	    Exit(1);
 	}
 }
 
@@ -448,8 +449,8 @@ hash_ksym(nm)
 	int i, j;
 
 	for (i = j = 0; *nm; nm++) {
-		i ^= (int)*nm << j;
-		j ^= 1;
+	    i ^= (int)*nm << j;
+	    j ^= 1;
 	}
 	return(i % KSHASHSZ);
 }
@@ -470,17 +471,17 @@ hasparmhash(nm)
 	char *cp, *rv;
 	int n;
 
-	if ((cp = strrchr(nm, '_')) == NULL || *(cp + 1) != 'R')
-		return(NULL);
+	if (!(cp = strrchr(nm, '_')) || *(cp + 1) != 'R')
+	    return((char *)NULL);
 	rv = cp;
 	for (cp += 2, n = 0; n < 8; cp++, n++) {
-		if ((*cp >= '0' && *cp <= '9')
-		||  (*cp >= 'a' && *cp <= 'f')
-		||  (*cp >= 'A' && *cp <= 'F'))
-			continue;
-		return(NULL);
+	    if ((*cp >= '0' && *cp <= '9')
+	    ||  (*cp >= 'a' && *cp <= 'f')
+	    ||  (*cp >= 'A' && *cp <= 'F'))
+		continue;
+	    return((char *)NULL);
 	}
-	return((*cp == '\0') ? rv : NULL);
+	return((*cp == '\0') ? rv : (char *)NULL);
 }
 
 
@@ -495,6 +496,8 @@ initialize()
 	unsigned long hi, st;
 
 	get_kernel_access();
+	if (Selinet == 0)
+	    (void) readmnt();
 }
 
 
@@ -514,8 +517,13 @@ kread(addr, buf, len)
 {
 	int br;
 
+#if	defined(__alpha__)
+	if (lseek(Kd, (off_t)(addr & 0xffffffff), SEEK_SET) == (off_t)-1)
+#else	!defined(__alpha__)
 	if (lseek(Kd, (off_t)addr, SEEK_SET) == (off_t)-1)
-		return(-1);
+#endif	/* defined(__alpha__) */
+
+	    return(-1);
 	br = read(Kd, buf, len);
 	return((br == (int)len) ? 0 : 1);
 }
@@ -537,9 +545,9 @@ nlist_private(fnm, nl)
 	int i, j, mm, nf, nk, nn;
 	FILE *nfs;
 	struct ksym {
-		char *name;
-		unsigned long addr;
-		struct ksym *next;
+	    char *name;
+	    unsigned long addr;
+	    struct ksym *next;
 	} **kh, *kp, *kpn;
 	struct NLIST_TYPE *np;
 
@@ -550,12 +558,12 @@ nlist_private(fnm, nl)
 	struct module_symbol *qp;
 #else	/* !defined(HAS_QUERY_MODULE) */
 	int ik;
-	struct kernel_sym *ksa = NULL;
+	struct kernel_sym *ksa = (struct kernel_sym *)NULL;
 	struct kernel_sym *ks;
 #endif	/* defined(HAS_QUERY_MODULE) */
 
-	if ((nfs = fopen(fnm, "r")) == NULL)
-		return(-1);
+	if (!(nfs = fopen(fnm, "r")))
+	    return(-1);
 
 #if	defined(HAS_QUERY_MODULE)
 /*
@@ -609,9 +617,9 @@ nlist_private(fnm, nl)
 	/*
 	 * Allocate hash table space.
 	 */
-	    if ((kh = (struct ksym **)calloc((MALLOC_S)sizeof(struct ksym),
-		      KSHASHSZ))
-	    == NULL) {
+	    if (!(kh = (struct ksym **)calloc((MALLOC_S)sizeof(struct ksym),
+		       KSHASHSZ)))
+	    {
 		(void) fprintf(stderr,
 		    "%s: no space for %d byte kernel symbol hash table\n",
 		    Pn, (KSHASHSZ * sizeof(struct ksym)));
@@ -632,17 +640,16 @@ nlist_private(fnm, nl)
 		cp = (char *)(qb + j);
 		if (!*cp || *cp == '#')
 		    continue;
-		if ((kp = (struct ksym *)malloc((MALLOC_S)sizeof(struct ksym)))
-		== NULL) {
+		if (!(kp = (struct ksym *)malloc((MALLOC_S)sizeof(struct ksym)))
+		) {
 		    (void) fprintf(stderr,
 			"%s: no space for kernel symbol structure: %s\n",
 			Pn, nm);
 		    Exit(1);
 		}
-		if ((nm = hasparmhash(cp)) != (char *)NULL)
+		if ((nm = hasparmhash(cp)))
 		    *nm = '\0';	/* chop off module version number */
-		if ((kp->name = (char *)malloc((MALLOC_S)(strlen(cp) + 1)))
-		== (char *)NULL) {
+		if (!(kp->name = (char *)malloc((MALLOC_S)(strlen(cp) + 1)))) {
 		    (void) fprintf(stderr,
 			"%s: no space for kernel symbol name: %s\n", Pn, nm);
 		    Exit(1);
@@ -664,7 +671,7 @@ nlist_private(fnm, nl)
 /*
  * Read the kernel symbols via get_kernel_syms().
  */
-	if ((nk = get_kernel_syms(NULL)) < 0) {
+	if ((nk = get_kernel_syms((struct kernel_sym *)NULL)) < 0) {
 	    if (!Fwarn) {
 		if (errno == ENOSYS) {
 		    (void) fprintf(stderr,
@@ -681,7 +688,7 @@ nlist_private(fnm, nl)
 	    }
 	} else {
 	    i = nk * sizeof(struct kernel_sym);
-	    if ((ksa = (struct kernel_sym *)malloc((MALLOC_S)i)) == NULL) {
+	    if (!(ksa = (struct kernel_sym *)malloc((MALLOC_S)i))) {
 		(void) fprintf(stderr, "%s: no space for kernel symbols\n", Pn);
 		Exit(1);
 	    }
@@ -695,9 +702,9 @@ nlist_private(fnm, nl)
 	/*
 	 * Allocate hash table space.
 	 */
-	    if ((kh = (struct ksym **)calloc((MALLOC_S)sizeof(struct ksym),
-		      KSHASHSZ))
-	    == NULL) {
+	    if (!(kh = (struct ksym **)calloc((MALLOC_S)sizeof(struct ksym),
+		       KSHASHSZ)))
+	    {
 		(void) fprintf(stderr,
 		    "%s: no space for %d byte kernel symbol hash table\n",
 		    Pn, (KSHASHSZ * sizeof(struct ksym)));
@@ -723,17 +730,17 @@ nlist_private(fnm, nl)
 		}
 		if (!ik)
 		    continue;
-		if ((kp = (struct ksym *)malloc((MALLOC_S)sizeof(struct ksym)))
-		== NULL) {
+		if (!(kp = (struct ksym *)malloc((MALLOC_S)sizeof(struct ksym)))
+		) {
 		    (void) fprintf(stderr,
 			"%s: no space for kernel symbol structure: %s\n",
 			Pn, nm);
 		    Exit(1);
 		}
-		if ((nm = hasparmhash(ks->name)) != (char *)NULL)
+		if ((nm = hasparmhash(ks->name)))
 		    *nm = '\0';	/* chop off module version number */
-		if ((kp->name = (char *)malloc((MALLOC_S)(strlen(ks->name)+1)))
-		== (char *)NULL) {
+		if (!(kp->name = (char *)malloc((MALLOC_S)(strlen(ks->name)+1)))
+		) {
 		    (void) fprintf(stderr,
 			"%s: no space for kernel symbol name: %s\n", Pn, nm);
 		    Exit(1);
@@ -760,11 +767,11 @@ nlist_private(fnm, nl)
 	if ((!Coff && !elf) || (Coff && elf)) {
 
 #if	defined(KERN_LD_ELF)
-		Coff = 0;
-		elf = 1;
+	    Coff = 0;
+	    elf = 1;
 #else	/* !defined(KERN_LD_ELF) */
-		Coff = 1;
-		elf = 0;
+	    Coff = 1;
+	    elf = 0;
 #endif	/* defined(KERN_LD_ELF) */
 
 	    if (!Fwarn) {
@@ -788,12 +795,12 @@ nlist_private(fnm, nl)
  * in Nl[] and quit when addresses have been located for all of them.
  */
 	for (nn = 0, np = nl; np->NL_NAME; nn++, np++)
-		;
+	    ;
 	mm = nf = 0;
-	while (nf < nn && fgets(buf, sizeof(buf), nfs) != NULL) {
-	    if ((eol = strchr(buf, '\n')) == NULL)
+	while (nf < nn && fgets(buf, sizeof(buf), nfs)) {
+	    if (!(eol = strchr(buf, '\n')))
 		continue;
-	    if ((cp = strchr(buf, ' ')) == NULL)
+	    if (!(cp = strchr(buf, ' ')))
 		continue;
 	    nm = cp + 3;
 	    *eol = '\0';
@@ -875,7 +882,7 @@ nlist_private(fnm, nl)
  */
 	if (kh) {
 	    for (i = 0; i < KSHASHSZ; i++) {
-		if ((kp = kh[i]) == NULL)
+		if (!(kp = kh[i]))
 		    continue;
 		while (kp) {
 		    kpn = kp->next;
@@ -916,78 +923,78 @@ process_mapmem(x, m)
 	/*
 	 * Avoid infinite loop.
 	 */
-		if (lm > 1000) {
-			if (!Fwarn)
-				(void) fprintf(stderr,
-					"%s: too many memory maps, PID %d\n",
-					Pn, Lp->pid);
-				return;
-		}
-		if (f) {
+	    if (lm > 1000) {
+		if (!Fwarn)
+		    (void) fprintf(stderr,
+			"%s: too many memory maps, PID %d\n",
+			Pn, Lp->pid);
+		return;
+	    }
+	    if (f) {
 		
-		/*
-		 * The first time through the loop, use the executable inode
-		 * address.
-		 */
-			ty = "txt";
-			f = 0;
-			if ((ia = x) == (struct inode *)NULL)
-				continue;
-		} else {
+	    /*
+	     * The first time through the loop, use the executable inode
+	     * address.
+	     */
+		ty = "txt";
+		f = 0;
+		if (!(ia = x))
+		    continue;
+	    } else {
 
-		/*
-		 * After the first time through the loop, use the inode
-		 * addresses in the memory map.
-		 */
-			if (kread((KA_T)m, (char *)&v, sizeof(v)))
-				break;
-			m = v.vm_next;
+	    /*
+	     * After the first time through the loop, use the inode
+	     * addresses in the memory map.
+	     */
+		if (kread((KA_T)m, (char *)&v, sizeof(v)))
+		    break;
+		m = v.vm_next;
 
 #if	defined(HASDENTRY)
-			if (!v.vm_dentry
-			||  kread((KA_T)v.vm_dentry, (char *)&de, sizeof(de))
-			||  !(ia = de.d_inode))
+		if (!v.vm_dentry
+		||  kread((KA_T)v.vm_dentry, (char *)&de, sizeof(de))
+		||  !(ia = de.d_inode))
 #else	/* !defined(HASDENTRY) */
-			if (!(ia = v.vm_inode))
+		if (!(ia = v.vm_inode))
 #endif	/* defined(HASDENTRY) */
 
-				continue;
-			ty = "mem";
-		}
+		    continue;
+		ty = "mem";
+	    }
 	/*
 	 * Skip duplicate inode addresses.
 	 */
-		for (j = 0; j < i; j++) {
-			if (Nc[j] == (caddr_t)ia)
-				break;
-		}
-		if (j < i)
-			continue;
+	    for (j = 0; j < i; j++) {
+		if (Nc[j] == (caddr_t)ia)
+		    break;
+	    }
+	    if (j < i)
+		continue;
 	/*
 	 * Cache the inode address for duplicate checking.
 	 */
-		if (i >= Nn) {
-			Nn += 10;
-			ma = (MALLOC_S)(Nn * sizeof(caddr_t));
-			if (Nc)
-				Nc = (caddr_t *)realloc(Nc, ma);
-			else
-				Nc = (caddr_t *)malloc(ma);
-			if (!Nc) {
-			    (void) fprintf(stderr,
-				"%s: no space for memmap pointers, PID %d\n",
-				Pn, Lp->pid);
-			    Exit(1);
-			}
+	    if (i >= Nn) {
+		Nn += 10;
+		ma = (MALLOC_S)(Nn * sizeof(caddr_t));
+		if (Nc)
+		    Nc = (caddr_t *)realloc(Nc, ma);
+		else
+		    Nc = (caddr_t *)malloc(ma);
+		if (!Nc) {
+		    (void) fprintf(stderr,
+			"%s: no space for memmap pointers, PID %d\n",
+			Pn, Lp->pid);
+		    Exit(1);
 		}
-		Nc[i++] = (caddr_t)ia;
+	    }
+	    Nc[i++] = (caddr_t)ia;
 	/*
 	 * Save the inode information.
 	 */
-		alloc_lfile(ty, -1);
-		process_inode(ia, (struct file_operations *)NULL);
-		if (Lf->sf)
-			link_lfile();
+	    alloc_lfile(ty, -1);
+	    process_inode(ia, (struct file_operations *)NULL);
+	    if (Lf->sf)
+		link_lfile();
 	}
 }
 
@@ -1040,11 +1047,11 @@ _PROTOTYPE(static struct l_nch *ncache_addr,(dev_t *d, unsigned long i));
 
 _PROTOTYPE(static void ncache_ent_nm,(struct l_nch *lc, char *nm, int len));
 
-#  if	LINUXV<=1321
+#  if	LINUXV<=13021
 #define	DEV	dev
-#  else	/* LINUXV>1321 */
+#  else	/* LINUXV>13021 */
 #define	DEV	dc_dev
-#  endif	/* LINUXV<=1321 */
+#  endif	/* LINUXV<=13021 */
 
 #  if	defined(HASINOKNC)
 #define hash(na)	Hp+((((int)(na)>>2)*31415)&Hm)
@@ -1142,7 +1149,7 @@ ncache_isroot(na)
  */
 	if (kread((KA_T)na, (char *)&i, sizeof(i)))
 	    return(0);
-	for (mtp = Mtab; mtp; mtp = mtp->next) {
+	for (mtp = readmnt(); mtp; mtp = mtp->next) {
 	    if (!mtp->dir || !mtp->inode)
 		continue;
 	    if (i.i_dev == mtp->dev && i.i_ino == mtp->inode)
@@ -1270,7 +1277,7 @@ ncache_load()
 	/*
 	 * Allocate a buffer to read the kernel caches.
 	 */
-	    if ((c = (char *)malloc(cl)) == (char *)NULL) {
+	    if (!(c = (char *)malloc(cl))) {
 		(void) fprintf(stderr,
 		    "%s: no space for %d byte kernel name cache copy\n",
 		    Pn, cl);
@@ -1289,7 +1296,7 @@ ncache_load()
 		    }
 		}
 		(void) free((FREE_P *)Hp);
-		Hp = NULL;
+		Hp = (struct l_nch **)NULL;
 	    }
 	}
 	Csz = 0;
@@ -1442,9 +1449,7 @@ ncache_load()
 	}
 	if (Csz < Asz && !RptTm) {
 	    len = Csz * sizeof(struct l_nch);
-	    if ((Nmc = (struct l_nch *)realloc(Nmc, len))
-	    == (struct l_nch *)NULL)
-	    {
+	    if (!(Nmc = (struct l_nch *)realloc(Nmc, len))) {
 		(void) fprintf(stderr, "%s: can't realloc local name cache\n",
 		    Pn);
 		Exit(1);
@@ -1458,8 +1463,8 @@ ncache_load()
 	    ;
 	Nh <<= 1;
 	Hm = Nh - 1;
-	if ((Hp = (struct l_nch **)calloc(Nh + Csz, sizeof(struct l_nch *)))
-	== (struct l_nch **)NULL) {
+	if (!(Hp = (struct l_nch **)calloc(Nh + Csz, sizeof(struct l_nch *))))
+	{
 	    if (!Fwarn)
 		(void) fprintf(stderr,
 		    "%s: no space for %d name cache hash pointers\n",
@@ -1586,7 +1591,7 @@ ncache_lookup(buf, blen, fp)
 	if (!Lf->dev_def || Lf->inp_ty != 1)
 #  endif	/* defined(HASINOKNC) */
 
-		return(NULL);
+		return((char *)NULL);
 	if (Csz
 
 #  if	defined(HASINOKNC)
@@ -1598,7 +1603,7 @@ ncache_lookup(buf, blen, fp)
 	&&  !lc->dup)
 	{
 	    if ((nl = (int)lc->nl) > (blen - 1))
-		return(*cp ? cp : NULL);
+		return(*cp ? cp : (char *)NULL);
 	    cp = buf + blen - nl - 1;
 	    rlen = blen - nl - 1;
 	    (void) strcpy(cp, lc->nm);
@@ -1606,7 +1611,7 @@ ncache_lookup(buf, blen, fp)
 #  if	defined(HASINOKNC)
 	    if (!lc->pa && lc->rp) {
 		*fp = 1;
-		return(*cp ? cp : NULL);
+		return(*cp ? cp : (char *)NULL);
 	    }
 #  endif	/* defined(HASINOKNC) */
 
@@ -1663,10 +1668,10 @@ ncache_lookup(buf, blen, fp)
 # endif	/* defined(HASINOKNC) */
 
 	    }
-	    return(*cp ? cp : NULL);
+	    return(*cp ? cp : (char *)NULL);
 	}
 # endif	/* defined(HASDEVKNC) || defined(HASINOKNC) || defined(HASNFSKNC) */
 
-	return(NULL);
+	return((char *)NULL);
 }
 #endif	/* defined(HASNCACHE) */
