@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.3 98/01/16 14:30:18 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.4 98/05/22 07:14:38 abe Exp $";
 #endif
 
 
@@ -1055,7 +1055,8 @@ process_proc_sock(p, s, l)
 {
 	struct ax25sin *ap;
 	char *cp, dev_ch[32], *path;
-	struct in_addr *fa, fs, *la, ls;
+	unsigned char *fa, *la;
+	struct in_addr fs, ls;
 	struct ipxsin *ip;
 	int len, nl;
 	char *pr;
@@ -1091,15 +1092,15 @@ process_proc_sock(p, s, l)
 	    enter_dev_ch(dev_ch);
 	    if (tp->faddr || tp->fport) {
 		fs.s_addr = tp->faddr;
-		fa = &fs;
+		fa = (unsigned char *)&fs;
 	    } else
-		fa = (struct in_addr *)NULL;
+		fa = (unsigned char *)NULL;
 	    if (tp->laddr || tp->lport) {
 		ls.s_addr = tp->laddr;
-		la = &ls;
+		la = (unsigned char *)&ls;
 	    } else
-		la = (struct in_addr *)NULL;
-	    ent_inaddr(la, tp->lport, fa, tp->fport);
+		la = (unsigned char *)NULL;
+	    ent_inaddr(la, tp->lport, fa, tp->fport, AF_INET);
 	    Lf->lts.type = tp->proto;
 	    Lf->lts.state.i = tp->state;
 
