@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.7 98/03/06 08:25:36 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.9 98/06/29 15:54:19 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -69,6 +69,43 @@ _PROTOTYPE(static void process_text,(KA_T prp));
 _PROTOTYPE(static void readfsinfo,(void));
 
 
+/*
+ * ckkv - check kernel version
+ */
+
+static void
+ckkv(d, er, ev, ea)
+	char *d;			/* dialect */
+	char *er;			/* expected release */
+	char *ev;			/* expected version */
+	char *ea;			/* expected architecture */
+{
+
+#if	defined(HASKERNIDCK)
+	struct scoutsname s;
+
+	if (Fwarn)
+	    return;
+/*
+ * Read OSR kernel information.
+ */
+	if (__scoinfo(&s, sizeof(s)) < 0) {
+	    (void) fprintf(stderr, "%s: can't get __scoinfo: %s\n",
+		Pn, strerror(errno));
+	    Exit(1);
+	}
+/*
+ * Warn if the actual and expected releases don't match.
+ */
+	if (!er || strcmp(er, s.release))
+	    (void) fprintf(stderr,
+		"%s: WARNING: compiled for %s release %s; this is %s.\n",
+		Pn, d, er ? er : "UNKNOWN", s.release);
+#endif	/* defined(HASKERNIDCK) */
+
+}
+
+
 /*
  * gather_proc_info() -- gather process information
  */
@@ -306,6 +343,10 @@ get_kernel_access()
 	time_t lbolt;
 	MALLOC_S len;
 	KA_T v;
+/*
+ * Check kernel version.
+ */
+	(void) ckkv("OSR", LSOF_VSTR, (char *)NULL, (char *)NULL);
 /*
  * See if the name list file is readable.
  */
