@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.6 98/03/06 08:27:34 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.8 98/06/13 09:46:43 abe Exp $";
 #endif
 
 
@@ -238,12 +238,12 @@ process_node(va)
 	int dl, fp, j, k, l, lp, pt, px;
 	struct l_dev *dp;
 	struct fifonode f;
-	struct in_addr *fa = (struct in_addr *)NULL;
+	unsigned char *fa = (unsigned char *)NULL;
 	static int ft = 1;
 	static KA_T fvops = (KA_T)0;
 	struct inode i;
 	KA_T ka, p, qp, sop;
-	struct in_addr *la = (struct in_addr *)NULL;
+	unsigned char *la = (unsigned char *)NULL;
 	struct module_info mi;
 	static KA_T nfsvops = (KA_T)0;
 	static KA_T nmvops = (KA_T)0;
@@ -469,7 +469,7 @@ read_vnode:
 	    if (!v.v_data || read_vxinode(&v, lvfs, &dev)) {
 		(void) sprintf(Namech,
 		    "vnode at %s: can't read vx_inode (%s)",
-		    print_kptr(va, tptr),
+		    print_kptr(va, tbuf),
 		    print_kptr((KA_T)v.v_data, (char *)NULL));
 		enter_nm(Namech);
 		return;
@@ -836,7 +836,7 @@ read_vnode:
 	    /*
 	     * Print the local and foreign addresses from the PCB.
 	     */
-		la = (struct in_addr *)&pcb.inp_laddr;
+		la = (unsigned char *)&pcb.inp_laddr;
 		lp = (int)ntohs(pcb.inp_lport);
 
 #if	defined(HASINADDRSTR)
@@ -846,11 +846,11 @@ read_vnode:
 #endif	/* defined(HASINADDRSTR) */
 
 		{
-		    fa = (struct in_addr *)&pcb.inp_faddr;
+		    fa = (unsigned char *)&pcb.inp_faddr;
 		    fp = (int)ntohs(pcb.inp_fport);
 		}
 		if (fa || la)
-		    (void) ent_inaddr(la, lp, fa, fp);
+		    (void) ent_inaddr(la, lp, fa, fp, AF_INET);
 		Lf->inp_ty = 2;
 	    } else if (sop) {
 		    
