@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.3 98/03/06 08:27:39 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.7 98/06/26 14:38:20 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -329,6 +329,10 @@ get_kernel_access()
 	int warn;
 #endif	/* PTXV>=400 */
 
+/*
+ * Check kernel version.
+ */
+	(void) ckkv("PTX", (char *)NULL, LSOF_VSTR, (char *)NULL);
 #if	PTXV<400
 /*
  * Open swap access.
@@ -792,7 +796,7 @@ LNC_isroot(np, cp, ty)
 		return(0);
 	    if ((len + 1 + strlen(cp) + 1) > sizeof(buf))
 		return(0);
-	    for (mtp = Mtab; mtp; mtp = mtp->next) {
+	    for (mtp = readmnt(); mtp; mtp = mtp->next) {
 		if (!mtp->dir || !mtp->inode)
 		    continue;
 		if (strcmp(Lf->fsdir, mtp->dir) == 0)
@@ -1009,7 +1013,7 @@ ncache_lookup(buf, blen, fp)
 	 */
 	    if (!Lf->fsdir || !Lf->dev_def || Lf->inp_ty != 1)
 		return(NULL);
-	    for (mtp = Mtab; mtp; mtp = mtp->next) {
+	    for (mtp = readmnt(); mtp; mtp = mtp->next) {
 		if (!mtp->dir || !mtp->inode)
 		    continue;
 		if (Lf->dev == mtp->dev
@@ -1117,16 +1121,18 @@ VN_load()
  */
 	lc.type = 0;
 	for (cp = kc, i = 0; i < kcn; cp++, i++) {
-	    if (!cp->vp || !cp->name[0] || cp->namlen < 1)
+	    if (!cp->NCACHE_VP || !cp->NCACHE_NAME[0] || cp->NCACHE_NAMLEN < 1)
 		continue;
-	    if (cp->namlen < 3 && cp->name[0] == '.') {
-		if (cp->namlen == 1 || (cp->namlen == 2 && cp->name[1] == '.'))
+	    if (cp->NCACHE_NAMLEN < 3 && cp->NCACHE_NAME[0] == '.') {
+		if (cp->NCACHE_NAMLEN == 1
+		||  (cp->NCACHE_NAMLEN == 2 && cp->NCACHE_NAME[1] == '.'))
 		    continue;
 	    }
-	    len = (cp->namlen > NC_NAMLEN) ? NC_NAMLEN : cp->namlen;
-	    lc.np = (KA_T)cp->vp;
-	    lc.dp = (KA_T)cp->dp;
-	    LNC_enter(&lc, cp->name, len);
+	    len = (cp->NCACHE_NAMLEN > NC_NAMLEN) ? NC_NAMLEN
+						  : cp->NCACHE_NAMLEN;
+	    lc.np = (KA_T)cp->NCACHE_VP;
+	    lc.dp = (KA_T)cp->NCACHE_DP;
+	    LNC_enter(&lc, cp->NCACHE_NAME, len);
 	}
 /*
  * If not repeating, free space allocated for the copy of the kernel's
