@@ -33,7 +33,7 @@
 
 
 /*
- * $Id: dproto.h,v 1.4 96/03/24 12:49:32 abe Exp $
+ * $Id: dproto.h,v 1.3 98/03/06 08:27:41 abe Exp $
  */
 
 
@@ -41,9 +41,20 @@ _PROTOTYPE(extern void completevfs,(struct l_vfs *vfs, dev_t *dev));
 _PROTOTYPE(extern void findclones,(void));
 _PROTOTYPE(extern int is_file_named,(char *p, enum vtype ty));
 _PROTOTYPE(extern void process_socket,(char *pn, int fam, int ipid, KA_T pa));
-_PROTOTYPE(extern int readinode,(struct inode *ia, struct inode *i));
+_PROTOTYPE(extern int readinode,(KA_T ia, struct inode *i));
 _PROTOTYPE(extern struct l_vfs *readvfs,(struct vnode *va));
 
 #if	defined(HAS_NFS)
-_PROTOTYPE(extern int readrnode,(caddr_t ra, struct rnode *r));
+_PROTOTYPE(extern int readrnode,(KA_T ra, struct rnode *r));
 #endif	/* defined(HAS_NFS) */
+
+#if	PTXV>=421
+_PROTOTYPE(extern void process_unix_socket,(struct socket *s, struct unpcb *u));
+#endif	/* PTXV>=421 */
+
+#if	defined(HASVXFS)
+_PROTOTYPE(extern int read_vxinode,(struct vnode *v, struct l_vfs *vfs, dev_t *dev));
+# if	defined(HASNCACHE) && defined(HASVXFSDNLC)
+_PROTOTYPE(extern int read_vxvnode,(KA_T va, struct vnode *v));
+# endif	/* defined(HASNCACHE) && defined(HASVXFSDNLC) */
+#endif	/* defined(HASVXFS) */
