@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.8 98/03/06 08:27:43 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.9 98/05/22 07:15:50 abe Exp $";
 #endif
 
 
@@ -241,9 +241,9 @@ process_socket(pn, fam, ipid, pa)
 	KA_T pa;			/* kernel PCB address */
 {
 	char *cp;
-	struct in_addr *fa = (struct in_addr *)NULL;
+	unsigned char *fa = (unsigned char *)NULL;
 	int fp, lp;
-	struct in_addr *la = (struct in_addr *)NULL;
+	unsigned char *la = (unsigned char *)NULL;
 	struct inpcb pcb;
 	struct queue q;
 	struct stdata sd;
@@ -288,7 +288,7 @@ process_socket(pn, fam, ipid, pa)
 	    case IPPROTO_ICMP:
 	    case IPPROTO_TCP:
 	    case IPPROTO_UDP:
-		la = (struct in_addr *)&pcb.inp_laddr;
+		la = (unsigned char *)&pcb.inp_laddr;
 		lp = (int)ntohs(pcb.inp_lport);
 
 #if	defined(HASINADDRSTR)
@@ -298,11 +298,11 @@ process_socket(pn, fam, ipid, pa)
 #endif	/* defined(HASINADDRSTR) */
 
 		{
-		    fa = (struct in_addr *)&pcb.inp_faddr;
+		    fa = (unsigned char *)&pcb.inp_faddr;
 		    fp = (int)ntohs(pcb.inp_fport);
 		}
 		if (fa || la)
-		    (void) ent_inaddr(la, lp, fa, fp);
+		    (void) ent_inaddr(la, lp, fa, fp, AF_INET);
 		if (ipid == IPPROTO_TCP && pcb.inp_ppcb
 		&&  !kread((KA_T)pcb.inp_ppcb, (char *)&tcb, sizeof(tcb))) {
 		    if (!Fsize) {
