@@ -31,18 +31,30 @@
 
 
 /*
- * $Id: machine.h,v 1.19 96/03/24 12:50:53 abe Exp $
+ * $Id: machine.h,v 1.15 98/06/29 15:41:32 abe Exp $
  */
 
 
-#if	_PTXV>=400
+#if	!defined(MACHINE_H)
+#define	MACHINE_H	1
+
+
+# if	PTXV>=400
 #define	_KMEMUSER	1
-#endif	/* _PTXV>=400 */
+# endif	/* PTXV>=400 */
 
 #include <sys/types.h>
 #include <sys/param.h>
 
 
+/*
+ * GET_MAX_FD is defined for those dialects that provide a function other than
+ * getdtablesize() to obtain the maximum file descriptor number plus one.
+ */
+
+/* #define	GET_MAX_FD	?	*/
+
+
 /*
  * HASAOPT is defined for those dialects that have AFS support; it specifies
  * that the default path to an alternate AFS kernel name list file may be
@@ -52,6 +64,14 @@
 /* #define	HASAOPT		1 */
 
 
+/*
+ * HASBLKDEV is defined for those dialects that want block device information
+ * recorded in BDevtp[].
+ */
+
+#define	HASBLKDEV	1
+
+
 /*
  * HASDCACHE is defined for those dialects that support a device cache
  * file.
@@ -121,7 +141,8 @@
 
 
 /*
- * HASINODE is defined for those dialects that have inodes.
+ * HASINODE is defined for those dialects that have inodes and wish to
+ * use readinode() from node.c.
  */
 
 /* #define	HASINODE	1 */
@@ -135,6 +156,14 @@
 /* #define	HASINTSIGNAL	1 */
 
 
+/*
+ * HASKERNIDCK is defined for those dialects that support the comparison of
+ * the build to running kernel identity.
+ */
+
+#define	HASKERNIDCK	1
+
+
 /*
  * HASKOPT is defined for those dialects that support the -k option of
  * reading the kernel's name list from an optional file.
@@ -160,18 +189,18 @@
  * private lfile elements are used.
  */
 
-#define HASLFILEADD unsigned char fct_def, ina_def; long fct; caddr_t ina;
+#define HASLFILEADD unsigned char fct_def, ina_def; long fct; KA_T ina;
 #define SETLFILEADD Lf->fct_def = Lf->ina_def = 0;
 #define	HASFIELDAP1	"PTX link count"
 #define LISTLFILEAP1 \
-	if (FieldSel[LSOF_FIX_TERM + 1].n && Lf->fct_def) { \
+	if (FieldSel[LSOF_FIX_TERM + 1].st && Lf->fct_def) { \
 		(void) printf("1%ld%c", Lf->fct, Terminator); \
 		lc++; \
 	}
 
 #define	HASFIELDAP2	"PTX inode address (0x<hex>)"
 #define LISTLFILEAP2 \
-	if (FieldSel[LSOF_FIX_TERM + 2].n && Lf->ina_def) { \
+	if (FieldSel[LSOF_FIX_TERM + 2].st && Lf->ina_def) { \
 		(void) printf("20x%lx%c", Lf->ina, Terminator); \
 		lc++; \
 	}
@@ -205,6 +234,17 @@
 #define	HASNLIST	1
 
 
+/*
+ * HASPIPEFN is defined for those dialects that have a special function to
+ * process DTYPE_PIPE file structure entries.  Its value is the name of the
+ * function.
+ *
+ * NOTE: don't forget to define a prototype for this function in dproto.h.
+ */
+
+/* #define	HASPIPEFN	process_pipe */
+
+
 /*
  * HASPIPENODE is defined for those dialects that have pipe nodes.
  */
@@ -212,6 +252,62 @@
 /* #define	HASPIPENODE	1 */
 
 
+/*
+ * HASPMAPENABLED is defined when the reporting of portmapper registration
+ * info is enabled by default.
+ */
+
+/* #define	HASPMAPENABLED	1 */
+
+
+/*
+ * HASPPID is defined for those dialects that support identification of
+ * the parent process IDentifier (PPID) of a process.
+ */
+
+#define	HASPPID		1
+
+
+/*
+ * HASPRINTDEV, HASPRINTINO, HASPRINTNM, HASPRINTOFF, and HASPRINTSZ
+ * define private dialect-specific functions for printing DEVice numbers,
+ * INOde numbers, NaMes, file OFFsets, and file SiZes.  The functions are
+ * called from print_file().
+ */
+
+#define	HASPRINTDEV	print_dev
+/* #define	HASPRINTINO	print_ino	*/
+#define	HASPRINTNM	print_nm
+/* #define	HASPRINTOFF	print_off	*/
+/* #define	HASPRINTSZ	print_sz	*/
+
+
+/*
+ * HASPRIVFILETYPE and PRIVFILETYPE are defined for dialects that have a
+ * file structure type that isn't defined by a DTYPE_* symbol.  They are
+ * used in lib/prfp.c to select the type's processing.
+ *
+ * PRIVFILETYPE is the definition of the f_type value in the file struct.
+ *
+ * HASPRIVFILETYPE is the name of the processing function.
+ */
+
+/* #define	HASPRIVFILETYPE	process_shmf	*/
+/* #define	PRIVFILETYPE	??	*/
+
+
+/*
+ * HASPRIVNMCACHE is defined for dialects that have a private method for
+ * printing cached NAME column values for some files.  HASPRIVNAMECACHE
+ * is defined to be the name of the function.
+ *
+ * The function takes one argument, a struct lfile pointer to the file, and
+ * returns non-zero if it prints a name to stdout.
+ */
+
+/* #define	HASPRIVNMCACHE	<function name>	*/
+
+
 /*
  * HASPROCFS is defined for those dialects that have a proc file system --
  * usually /proc and usually in SYSV4 derivatives.
@@ -225,33 +321,23 @@
  * definition and the providing of an fstype element in the local mounts
  * structure (defined in dlsof.h).
  *
- * The HASPROCFS definition may be accompanied by the HASPINFO definition.
- * HASPINFO names the subdirectory of HASPROCFS that contains read-only
- * information (typically /pinfo).
+ * The HASPROCFS definition may be accompanied by the HASPINODEN definition.
+ * HASPINODEN specifies that searching for files in HASPROCFS is to be done
+ * by inode number.
  */
 
-/* #define		HASPROCFS	"proc" */
-/* #define		HASFSTYPE	1 */
-/* #define		HASPINFO	"/pinfo" */
-
-
-/*
- * HASPWSTAYOPEN is defined for those dialects that have a method to hold
- * the password data base open between searches.  Setting this definition
- * implies the availability of a dialect-specific pw_stay_open() function
- * to implement the method.
- */
-
-/* #define	HASPWSTAYOPEN	1 */
+/* #define	HASPROCFS	"proc" */
+/* #define	HASFSTYPE	1 */
+/* #define	HASPINODEN	1 */
 
 
 /*
  * HASRNODE is defined for those dialects that have rnodes.
  */
 
-#if	defined(HAS_NFS)
+# if	defined(HAS_NFS)
 #define	HASRNODE	1
-#endif	/* defined(HAS_NFS) */
+# endif	/* defined(HAS_NFS) */
 
 
 /*
@@ -278,6 +364,22 @@
 #define	HASSTREAMS	1
 
 
+/*
+ * HASTCPTPIQ is defined for dialects where it is possible to report the
+ * TCP/TPI Recv-Q and Send-Q values produced by netstat.
+ */
+
+#define	HASTCPTPIQ	1
+
+
+/*
+ * HASTCPTPIW is defined for dialects where it is possible to report the
+ * TCP/TPI send and receive window sizes produced by netstat.
+ */
+
+/* #define	HASTCPTPIW	1 */
+
+
 /*
  * HASTMPNODE is defined for those dialects that have tmpnodes.
  */
@@ -300,7 +402,7 @@
  * option's default binary value -- 0 or 1.
  */
 
-#define	HASXOPT		"list count and inode address"
+#define	HASXOPT		"list inode count, addr"
 #define	HASXOPT_VALUE	0
 
 
@@ -312,6 +414,34 @@
 #define	UID_ARG	int
 
 
+/*
+ * Each USE_LIB_<function_name> is defined for dialects that use the
+ * <function_name> in the lsof library.
+ *
+ * Note: other definitions and operations may be required to condition the
+ * library function source code.  They may be found in the dialect dlsof.h
+ * header files.
+ */
+
+#define	USE_LIB_CKKV				1	/* ckkv.c */
+#define	USE_LIB_COMPLETEVFS			1	/* cvfs.c */
+#define	USE_LIB_FIND_CH_INO			1	/* fino.c */
+#define	USE_LIB_IS_FILE_NAMED			1	/* isfn.c */
+#define	USE_LIB_LKUPDEV				1	/* lkud.c */
+/* #define	USE_LIB_PRINTCHDEVNAME		1	   pdvn.c */
+/* #define	USE_LIB_PROCESS_FILE		1	   prfp.c */
+/* #define	USE_LIB_PRINT_TCPTPI		1	   ptti.c */
+#define	USE_LIB_READDEV				1	/* rdev.c */
+#define	USE_LIB_READMNT				1	/* rmnt.c */
+/* #define	USE_LIB_RNAM			1	   rnam.c */
+
+#if	!defined(HASVXFSDNLC)
+#define	USE_LIB_RNCH				1	/* rnch.c */
+#endif	/* !defined(HASVXFSDNLC) */
+
+/* #define	USE_LIB_RNMH			1	   rnmh.c */
+
+
 /*
  * WARNDEVACCESS is defined for those dialects that should issue a warning
  * when lsof can't access /dev (or /device) or one of its sub-directories.
@@ -321,6 +451,14 @@
 #define	WARNDEVACCESS	1
 
 
+/*
+ * WARNINGSTATE is defined for those dialects that want to suppress all lsof
+ * warning messages.
+ */
+
+/* #define	WARNINGSTATE	1	warnings are enabled by default */
+
+
 /*
  * WILLDROPGID is defined for those dialects whose lsof executable runs
  * setgid(not_real_GID) and whose setgid power can be relinquished after
@@ -336,3 +474,5 @@
 
 /* #define	zeromem(a, l)		bzero(a, l) */
 #define	zeromem(a, l)		memset(a, 0, l)
+
+#endif	/* !defined(MACHINE_H) */
