@@ -32,12 +32,18 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.10 98/03/06 08:28:24 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.13 98/09/14 13:39:50 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 #include <sys/fs/namenode.h>
+
+#if	defined(RELIANTV) && RELIANTV>=54400
+#undef	fs_bsize
+#include <sys/fs/ufs_fs.h>
+#endif	/* defined(RELIANTV) && RELIANTV>=54400 */
+
 #include <sys/fs/ufs_inode.h>
 #include <sys/fs/xnamnode.h>
 
@@ -273,7 +279,6 @@ process_node(na)
 	char *ty;
 	enum vtype type;
 	struct sockaddr_un ua;
-	struct l_vfs *lvfs;
 
 #if	defined(HASPROCFS)
 	struct as as;
@@ -342,7 +347,7 @@ process_node(na)
 	} else
 	    fx = Fsinfomax;
 	if (Ntype == N_REGLR) {
-	    if (v.v_type == VFIFO)
+	    if (v.v_type == VFIFO && !v.v_stream)
 		Ntype = N_FIFO;
 	    else if (v.v_stream) {
 		Ntype = N_STREAM;
@@ -353,22 +358,6 @@ process_node(na)
  * Determine the lock state.
  */
 	Lf->lock = isvlocked(&v);
-/*
- * Establish the local virtual file system structure.
- */
-	if (!v.v_vfsp)
-	    lvfs = (struct l_vfs *)NULL;
-	else {
-	    if (!(lvfs = readvfs((KA_T)v.v_vfsp,
-				 kvs ? &kv : (struct vfs *)NULL)))
-	    {
-		(void) sprintf(Namech, "vnode@%s; can't read vfs@%s",
-		    print_kptr(na, tbuf),
-		    print_kptr((KA_T)v.v_vfsp, (char *)NULL));
-		enter_nm(Namech);
-		return;
-	    }
-	}
 /*
  * Read the fifonode, inode, pipenode, prnode, rnode, or snode.
  */
@@ -624,7 +613,7 @@ process_node(na)
 			enter_nm(Namech);
 			return;
 		    }
-		    ni = 1;
+		    Lf->is_com = ni = 1;
 		} else
 		    ni = 1;
 		break;
@@ -672,8 +661,22 @@ process_node(na)
 
 	case N_STREAM:
 	    if (sqp) {
-		dev = so.lux_dev.addr.tu_addr.dev;
-		ds = 1;
+		if (so.lux_dev.size >= 8) {
+		    dev = so.lux_dev.addr.tu_addr.dev;
+		    ds = 1;
+		} else
+		    enter_dev_ch(print_kptr(sqp, (char *)NULL));
+		break;
+	    }
+	    if (v.v_type == VFIFO) {
+		KA_T ta;
+
+		if (v.v_data)
+		    ta = (KA_T)v.v_data;
+		else
+		    ta = (KA_T)v.v_stream;
+		if (ta)
+		    enter_dev_ch(print_kptr(ta, (char *)NULL));
 		break;
 	    }
 	    /* fall through */
@@ -687,8 +690,6 @@ process_node(na)
 	    }
 	}
 	type = v.v_type;
-	if (lvfs && ds && !lvfs->dir)
-	    (void) completevfs(lvfs, &dev);
 /*
  * Obtain the inode number.
  */
@@ -734,7 +735,7 @@ process_node(na)
 	    }
 	    break;
 	case N_STREAM:
-	    if (sqp) {
+	    if (sqp && so.lux_dev.size >= 8) {
 		Lf->inode = (unsigned long)so.lux_dev.addr.tu_addr.ino;
 		Lf->inp_ty = 1;
 	    }
@@ -787,12 +788,9 @@ process_node(na)
 	if (Ntype == N_NFS && Fnfs)
 	    Lf->sf |= SELNFS;
 /*
- * Save the file system names.
+ * Defer file system info lookup until printname().
  */
-	if (lvfs) {
-	    Lf->fsdir = lvfs->dir;
-	    Lf->fsdev = lvfs->fsname;
-	}
+	Lf->lmi_srch = 1;
 /*
  * Format the vnode type, and possibly the device name.
  */
@@ -899,7 +897,14 @@ process_node(na)
 	    &&  !kread((KA_T)so.laddr.buf, (char *)&ua, sizeof(ua))) {
 		ua.sun_path[sizeof(ua.sun_path) - 1] = '\0';
 		(void) strcpy(Namech, ua.sun_path);
+		if (so.lux_dev.size >= 8) {
+		    Lf->inode = (unsigned long)so.lux_dev.addr.tu_addr.ino;
+		    Lf->inp_ty = 1;
+		}
 	    }
+	    if (!Namech[0] && so.so_conn)
+		(void) sprintf(Namech, "->%s",
+		    print_kptr((KA_T)so.so_conn, (char *)NULL));
 	    Lf->is_stream = 0;
 	}
 /*
