@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: ddev.c,v 1.7 98/03/10 07:26:30 abe Exp $";
+static char *rcsid = "$Id: ddev.c,v 1.9 98/08/03 07:35:53 abe Exp $";
 #endif
 
 
@@ -270,7 +270,7 @@ read_clone()
 	 /*
 	  * Create a clone structure entry for "clone*:" devices.
 	  *
-	  * Make special note of network clones -- icmp, ip, tcp, and udp.
+	  * Make special note of network clones -- tcp, and udp.
 	  */
 	    if (strncmp(&fp[pl], "clone", 5) == 0) {
 		if (!(cn = strrchr(&fp[pl], ':')))
@@ -302,8 +302,7 @@ read_clone()
 	     * Make special note of a network clone device.
 	     */
 		cn++;
-		if (strcmp(cn, "icmp") == 0 || strcmp(cn, "ip")  == 0
-		||  strcmp(cn, "tcp")  == 0 || strcmp(cn, "udp") == 0)
+		if (strcmp(cn, "tcp")  == 0 || strcmp(cn, "udp") == 0)
 		    c->n = cn - fp;
 		else
 		    c->n = 0;
@@ -526,8 +525,8 @@ readdev(skip)
 			if (!(c = (struct clone *)malloc(sizeof(struct clone))))
 			{
 			    (void) fprintf(stderr,
-				"%s: no space for clone device: %s\n",
-				Pn, path);
+				"%s: no space for clone device: ", Pn);
+			    safestrprt(fp, stderr, 1);
 			    Exit(1);
 			}
 		    /*
