@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.8 98/03/06 08:38:27 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.16 98/07/14 12:33:05 abe Exp $";
 #endif
 
 
@@ -44,16 +44,26 @@ static char *rcsid = "$Id: dnode.c,v 1.8 98/03/06 08:38:27 abe Exp $";
 #endif	/* defined(solaris) */
 
 
-_PROTOTYPE(static char *finddev,(dev_t *dev, int stream));
+_PROTOTYPE(static struct l_dev *finddev,(dev_t *dev, int flags));
+
+/*
+ * Finddev() "look-in " flags
+ */
+
+#define	LOOKDEV_TAB	1		/* look in device table */
+#define	LOOKDEV_CLONE	2		/* look in Clone table */
+#define	LOOKDEV_PSEUDO	4		/* look in Pseudo table */
+#define	LOOKDEV_ALL	(LOOKDEV_TAB | LOOKDEV_CLONE | LOOKDEV_PSEUDO)
+					/* look all places */
 
 #if	defined(solaris)
 _PROTOTYPE(static char isvlocked,(struct vnode *va));
 _PROTOTYPE(static int readinode,(KA_T ia, struct inode *i));
 #else	/* !defined(solaris) */
-_PROTOTYPE(static char isvlocked,(struct vnode *va, caddr_t kv));
+_PROTOTYPE(static char isvlocked,(struct vnode *va, KA_T kv));
 #endif	/* defined(solaris) */
 
-_PROTOTYPE(static void read_mi,(KA_T s, dev_t *dev, caddr_t so, int *so_st));
+_PROTOTYPE(static void read_mi,(KA_T s, dev_t *dev, caddr_t so, int *so_st, KA_T *so_ad, struct l_dev **sdp));
 
 #if	defined(solaris) && solaris>=20500
 # if	solaris>=20600
@@ -63,7 +73,7 @@ _PROTOTYPE(static int read_nusa,(struct soaddr *so, struct sockaddr_un *ua));
 # else	/* solaris<20600 */
 _PROTOTYPE(static int read_nan,(KA_T na, KA_T aa, struct autonode *a));
 # endif	/* solaris>=20600 */
-_PROTOTYPE(static int read_ndn,(KA_T na, KA_T da, struct door_node *d));
+_PROTOTYPE(static int read_ndn,(KA_T na, struct door_node *d));
 #endif	/* defined(solaris) && solaris>=20500 */
 
 _PROTOTYPE(static int read_nfn,(KA_T na, KA_T fa, struct fifonode *f));
@@ -200,14 +210,16 @@ isvlocked(va)
 
 # if	solaris<20500
 	struct filock f;
-	KA_T ff, fp;
+	KA_T ff;
+	KA_T fp;
 # endif	/* solaris<20500 */
 
 	int l;
 
 # if	solaris>=20300
 	struct lock_descriptor ld;
-	KA_T lf, lp;
+	KA_T lf;
+	KA_T lp;
 #  if	solaris<20500
 #define	LOCK_END	ld.info.li_sleep.sli_flock.l_len
 #define	LOCK_FLAGS	ld.flags
@@ -241,7 +253,7 @@ isvlocked(va)
 		if (f.set.l_pid != (pid_t)Lp->pid)
 		    continue;
 		if (f.set.l_whence == 0 && f.set.l_start == 0
-		&&  f.set.l_len == 0x7fffffff)
+		&&  f.set.l_len == MAXEND)
 		    l = 1;
 		else
 		    l = 0;
@@ -270,9 +282,9 @@ isvlocked(va)
 	    &&  (LOCK_END == 0
 
 #  if	solaris<20500
-	    ||   LOCK_END == 0x7fffffff
+	    ||   LOCK_END == MAXEND
 #  else	/* solaris>=20500 */
-	    ||   LOCK_END == 0xffffffff
+	    ||   LOCK_END == MAXEND
 #  endif	/* solaris<20500 */
 
 	    ))
@@ -296,18 +308,23 @@ isvlocked(va)
 }
 #else	/* !defined(solaris) */
 /*
- * isvlocked() -s non-Solaris vnode locked?
+ * isvlocked() is non-Solaris vnode locked?
  */
 
 static char
 isvlocked(va, kv)
 	struct vnode *va;		/* local vnode address */
-	caddr_t kv;			/* vnode address in kernel memory */
+	KA_T kv;			/* vnode address in kernel memory */
 {
+	KA_T dlf;
+	KA_T dlp;
 	int i, l;
 	struct lock_list ll;
-	KA_T dlf, dlp, plf, plp, prf, prp;
+	KA_T plf;
+	KA_T plp;
 	struct data_lock dl, prdl;
+	KA_T prf;
+	KA_T prp;
 	struct process_locks pl;
 
 	if (!va->v_filocks
@@ -347,8 +364,7 @@ isvlocked(va, kv)
 		     */
 			if (kread(prf, (char *)&prdl, sizeof(prdl)))
 			    break;
-			if (prdl.pid != (int)Lp->pid
-			||  prdl.vp != (struct vnode *)kv)
+			if (prdl.pid != (int)Lp->pid || (KA_T)prdl.vp != kv)
 			    continue;
 			if (prdl.base == 0
 			&&  (prdl.length == 0 || prdl.length == -1))
@@ -372,10 +388,11 @@ isvlocked(va, kv)
  * finddev() - look up device by device number
  */
 
-static char *
-finddev(dev, stream)
+static struct l_dev *
+finddev(dev, flags)
 	dev_t *dev;			/* device */
-	int stream;			/* stream if 1 */
+	int flags;			/* look flags -- see LOOKDEV_* symbol
+					 * definitions */
 {
 	struct clone *c;
 	struct l_dev *dp;
@@ -385,7 +402,7 @@ finddev(dev, stream)
 #endif	/* defined(solaris) */
 
 	if (!Sdev)
-		readdev(0);
+	    readdev(0);
 /*
  * Search device table for match.
  */
@@ -396,12 +413,14 @@ finddev_again:
 
 #endif	/* defined(HASDCACHE) */
 
-	if ((dp = lkupdev(dev, 0, 0)))
-	    return(dp->name);
+	if (flags & LOOKDEV_TAB) {
+	    if ((dp = lkupdev(dev, 0, 0)))
+		return(dp);
+	}
 /*
  * Search for clone.
  */
-	if (stream && Clone) {
+	if ((flags & LOOKDEV_CLONE) && Clone) {
 	    for (c = Clone; c; c = c->next) {
 		if (major(*dev) == minor(c->cd.rdev)) {
 
@@ -410,7 +429,7 @@ finddev_again:
 			goto finddev_again;
 #endif	/* defined(HASDCACHE) */
 
-		    return(c->cd.name);
+		    return(&c->cd);
 		}
 	    }
 	}
@@ -419,20 +438,22 @@ finddev_again:
 /*
  * Search for pseudo device match on major device only.
  */
-	for (p = Pseudo; p; p = p->next) {
-	    if (major(*dev) == major(p->pd.rdev)) {
+	if ((flags & LOOKDEV_PSEUDO) && Pseudo) {
+	    for (p = Pseudo; p; p = p->next) {
+		if (major(*dev) == major(p->pd.rdev)) {
 
 #if	defined(HASDCACHE)
-		if (DCunsafe && !p->pd.v && !vfy_dev(&p->pd))
-		    goto finddev_again;
+		    if (DCunsafe && !p->pd.v && !vfy_dev(&p->pd))
+			goto finddev_again;
 #endif	/* defined(HASDCACHE) */
 
-		return(p->pd.name);
+		    return(&p->pd);
+		}
 	    }
 	}
 #endif	/* defined(solaris) */
 
-	return((char *)NULL);
+	return((struct l_dev *)NULL);
 }
 
 
@@ -444,7 +465,7 @@ void
 process_node(va)
 	KA_T va;			/* vnode kernel space address */
 {
-	dev_t dev;
+	dev_t dev, tdev;
 	unsigned char dev_def = 1;
 	struct fifonode f;
 	static int ft = 1;
@@ -460,9 +481,12 @@ process_node(va)
 	struct vnode rv;
 	int rvs = 0;
 	struct snode s;
+	struct l_dev *sdp = (struct l_dev *)NULL;
+	KA_T so_ad[2];
 	int so_st = 0;
 	struct tmpnode t;
-	char tbuf[32], *ty;
+	char tbuf[32], *ty, ubuf[128];
+	int tdevs = 0;
 	enum vtype type;
 	struct sockaddr_un ua;
 	int unix_sock = 0;
@@ -473,21 +497,21 @@ process_node(va)
 	int  vty_tmp;
 
 #if	defined(solaris)
+	int len, nl, snl, sepl;
 # if	solaris>=20500
 #  if	solaris>=20600
 	struct fnnode fnn;
-	int len, nl;
 	struct pairaddr {
 	    short f;
 	    unsigned short p;
 	} *pa;
 	unsigned int peer;
 	struct sonode so;
+	KA_T soa;
 #  else	/* solaris<20600 */
 	struct autonode au;
 #  endif	/* solaris>=20600 */
 
-	KA_T da = (KA_T)NULL;
 	struct door_node dn;
 # endif	/* solaris >=20500 */
 
@@ -524,11 +548,14 @@ process_node(va)
 /*
  * Do first-time only operations.
  */
+	so_ad[0] = so_ad[1] = (KA_T)0;
 	if (ft) {
 	    if (get_Nl_value("auvops", Drive_Nl, &Auvops) < 0)
 		Auvops = (KA_T)0;
-	    if (get_Nl_value("avops", Drive_Nl, &Avops) < 0)
-		Avops = (KA_T)0;
+	    if (get_Nl_value("avops", Drive_Nl, &Avops) < 0 || !Avops) {
+		if (get_Nl_value("Avops", Drive_Nl, &Avops) < 0)
+		    Avops = (KA_T)0;
+	    }
 	    if (get_Nl_value("cvops", Drive_Nl, &Cvops) < 0)
 		Cvops = (KA_T)0;
 	    if (get_Nl_value("dvops", Drive_Nl, &Dvops) < 0)
@@ -737,7 +764,7 @@ vfs_read_error:
 	    vty = N_REGLR;
 	    if (v->v_type == VFIFO)
 		Ntype = N_FIFO;
-	    else if (v->v_type == VCHR && !v->v_vfsp && vs) {
+	    else if (v->v_type == VCHR && vs) {
 		Ntype = N_STREAM;
 		Lf->is_stream = 1;
 	    }
@@ -804,7 +831,7 @@ vfs_read_error:
 	case N_SPEC:
 	
 	/*
-	 * A N_SPEC node is node that resides in in an underlying file system
+	 * A N_SPEC node is a node that resides in in an underlying file system
 	 * type -- e.g. NFS, HSFS.  Its vnode points to an snode.  Subsequent
 	 * node structures are implied by the underlying node type.
 	 */
@@ -829,13 +856,15 @@ vfs_read_error:
 	    if (!realvp) {
 
 	    /*
-	     * If the snode lacks a real vnode (and also lacks a
-	     * Solaris common vnode), it's original type is N_STREAM,
-	     * and it has a stream pointer, get the module names.
+	     * If the snode lacks a real vnode (and also lacks a common vnode),
+	     * it's original type is N_STREAM or N_REGLR, and it has a stream
+	     * pointer, get the module names.
 	     */
-		if (vty == N_STREAM && vs) {
-		    (void) strcpy(Lf->iproto, "STR");
-		    read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st);
+		if ((vty == N_STREAM || vty == N_REGLR) && vs) {
+		    Lf->is_stream = 1;
+		    vty = N_STREAM;
+		    read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st,
+			    so_ad, &sdp);
 		    vs = (KA_T)NULL;
 		}
 	    }
@@ -861,9 +890,8 @@ vfs_read_error:
 		return;
 	    break;
 	case N_DOOR:
-	    if (read_ndn(va, (KA_T)v->v_data, &dn))
+	    if (read_ndn(va, &dn))
 		return;
-	    da = va;
 	    break;
 # endif	/* solaris>=20500 */
 
@@ -1002,8 +1030,9 @@ vfs_read_error:
 	    if (read_nsn(va, (KA_T)v->v_data, &s))
 		return;
 	    if (vs) {
-		(void) strcpy(Lf->iproto, "STR");
-		read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st);
+		Lf->is_stream = 1;
+		read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st, so_ad,
+		    &sdp);
 		vs = (KA_T)NULL;
 	    }
 	    break;
@@ -1036,8 +1065,9 @@ vfs_read_error:
 	 * a stream pointer, get the module names.
 	 */
 	    if (vty == N_STREAM && vs) {
-		(void) strcpy(Lf->iproto, "STR");
-		read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st);
+		Lf->is_stream = 1;
+		read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st, so_ad,
+		    &sdp);
 		vs = (KA_T)NULL;
 	    }
 	/*
@@ -1087,9 +1117,14 @@ vfs_read_error:
 		    return;
 		break;
 	    case N_DOOR:
-		if (read_ndn(va, (KA_T)v->v_data, &dn))
+
+#  if	solaris<20600
+		if (read_ndn(realvp, &dn))
+#  else	/* solaris>=20600 */
+		if (read_ndn(va, &dn))
+#  endif	/* solaris<20500 */
+
 		    return;
-		da = va;
 		break;
 # endif	/* solaris>=20500 */
 	    case N_CACHE:
@@ -1132,8 +1167,9 @@ vfs_read_error:
 		break;
 	    case N_STREAM:
 		if (vs) {
-		    (void) strcpy(Lf->iproto, "STR");
-		    read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st);
+		    Lf->is_stream = 1;
+		    read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st,
+			    so_ad, &sdp);
 		    vs = (KA_T)NULL;
 		}
 		break;
@@ -1182,14 +1218,22 @@ vfs_read_error:
 # if	solaris>=20500
 	case N_AUTO:
 	    if (kvs)
-		dev = kv.vfs_fsid.val[0];
+		dev = (dev_t)kv.vfs_fsid.val[0];
 	    else
 		dev_def = 0;
 	    break;
 	case N_DOOR:
-	    dev_def = 0;
-	    if (da)
-		enter_dev_ch(print_kptr((KA_T)da, (char *)NULL));
+
+#  if	solaris<20600
+	    if (kvs) {
+		dev = (dev_t)kv.vfs_fsid.val[0];
+		dev_def = 1;
+	    }
+#  else	/* solaris>=20600 */
+	    dev = (dev_t)dn.door_index;
+	    dev_def = 1;
+#  endif	/* solaris<20600 */
+
 	    break;
 # endif	/* solaris>=20500 */
 	case N_CACHE:
@@ -1208,37 +1252,54 @@ vfs_read_error:
 	     * If this is an AF_UNIX socket node:
 	     *
 	     *    Enter the sonode address as the device (netstat's local
-	     *	 address);
+	     *	  address);
 	     *    Get a non-NULL local sockaddr_un and enter it in Namech;
 	     *    Get a non-NULL foreign sockaddr_un and enter it in Namech;
 	     *    Check for matches on sockaddr_un.sun_path names.
 	     */
-		enter_dev_ch(print_kptr((KA_T)v->v_data, (char *)NULL));
-		nl = 0;
+		if (!sdp)
+		    sdp = finddev(&so.so_vnode.v_rdev, LOOKDEV_ALL);
+		if (sdp) {
+		    dev = so.so_vnode.v_rdev;
+		    tdev = sdp->rdev;
+		    tdevs = 1;
+		    Lf->inode = (unsigned long)sdp->inode;
+		    Lf->inp_ty = 1;
+		    (void) strcpy(Namech, sdp->name);
+		} else {
+		    dev_def = 0;
+		    Namech[0] = '\0';
+		}
+		nl = snl = strlen(Namech);
 		if ((len = read_nusa(&so.so_laddr, &ua))) {
-		    if (len > (MAXPATHLEN - 2 - 1)) {
-			len = MAXPATHLEN - 2 - 1;
-			ua.sun_path[len] = '\0';
-		    }
 		    if (Sfile
 		    &&  is_file_named(ua.sun_path, Ntype, VSOCK, 0))
 			Lf->sf |= SELNM;
-		    (void) strcpy(Namech, ua.sun_path);
-		    nl = len;
+		    sepl = Namech[0] ? 2 : 0;
+		    if (len > (MAXPATHLEN - nl - sepl - 1))
+			len = MAXPATHLEN - nl - sepl - 1;
+		    if (len > 0) {
+			ua.sun_path[len] = '\0';
+			(void) sprintf(&Namech[nl], "%s%s", sepl ? "->" : "",
+			    ua.sun_path);
+			nl += (len + sepl);
+		    }
 		}
 		if ((len = read_nusa(&so.so_faddr, &ua))) {
-		    (void) strcpy(&Namech[nl], "->");
-		    nl += 2;
-		    if (len > (MAXPATHLEN - nl - 1))
-			len = MAXPATHLEN -  nl - 1;
-		    ua.sun_path[len] = 0;
 		    if (Sfile
 		    &&  is_file_named(ua.sun_path, Ntype, VSOCK, 0))
 			Lf->sf |= SELNM;
-		    (void) strcpy(&Namech[nl], ua.sun_path);
-		    nl += len;
+		    sepl = Namech[0] ? 2 : 0;
+		    if (len > (MAXPATHLEN - nl - sepl - 1))
+			len = MAXPATHLEN - nl - sepl - 1;
+		    if (len > 0) {
+			ua.sun_path[len] = 0;
+			(void) sprintf(&Namech[nl], "%s%s", sepl ? "->" : "",
+			    ua.sun_path);
+			nl += (len + sepl);
+		    }
 		}
-		if (!nl
+		if (nl == snl
 		&&  so.so_ux_laddr.sou_magic == SOU_MAGIC_IMPLICIT) {
 
 		/*
@@ -1249,9 +1310,44 @@ vfs_read_error:
 		    if (!(peer = (unsigned int)pa->p))
 			peer = (unsigned int)so.so_ux_laddr.sou_vp & 0xffff;
 		    if (peer)
-			(void) sprintf(Namech, "(socketpair: 0x%x)", peer);
+			(void) sprintf(ubuf, "(socketpair: 0x%x)", peer);
 		    else
-			(void) strcpy(Namech, "(socketpair)");
+			(void) strcpy(ubuf, "(socketpair)");
+		    len = strlen(ubuf);
+		    sepl = Namech[0] ? 2 : 0;
+		    if (len > (MAXPATHLEN - nl - sepl - 1))
+			len = MAXPATHLEN - nl - sepl - 1;
+		    if (len > 0) {
+			(void) sprintf(&Namech[nl], "%s%s", sepl ? "->" : "",
+			    ubuf);
+			nl += (len + sepl);
+		    }
+		}
+	    /*
+	     * Add the local and foreign addresses, ala `netstat -f unix` to
+	     * the name.
+	     */
+		soa = (KA_T)so.so_ux_faddr.sou_vp;
+		(void) sprintf(ubuf, "%s(%s%s%s)", Namech[0] ? " " : "",
+		    print_kptr((KA_T)v->v_data, (char *)NULL),
+		    soa ? "->" : "",
+		    soa ? print_kptr(soa, tbuf) : "");
+		len = strlen(ubuf);
+		if (len <= (MAXPATHLEN - nl - 1)) {
+		    (void) strcpy(&Namech[nl], ubuf);
+		    nl += len;
+		}
+	    /*
+	     * If there is a bound vnode, add its address to the name.
+	     */
+		if (so.so_ux_bound_vp) {
+		    (void) sprintf(ubuf, "%s(Vnode=%s)", Namech[0] ? " " : "",
+			print_kptr((KA_T)so.so_ux_bound_vp, (char *)NULL));
+		    len = strlen(ubuf);
+		    if (len <= (MAXPATHLEN - nl - 1)) {
+			(void) strcpy(&Namech[nl], ubuf);
+			nl += len;
+		    }
 		}
 	    } else
 		dev_def = 0;
@@ -1279,7 +1375,12 @@ vfs_read_error:
 
 #if	defined(solaris)
 	case N_NM:
-	    enter_dev_ch("    NMFS");
+	    if (nns)
+		dev = (dev_t)nn.nm_vattr.va_fsid;
+	    else {
+		enter_dev_ch("    NMFS");
+		dev_def = 0;
+	    }
 	    break;
 #endif	/* defined(solaris) */
 
@@ -1299,7 +1400,25 @@ vfs_read_error:
 		if (Funix)
 		    Lf->sf |= SELUNX;
 		unix_sock = 1;
-		if (soso.lux_dev.addr.tu_addr.ino)
+		if (so_ad[0]) {
+		    if (sdp) {
+			dev = sdp->rdev;
+			Lf->inode = (unsigned long)sdp->inode;
+			Lf->inp_ty = 1;
+			(void) sprintf(ubuf, "(%s%s%s)",
+			    print_kptr(so_ad[0], (char *)NULL),
+			    so_ad[1] ? "->" : "",
+			    so_ad[1] ? print_kptr(so_ad[1], tbuf) : "");
+		    } else {
+			dev_def = 0;
+			enter_dev_ch(print_kptr(so_ad[0], (char *)NULL));
+			if (so_ad[1])
+			    (void) sprintf(ubuf, "(->%s)",
+				print_kptr(so_ad[1], (char *)NULL));
+		    }
+		    if (!Lf->nma && (Lf->nma = (char *)malloc(strlen(ubuf)+1)))
+			(void) strcpy(Lf->nma, ubuf);
+		} else if (soso.lux_dev.addr.tu_addr.ino)
 		    dev = soso.lux_dev.addr.tu_addr.dev;
 		else {
 		    int dc, dl, dr;
@@ -1320,8 +1439,21 @@ vfs_read_error:
 		    if (kread((KA_T)soso.laddr.buf, (char *)&ua, sizeof(ua))
 		    == 0) {
 			ua.sun_path[sizeof(ua.sun_path) - 1] = '\0';
-			if (ua.sun_path[0])
-			    (void) strcpy(Namech, ua.sun_path);
+			if (ua.sun_path[0]) {
+			    if (Sfile
+			    &&  is_file_named(ua.sun_path, Ntype, type, 0))
+				Lf->sf |= SELNM;
+			    len = strlen(ua.sun_path);
+			    nl = strlen(Namech);
+			    sepl = Namech[0] ? 2 : 0;
+			    if (len > (MAXPATHLEN - nl - sepl - 1))
+				len = MAXPATHLEN - nl - sepl - 1;
+			    if (len > 0) {
+				ua.sun_path[len] = '\0';
+				(void) sprintf(&Namech[nl], "%s%s",
+				sepl ? "->" : "", ua.sun_path);
+			    }
+			}
 		    }
 		}
 	    } else
@@ -1387,15 +1519,20 @@ vfs_read_error:
 
 	    Lf->inp_ty = 1;
 	    break;
+	case N_DOOR:
+	    if (nns && (Lf->inode = (unsigned long)nn.nm_vattr.va_nodeid)) {
+		Lf->inp_ty = 1;
+		break;
+	    }
+	    if ((Lf->inode = (unsigned long)dn.door_active)) 
+		Lf->inp_ty = 1;
+	    break;
 # endif	/* solaris>=20500 */
+
 	case N_CACHE:
 	    Lf->inode = (unsigned long)cn.c_fileno;
 	    Lf->inp_ty = 1;
 	    break;
-# if	solaris>=20600
-	case N_SOCK:
-	    break;
-# endif	/* solaris>=20600 */
 #endif	/* defined(solaris) */
 
 	case N_HSFS:
@@ -1440,10 +1577,23 @@ vfs_read_error:
 #if	defined(solaris)
 	    if (kvs && kv.vfs_data
 	    && !kread((KA_T)kv.vfs_data, (char *)&pcfs, sizeof(pcfs))) {
+
+# if	solaris>=20700
+		Lf->inode = (unsigned long)pc_makenodeid(pc.pc_eblkno,
+			    pc.pc_eoffset,
+			    pc.pc_entry.pcd_attr,
+			    IS_FAT32(&pcfs)
+				? ltohs(pc.pc_entry.pcd_scluster_lo) |
+				  (ltohs(pc.pc_entry.un.pcd_scluster_hi) << 16)
+				: ltohs(pc.pc_entry.pcd_scluster_lo),
+			    pcfs.pcfs_entps);
+# else	/* solaris<20700 */
 		Lf->inode = (unsigned long)pc_makenodeid(pc.pc_eblkno,
 			    pc.pc_eoffset,
 			    &pc.pc_entry,
 			    pcfs.pcfs_entps);
+# endif	/* solaris>=20700 */
+
 		Lf->inp_ty = 1;
 	    }
 #else	/* !defined(solaris) */
@@ -1465,8 +1615,21 @@ vfs_read_error:
 #if	defined(solaris)
 	case N_STREAM:
 	    if (so_st && soso.lux_dev.addr.tu_addr.ino) {
-		Lf->inode = (unsigned long)soso.lux_dev.addr.tu_addr.ino;
-		Lf->inp_ty = 1;
+		if (Lf->inp_ty) {
+		    nl = Lf->nma ? strlen(Lf->nma) : 0;
+		    (void) sprintf(ubuf, "%s(Vnode=%lu)", nl ? " " : "",
+			(unsigned long)soso.lux_dev.addr.tu_addr.ino);
+		    len = nl + strlen(ubuf) + 1;
+		    if (Lf->nma)
+			Lf->nma = (char *)realloc(Lf->nma, len);
+		    else
+			Lf->nma = (char *)malloc(len);
+		    if (Lf->nma)
+			(void) strcpy(&Lf->nma[nl], ubuf);
+		} else {
+		    Lf->inode = (unsigned long)soso.lux_dev.addr.tu_addr.ino;
+		    Lf->inp_ty = 1;
+		}
 	    }
 	    break;
 #endif	/* defined(solaris) */
@@ -1537,6 +1700,7 @@ vfs_read_error:
 		break;
 #endif	/* defined(solaris) */
 
+	    case N_DOOR:
 	    case N_FIFO:
 		if (!Fsize)
 		    Lf->off_def = 1;
@@ -1636,10 +1800,15 @@ vfs_read_error:
 /*
  * Save the file system names.
  */
-	if (vfs && Ntype != N_NM && Ntype != N_DOOR) {
+	if (vfs) {
 	    Lf->fsdir = vfs->dir;
 	    Lf->fsdev = vfs->fsname;
 
+#if	defined(solaris)
+	    if (!Lf->fsdir && !Lf->fsdev && kvs)
+		Lf->fsdev = Fsinfo[kv.vfs_fstype - 1];
+#endif	/* defined(solaris) */
+
 #if	defined(HASFSINO)
 	    Lf->fs_ino = vfs->fs_ino;
 #endif	/* defined(HASFSINO) */
@@ -1725,9 +1894,10 @@ vfs_read_error:
 		ty = "sock";
 # else	/* !defined(solaris) || solaris<20600 */
 	    ty = "sock";
+# endif	/* defined(solaris) && solaris>=20600 */
+
 	    Lf->dev = dev;
 	    Lf->dev_def = dev_def;
-# endif	/* defined(solaris) && solaris>=20600 */
 	    break;
 #endif	/* defined(HAS_VSOCK) */
 
@@ -1774,8 +1944,51 @@ vfs_read_error:
  * If this is a VCHR file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VCHR && Lf->dev_def)
+	if (Lf->inp_ty == 0 && type == VCHR && Lf->dev_def) {
 	    find_ch_ino();
+
+#if	defined(solaris)
+	/*
+	 * If the VCHR inode number still isn't known and this is a
+	 * COMMON vnode file or a stream, see if an inode number can
+	 * be derived from a pseudo or clone file.
+	 *
+	 * If it can, save the pseudo or clone device for temporary
+	 * use when searching for a match with a named file argument.
+	 */
+	    if (Lf->inp_ty == 0 && (Lf->is_com || Lf->is_stream)
+	    &&  (Clone || Pseudo))
+	    {
+		if (!sdp) {
+		    if (Lf->is_stream) {
+			if (!(sdp = finddev(&Lf->dev, LOOKDEV_CLONE)))
+			    sdp = finddev(&Lf->dev, LOOKDEV_ALL);
+		    } else {
+			if (!(sdp = finddev(&Lf->dev, LOOKDEV_PSEUDO)))
+			    sdp = finddev(&Lf->dev, LOOKDEV_ALL);
+		    }
+		}
+		if (sdp) {
+		    tdev = sdp->rdev;
+		    Lf->inode = sdp->inode;
+		    Lf->inp_ty = tdevs = 1;
+		    if (!Namech[0] || Lf->is_com)
+			(void) strcpy(Namech, sdp->name);
+		    if (Lf->is_com && !Lf->nma) {
+			if (!(Lf->nma = (char *)malloc(strlen("(COMMON)")+1)))
+			{
+			    (void) fprintf(stderr,
+				"%s: no space for (COMMON): PID %d; FD %s\n",
+				Pn, Lp->pid, Lf->fd);
+			    Exit(1);
+			}
+			(void) strcpy(Lf->nma, "(COMMON)");
+		    }
+		}
+	    }
+#endif	/* defined(solaris) */
+
+	}
 /*
  * Record stream status.
  */
@@ -1813,8 +2026,16 @@ vfs_read_error:
 #endif	/* defined(HASPROCFS) */
 
 	{
-	    if (Sfile && is_file_named(NULL, Ntype, type, 1))
-		Lf->sf |= SELNM;
+	    if (Sfile) {
+		if (tdevs) {
+		    dev = Lf->dev;
+		    Lf->dev = tdev;
+		}
+		if (is_file_named(NULL, Ntype, type, 1))
+		    Lf->sf |= SELNM;
+		if (tdevs)
+		    Lf->dev = dev;
+	    }
 	}
 /*
  * Enter name characters.
@@ -1889,16 +2110,15 @@ readinode(ia, i)
  */
 
 static int
-read_ndn(na, da, dn)
-	KA_T na;			/* containing node's address */
-	KA_T da;			/* door node address */
+read_ndn(na, dn)
+	KA_T na;			/* containing vnode's address */
 	struct door_node *dn;		/* door node receiver */
 {
 	char tbuf[32];
 
-	if (!da || kread((KA_T)da, (char *)dn, sizeof(struct door_node))) {
-	    (void) sprintf(Namech, "node at %s: can't read door_node: %s",
-		print_kptr(na, tbuf), print_kptr(da, (char *)NULL));
+	if (!na || kread((KA_T)na, (char *)dn, sizeof(struct door_node))) {
+	    (void) sprintf(Namech, "vnode at %s: can't read door_node: %s",
+		print_kptr(na, tbuf), print_kptr(na, (char *)NULL));
 	    enter_nm(Namech);
 	    return(1);
 	}
@@ -1914,20 +2134,23 @@ read_ndn(na, da, dn)
  */
 
 static void
-read_mi(s, dev, so, so_st)
+read_mi(s, dev, so, so_st, so_ad, sdp)
 	KA_T s;				/* kernel stream pointer address */
 	dev_t *dev;			/* device pointer */
 	caddr_t so;			/* so_so return (Solaris) */
 	int *so_st;			/* so_so status */
+	KA_T *so_ad;			/* so_so addresses */
+	struct l_dev **sdp;		/* returned device pointer */
 {
-	char *cp;
+	struct l_dev *dp;
 	int i, j, k, nl;
-	KA_T ka, qp;
+	KA_T ka;
 	struct module_info mi;
 	char mn[STRNML];
 	struct stdata sd;
 	struct queue q;
 	struct qinit qi;
+	KA_T qp;
 
 /*
  * If there is no stream pointer, or we can't read the stream head,
@@ -1952,15 +2175,19 @@ read_mi(s, dev, so, so_st)
  */
 	k = 0;
 	Namech[0] = '\0';
-	if ((cp = finddev(dev, 1)) != NULL) {
-	    (void) strcpy(Namech, cp);
+	if (!(dp = finddev(dev, LOOKDEV_CLONE)))
+	    dp = finddev(dev, LOOKDEV_ALL);
+	if (dp) {
+	    (void) strcpy(Namech, dp->name);
 	    k = strlen(Namech);
-	}
+	    *sdp = dp;
+	} else
+	    (void) strcpy(Lf->iproto, "STR");
 	nl = sizeof(mn) - 1;
 	mn[nl] = '\0';
 	qp = (KA_T)sd.sd_wrq;
 	for (i = 0; qp && i < 20; i++, qp = (KA_T)q.q_next) {
-	    if (kread(qp, (char *)&q, sizeof(q)))
+	    if (!qp ||  kread(qp, (char *)&q, sizeof(q)))
 		break;
 	    if ((ka = (KA_T)q.q_qinfo) == (KA_T)NULL
 	    ||  kread(ka, (char *)&qi, sizeof(qi)))
@@ -1986,8 +2213,13 @@ read_mi(s, dev, so, so_st)
 
 		    struct so_so s;
 
-		    if (!kread((KA_T)q.q_ptr, (char *)&s, sizeof(s)))
+		    if (!kread((KA_T)q.q_ptr, (char *)&s, sizeof(s))) {
+			if (!(*so_st))
+			    so_ad[0] = (KA_T)q.q_ptr;
+			else
+			    so_ad[1] = (KA_T)q.q_ptr;
 			(void) savesockmod(&s, (struct so_so *)so, so_st);
+		    }
 		}
 	    }
 #endif	/* defined(solaris) */
@@ -1996,7 +2228,7 @@ read_mi(s, dev, so, so_st)
 		if ((k + 2) > (MAXPATHLEN - 1))
 		    break;
 		(void) strcpy(&Namech[k], "->");
-		    k += 2;
+		k += 2;
 	    }
 	    if ((k + j) > (MAXPATHLEN - 1))
 		break;
@@ -2711,6 +2943,7 @@ savesockmod(so, sop, so_st)
 #define	ruxdev	rux_dev.addr.tu_addr.dev
 #define	ruxino	rux_dev.addr.tu_addr.ino
 
+# if	solaris<20500
 /*
  * If either address in the new structure is missing a device number, clear
  * its corresponding inode number.  Then sort the inode-less device numbers.
@@ -2837,6 +3070,16 @@ savesockmod(so, sop, so_st)
 		    sop->ruxdev = d2;
 	    }
 	}
+# else	/* solaris>=20500 */
+/*
+ * Save the first sockmod structure.
+ */
+	if (!*so_st) {
+	    *so_st = 1;
+	    *sop = *so;
+	}
+# endif	/* solaris<20500 */
+
 }
 #endif	/* defined(solaris) */
 
@@ -2858,39 +3101,39 @@ vop2ty(vp)
 
 	if (!vp->v_op)
 		return(-1);
-	if ((Uvops && Uvops == (unsigned long)vp->v_op)
-	||  (Spvops && Spvops == (unsigned long)vp->v_op))
+	if ((Uvops && Uvops == (KA_T)vp->v_op)
+	||  (Spvops && Spvops == (KA_T)vp->v_op))
 		return(N_REGLR);
-	if (Nvops && Nvops == (unsigned long)vp->v_op)
+	if (Nvops && Nvops == (KA_T)vp->v_op)
 		return(N_NFS);
-	else if (N3vops && N3vops == (unsigned long)vp->v_op)
+	else if (N3vops && N3vops == (KA_T)vp->v_op)
 		return(N_NFS);
-	else if (Vvops && Vvops == (unsigned long)vp->v_op)
+	else if (Vvops && Vvops == (KA_T)vp->v_op)
 		return(N_VXFS);
-	else if (Tvops && Tvops == (unsigned long)vp->v_op)
+	else if (Tvops && Tvops == (KA_T)vp->v_op)
 		return(N_TMP);
-	else if (Auvops && Auvops == (unsigned long)vp->v_op)
+	else if (Auvops && Auvops == (KA_T)vp->v_op)
 		return(N_AUTO);
-	else if (Hvops && Hvops == (unsigned long)vp->v_op)
+	else if (Hvops && Hvops == (KA_T)vp->v_op)
 		return(N_HSFS);
-	else if ((Pdvops && Pdvops == (unsigned long)vp->v_op)
-	     ||  (Pfvops && Pfvops == (unsigned long)vp->v_op))
+	else if ((Pdvops && Pdvops == (KA_T)vp->v_op)
+	     ||  (Pfvops && Pfvops == (KA_T)vp->v_op))
 		return(N_PCFS);
-	else if (Mvops && Mvops == (unsigned long)vp->v_op)
+	else if (Mvops && Mvops == (KA_T)vp->v_op)
 		return(N_MVFS);
-	else if (Cvops && Cvops == (unsigned long)vp->v_op)
+	else if (Cvops && Cvops == (KA_T)vp->v_op)
 		return(N_CACHE);
-	else if (Dvops && Dvops == (unsigned long)vp->v_op)
+	else if (Dvops && Dvops == (KA_T)vp->v_op)
 		return(N_DOOR);
-	else if (Fvops && Fvops == (unsigned long)vp->v_op)
+	else if (Fvops && Fvops == (KA_T)vp->v_op)
 		return(N_FIFO);
-	else if (Lvops && Lvops == (unsigned long)vp->v_op)
+	else if (Lvops && Lvops == (KA_T)vp->v_op)
 		return(N_LOFS);
-	else if (Nmvops && Nmvops == (unsigned long)vp->v_op)
+	else if (Nmvops && Nmvops == (KA_T)vp->v_op)
 		return(N_NM);
-	else if (Prvops && Prvops == (unsigned long)vp->v_op)
+	else if (Prvops && Prvops == (KA_T)vp->v_op)
 		return(N_PROC);
-	else if (Sckvops && Sckvops == (unsigned long)vp->v_op)
+	else if (Sckvops && Sckvops == (KA_T)vp->v_op)
 		return(N_SOCK);
 
 #if	defined(HAS_AFS)
