@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.23 96/04/05 09:49:38 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.10 98/08/18 08:07:56 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -41,17 +41,13 @@ static char *rcsid = "$Id: dproc.c,v 1.23 96/04/05 09:49:38 abe Exp $";
 #include "kernelbase.h"
 #endif	/* defined(solaris) && solaris<20500 */
 
-#if	defined(HASNCACHE)
-#include <sys/dnlc.h>
-#endif	/* defined(HASNCACHE) */
-
 
 /*
  * Local definitions
  */
 
 #if	defined(solaris) && solaris>=20500
-u_long Kb = 0l;				/* KERNELBASE for Solaris 2.5 */
+KA_T Kb = (KA_T)NULL;			/* KERNELBASE for Solaris 2.5 */
 #endif	/* defined(solaris) && solaris>=20500 */
 
 static int Knp = PROCDFLT;		/* numer of proc structures known
@@ -59,6 +55,7 @@ static int Knp = PROCDFLT;		/* numer of proc structures known
 static int Np;				/* number of P[], Pgrp[], and Pid[]
 					 * entries  */
 static struct proc *P = NULL;		/* local proc structure table */
+static KA_T Sgvops;			/* [_]segvn_ops address */
 
 #if	defined(solaris)
 static int *Pid = NULL;			/* PIDs for P[] entries */
@@ -70,13 +67,8 @@ static int Switchgid = 0;		/* must switch GIDs for kvm_open() */
 
 _PROTOTYPE(static void close_kvm,(void));
 _PROTOTYPE(static void get_kernel_access,(void));
-
-#if	defined(HASNCACHE)
-_PROTOTYPE(static void ncache_load,(void));
-#endif	/* defined(HASNCACHE) */
-
 _PROTOTYPE(static void open_kvm,(void));
-_PROTOTYPE(static void process_text,(struct as *pa));
+_PROTOTYPE(static void process_text,(KA_T pa));
 _PROTOTYPE(static void read_proc,(void));
 
 #if	defined(solaris)
@@ -92,13 +84,13 @@ _PROTOTYPE(static void readfsinfo,(void));
 static void
 close_kvm()
 {
-	if (Kd == NULL)
-		return;
+	if (!Kd)
+	    return;
 	if (kvm_close(Kd) != 0) {
-		(void) fprintf(stderr, "%s: kvm_close failed\n", Pn);
-		Exit(1);
+	    (void) fprintf(stderr, "%s: kvm_close failed\n", Pn);
+	    Exit(1);
 	}
-	Kd = NULL;
+	Kd = (kvm_t *)NULL;
 }
 
 
@@ -109,6 +101,7 @@ close_kvm()
 void
 gather_proc_info()
 {
+	int ft = 1;
 	int i, j;
 	struct proc *p;
 	int pgrp, pid, px;
@@ -130,9 +123,17 @@ gather_proc_info()
 	struct file **uf;
 	unsigned ui;
 	static int xnf = 0;
-	static struct file **xuf = NULL;
+	struct file **xuf = (struct file **)NULL;
 #endif	/* solaris */
 
+/*
+ * Do first-time only operations.
+ */
+	if (ft) {
+	    if (get_Nl_value("sgvops", Drive_Nl, &Sgvops) < 0)
+		Sgvops = (KA_T)NULL;
+	    ft = 0;
+	}
 /*
  * Read kernel name cache and process table.
  */
@@ -158,110 +159,122 @@ gather_proc_info()
 	 */
 
 #if	defined(solaris)
-		if (Fpgrp)
-			pgrp = Pgrp[px];
-		else
-			pgrp = 0;
-		pid = Pid[px];
+	    if (Fpgrp)
+		pgrp = Pgrp[px];
+	    else
+		pgrp = 0;
+	    pid = Pid[px];
 # if solaris<20500
-		uid = p->p_uid;
+	    uid = p->p_uid;
 # else	/* solaris >=20500 */
 	/*
 	 * Read credentials for Solaris 2.5 and above process.
 	 */
-		if ( ! p->p_cred
-		||  kread((KA_T)p->p_cred, (char *)&pc, sizeof(pc)))
-			continue;
-		uid = pc.cr_ruid;
+	    if (kread((KA_T)p->p_cred, (char *)&pc, sizeof(pc)))
+		continue;
+	    uid = pc.cr_uid;
 # endif	/* solaris<20500 */
 #else	/* !defined(solaris) */
-		pgrp = (int)p->p_pgrp;
-		pid = p->p_pid;
-		uid = p->p_uid;
+	    pgrp = (int)p->p_pgrp;
+	    pid = p->p_pid;
+	    uid = p->p_suid ? p->p_suid : p->p_uid;
 #endif	/* defined(solaris) */
 
 	/*
 	 * See if the process is excluded.
 	 */
-		if  (is_proc_excl(pid, pgrp, (UID_ARG)uid, &pss, &sf))
-			continue;
+	    if  (is_proc_excl(pid, pgrp, (UID_ARG)uid, &pss, &sf))
+		continue;
 	/*
 	 * Get the user area associated with the process.
 	 */
 
 #if	defined(solaris)
-		u = &p->p_user;
+	    u = &p->p_user;
 #else	/* !defined(solaris) */
-		if ((u = kvm_getu(Kd, p)) == NULL)
-			continue;
+	    if (!(u = kvm_getu(Kd, p)))
+		continue;
 #endif	/* defined(solaris) */
 
 	/*
 	 * Allocate a local process structure and start filling it.
 	 */
-		if (is_cmd_excl(u->u_comm, &pss, &sf))
-			continue;
-		alloc_lproc(pid, pgrp, (UID_ARG)uid, u->u_comm,
-			(int)pss, (int)sf);
-		Plf = NULL;
+	    if (is_cmd_excl(u->u_comm, &pss, &sf))
+		continue;
+	    alloc_lproc(pid, pgrp, (int)p->p_ppid, (UID_ARG)uid, u->u_comm,
+		(int)pss, (int)sf);
+	    Plf = (struct lfile *)NULL;
 
 #if	!defined(solaris)
 	/*
 	 * Read the SunOS current working directory and root directory
 	 * paths.
 	 */
-		*Cwd = *Rtd = '\0';
-		if (u->u_cwd && kread((KA_T)u->u_cwd, (char *)&cd, sizeof(cd))
-		== 0) {
-			if (cd.cw_dir) {
-				dl = (int)((char *)u->u_cwd + cd.cw_len
-				   - cd.cw_dir);
-				if (dl >= MAXPATHLEN)
-					dl = MAXPATHLEN - 1;
-				if (dl < 2 || kread((KA_T)cd.cw_dir, Cwd, dl))
-					*Cwd = '\0';
-				else
-					Cwd[dl] = '\0';
-			}
-			if (cd.cw_root) {
-				if (cd.cw_dir >= cd.cw_root)
-					dl = (int)(cd.cw_dir - cd.cw_root);
-				else
-					dl = (int)((char *)u->u_cwd + cd.cw_len
-					   - cd.cw_root);
-				if (dl >= MAXPATHLEN)
-					dl = MAXPATHLEN - 1;
-				if (dl < 2 || kread((KA_T)cd.cw_root, Rtd, dl))
-					*Rtd = '\0';
-				else
-					Rtd[dl] = '\0';
-			}
+	    *Cwd = *Rtd = '\0';
+	    if (u->u_cwd && !kread((KA_T)u->u_cwd, (char *)&cd, sizeof(cd))) {
+		if (cd.cw_dir) {
+		    dl = (int)((char *)u->u_cwd + cd.cw_len - cd.cw_dir);
+		    if (dl >= MAXPATHLEN)
+			dl = MAXPATHLEN - 1;
+		    if (dl < 2 || kread((KA_T)cd.cw_dir, Cwd, dl))
+			*Cwd = '\0';
+		    else
+			Cwd[dl] = '\0';
 		}
+		if (cd.cw_root) {
+		    if (cd.cw_dir >= cd.cw_root)
+			dl = (int)(cd.cw_dir - cd.cw_root);
+		    else
+			dl = (int)((char *)u->u_cwd + cd.cw_len - cd.cw_root);
+			if (dl >= MAXPATHLEN)
+			    dl = MAXPATHLEN - 1;
+			if (dl < 2 || kread((KA_T)cd.cw_root, Rtd, dl))
+			    *Rtd = '\0';
+			else
+			    Rtd[dl] = '\0';
+		}
+	    }
 #endif	/* ! solaris */
 
 	/*
 	 * Save current working directory information.
 	 */
-		if (u->u_cdir) {
-			alloc_lfile(CWD, -1);
-			process_node((caddr_t)u->u_cdir);
-			if (Lf->sf)
-				link_lfile();
-		}
+	    if (u->u_cdir) {
+		alloc_lfile(CWD, -1);
+
+#if	defined(FILEPTR)
+		FILEPTR = (struct file *)NULL;
+#endif	/* defined(FILEPTR) */
+
+		process_node((KA_T)u->u_cdir);
+		if (Lf->sf)
+		    link_lfile();
+	    }
 	/*
 	 * Save root directory information.
 	 */
-		if (u->u_rdir) {
-			alloc_lfile(RTD, -1);
-			process_node((caddr_t)u->u_rdir);
-			if (Lf->sf)
-				link_lfile();
-		}
+	    if (u->u_rdir) {
+		alloc_lfile(RTD, -1);
+
+#if	defined(FILEPTR)
+		FILEPTR = (struct file *)NULL;
+#endif	/* defined(FILEPTR) */
+
+		process_node((KA_T)u->u_rdir);
+		if (Lf->sf)
+		    link_lfile();
+	    }
 	/*
 	 * Save information on text files.
 	 */
-		if (p->p_as && Nl[X_SEGVN].n_value)
-			process_text(p->p_as);
+	    if (p->p_as && Sgvops) {
+
+#if	defined(FILEPTR)
+		FILEPTR = (struct file *)NULL;
+#endif	/* defined(FILEPTR) */
+
+		process_text((KA_T)p->p_as);
+	    }
 	/*
 	 * Save information on file descriptors.
 	 */
@@ -276,29 +289,31 @@ gather_proc_info()
 	 * contiguous memory block.
 	 */
 # if	solaris<20400
-		for (i = 0, j = 0; i < u->u_nofiles; i++) {
-		    if (++j > NFPCHUNK) {
-			if ( ! u->u_flist.uf_next)
-				break;
-			if (kread((KA_T)u->u_flist.uf_next,
-			    (char *)&u->u_flist, sizeof(struct ufchunk)))
-				break;
-			j = 1;
-		    }
-		    if (u->u_flist.uf_ofile[j-1] == NULL)
+	    for (i = 0, j = 0; i < u->u_nofiles; i++) {
+		if (++j > NFPCHUNK) {
+		    if (!u->u_flist.uf_next)
+			break;
+		    if (kread((KA_T)u->u_flist.uf_next,
+			(char *)&u->u_flist, sizeof(struct ufchunk)))
+			    break;
+		    j = 1;
+		}
+		if (!u->u_flist.uf_ofile[j-1])
 # else	/* solaris>=20400 */
-		for (i = 0, j = NFPCHUNK; i < u->u_nofiles; i++) {
-		    if (++j > NFPCHUNK) {
-			k = u->u_nofiles - i;
-			if (k > NFPCHUNK)
-			    k = NFPCHUNK;
-			if (kread((KA_T)((unsigned long)u->u_flist +
-					i * sizeof(uf_entry_t)),
-			    (char*)&uf, k * sizeof(uf_entry_t)))
-				    break;
-			j = 1;
+	    for (i = 0, j = NFPCHUNK; i < u->u_nofiles; i++) {
+		if (++j > NFPCHUNK) {
+		    k = u->u_nofiles - i;
+		    if (k > NFPCHUNK)
+			k = NFPCHUNK;
+		    if (kread((KA_T)((unsigned long)u->u_flist +
+				     i * sizeof(uf_entry_t)),
+				     (char*)&uf, k * sizeof(uf_entry_t)))
+		    {
+			break;
 		    }
-		    if (uf[j-1].uf_ofile == NULL)
+		    j = 1;
+		}
+		if (!uf[j-1].uf_ofile)
 # endif	/* solaris<20400 */
 #else	/* !solaris */
 
@@ -307,63 +322,59 @@ gather_proc_info()
 	 * user structure or in a separately allocated area.  Determine
 	 * which is the case and read the pointers.
 	 */
-		if ((caddr_t)u->u_ofile
-		== ((caddr_t)&u->u_ofile_arr[0] - (caddr_t)&u
-		+ (caddr_t)p->p_uarea)) {
-			nf = NOFILE_IN_U;
-			uf = &u->u_ofile_arr[0];
-		} else {
-			nf = u->u_lastfile + 1;
-			ui = nf * sizeof(struct file);
-			if (xuf == NULL) {
-				xnf = nf;
-				if ((xuf = (struct file **)malloc(ui))
-				== NULL) {
-					(void) fprintf(stderr,
-						"%s: PID %d, malloc file[]\n",
-						Pn, Lp->pid);
-					Exit(1);
-				}
-			} else if (nf > xnf) {
-				xnf = nf;
-				if ((xuf = (struct file **)realloc(
-					   (MALLOC_P *)xuf, ui))
-				== NULL) {
-					(void) fprintf(stderr,
-						"%s: PID %d, realloc file[]\n",
-						Pn, Lp->pid);
-					Exit(1);
-				}
-			}
-			if (kread((KA_T)u->u_ofile, (char *)xuf, (int)ui))
-				nf = 0;
-			uf = xuf;
+	    if ((KA_T)u->u_ofile
+		== ((KA_T)&u->u_ofile_arr[0] - (KA_T)&u
+		+ (KA_T)p->p_uarea))
+	    {
+		nf = NOFILE_IN_U;
+		uf = &u->u_ofile_arr[0];
+	    } else {
+		nf = u->u_lastfile + 1;
+		ui = nf * sizeof(struct file *);
+		if (!xuf) {
+		    xnf = nf;
+		    if (!(xuf = (struct file **)malloc(ui))) {
+			(void) fprintf(stderr, "%s: PID %d, malloc file[]\n",
+			    Pn, Lp->pid);
+			Exit(1);
+		    }
+		} else if (nf > xnf) {
+		    xnf = nf;
+		    if (!(xuf = (struct file **)realloc((MALLOC_P *)xuf, ui))) {
+			(void) fprintf(stderr, "%s: PID %d, realloc file[]\n",
+			     Pn, Lp->pid);
+			Exit(1);
+		    }
 		}
-		*Cwd = *Rtd = '\0';
-		for (i = 0; i < nf; i++) {
-			if (!uf[i])
+		if (kread((KA_T)u->u_ofile, (char *)xuf, (int)ui))
+		    nf = 0;
+		uf = xuf;
+	    }
+	    *Cwd = *Rtd = '\0';
+	    for (i = 0; i < nf; i++) {
+		if (!uf[i])
 #endif	/* solaris */
-				continue;
-			alloc_lfile(NULL, i);
+		    continue;
+		alloc_lfile((char *)NULL, i);
 
 #if	defined(solaris)
 # if	solaris<20400
-			process_file(u->u_flist.uf_ofile[j-1]);
+		process_file((KA_T)u->u_flist.uf_ofile[j-1]);
 # else	/* solaris>=20400 */
-			process_file(uf[j-1].uf_ofile);
+		process_file((KA_T)uf[j-1].uf_ofile);
 # endif	/* solaris <20400 */
 #else	/* !solaris */
-			process_file(uf[i]);
+		process_file((KA_T)uf[i]);
 #endif	/* solaris */
 
-			if (Lf->sf)
-				link_lfile();
-		}
+		if (Lf->sf)
+		    link_lfile();
+	    }
 	/*
 	 * Examine results.
 	 */
-		if (examine_lproc())
-			return;
+	    if (examine_lproc())
+		return;
 	}
 }
 
@@ -375,17 +386,74 @@ gather_proc_info()
 static void
 get_kernel_access()
 {
+	char er[64];
+	int l;
+	KA_T v;
 
 #if	defined(HAS_AFS)
 	struct nlist *nl = (struct nlist *)NULL;
-#define	NLSIZE	((X_LAST + 1) * sizeof(struct nlist))
 #endif	/* defined(HAS_AFS) */
 
+/*
+ * Check the Solaris or SunOS version number; check the SunOS architecture.
+ */
+	(void) ckkv(
+
+#if	defined(solaris)
+		    "Solaris", LSOF_VSTR, (char *)NULL, (char *)NULL
+#else	/* !defined(solaris) */
+		    "SunOS", LSOF_VSTR, (char *)NULL, LSOF_ARCH
+#endif	/* defined(solaris) */
+
+	);
+
+#if	defined(solaris) && solaris>=20700
+/*
+ * Compare the lsof compilation bit size with the kernel bit size.
+ * Quit on a mismatch.
+ */
+	{
+	    char *cp, isa[1024];
+	    short kbits = 32;
+
+#  if	defined(_LP64)
+	    int xkbits = 64;
+#  else	/* !defined(_LP64) */
+	    short xkbits = 32;
+#  endif	/* defined(_LP64) */
+
+	    if (sysinfo(SI_ISALIST, isa, (long)sizeof(isa)) < 0) {
+		(void) fprintf(stderr, "%s: can't get ISA list: %s\n",
+		    Pn, strerror(errno));
+		Exit(1);
+	    }
+	    for (cp = isa; *cp;) {
+		if (strncmp(cp, "sparcv9", strlen("sparcv9")) == 0) {
+		    kbits = 64;
+		    break;
+		}
+		if (!(cp = strchr(cp, ' ')))
+		    break;
+		cp++;
+	    }
+	    if (kbits != xkbits) {
+		(void) fprintf(stderr,
+		    "%s: FATAL: lsof was compiled for a %d bit kernel,\n",
+		    Pn, xkbits);
+		(void) fprintf(stderr,
+		    "      but this machine has booted a %d bit kernel.\n",
+		    kbits);
+		Exit(1);
+	    }
+	}
+#endif	/* defined(solaris) && solaris>=20700 */
+
 /*
  * Get kernel symbols.
  */
 	if (Nmlst && !is_readable(Nmlst, 1))
-		Exit(1);
+	    Exit(1);
+	(void) build_Nl(Drive_Nl);
 
 #if	defined(HAS_AFS)
 	if (!Nmlst) {
@@ -395,28 +463,25 @@ get_kernel_access()
 	 * from N_UNIX, make a copy of Nl[] for possible use with the AFS
 	 * modload file.
 	 */
-		if ((nl = (struct nlist *)malloc(NLSIZE))
-		== (struct nlist *)NULL)
-		{
-			(void) fprintf(stderr,
-				"%s: no space (%d) for Nl[] copy\n",
-				Pn, NLSIZE);
-			Exit(1);
-		}
+	    if (!(nl = (struct nlist *)malloc(Nll))) {
+		(void) fprintf(stderr, "%s: no space (%d) for Nl[] copy\n",
+		    Pn, Nll);
+		Exit(1);
+	    }
 
 # if	defined(solaris)
-		(void) memcpy((void *)nl, (void *)Nl, (size_t)NLSIZE);
+	    (void) memcpy((void *)nl, (void *)Nl, (size_t)Nll);
 # else	/* !defined(solaris) */
-		(void) memcpy((char *)nl, (char *)Nl, (int)NLSIZE);
+	    (void) memcpy((char *)nl, (char *)Nl, (int)Nll);
 # endif	/* defined(solaris) */
 
 	}
 #endif	/* defined(HAS_AFS) */
 
 	if (nlist(Nmlst ? Nmlst : N_UNIX, Nl) < 0) {
-		(void) fprintf(stderr, "%s: can't read namelist from %s\n",
-			Pn, Nmlst ? Nmlst : N_UNIX);
-		Exit(1);
+	    (void) fprintf(stderr, "%s: can't read namelist from %s\n",
+		Pn, Nmlst ? Nmlst : N_UNIX);
+	    Exit(1);
 	}
 
 #if	defined(HAS_AFS)
@@ -428,10 +493,11 @@ get_kernel_access()
 	 * the AFS modload file.  Make sure that other symbols that appear in
 	 * both name list files have the same values.
 	 */
-		if (!Nl[X_AFS_FID].n_value || !Nl[X_AFS_OPS].n_value
-		||  !Nl[X_AFS_VOL].n_value)
-			(void) ckAFSsym(nl);
-		(void) free((MALLOC_P *)nl);
+	    if ((get_Nl_value("arFID", Drive_Nl, &v) >= 0 && !v)
+	    ||  (get_Nl_value("avops", Drive_Nl, &v) >= 0 && !v)
+	    ||  (get_Nl_value("avol",  Drive_Nl, &v) >= 0 && !v))
+		(void) ckAFSsym(nl);
+	    (void) free((MALLOC_P *)nl);
 	}
 #endif	/* defined(HAS_AFS) */
 
@@ -444,9 +510,9 @@ get_kernel_access()
  * call to open_kvm().
  */
 	if (!Memory) {
-		Savedgid = getegid();
-		if (Setgid)
-			Switchgid = 1;
+	    Savedgid = getegid();
+	    if (Setgid)
+		Switchgid = 1;
 	}
 # endif	/* defined(solaris) */
 
@@ -455,13 +521,13 @@ get_kernel_access()
  * before attempting to open the (Memory) file.
  */
 	if (Memory)
-		(void) dropgid();
+	    (void) dropgid();
 #else	/* !defined(WILLDROPGID) */
 /*
  * See if the non-KMEM memory file is readable.
  */
 	if (Memory && !is_readable(Memory, 1))
-		Exit(1);
+	    Exit(1);
 #endif	/* defined(WILLDROPGID) */
 
 /*
@@ -471,32 +537,33 @@ get_kernel_access()
 /*
  * Get a proc structure count estimate.
  */
-	if (Nl[X_NPROC].n_value
-	&&  kread((KA_T)Nl[X_NPROC].n_value, (char *)&Knp, sizeof(Knp))
+	if (get_Nl_value("nproc", Drive_Nl, &v) < 0 || !v
+	||  kread((KA_T)v, (char *)&Knp, sizeof(Knp))
 	||  Knp < 1)
-		Knp = PROCDFLT;
+	    Knp = PROCDFLT;
 
 #if	defined(solaris)
 # if	solaris>=20500
 /*
  * Get the kernel's KERNELBASE value for Solaris 2.5 and above.
  */
-	if ( ! Nl[X_KERNELBASE].n_value
-	||  kread((KA_T)Nl[X_KERNELBASE].n_value, (char *)&Kb, sizeof(Kb))) {
-		(void) fprintf(stderr, "%s: can't read %s from %#lx\n", Pn,
-			Nl[X_KERNELBASE].n_name, Nl[X_KERNELBASE].n_value);
-		Exit(1);
+	v = (KA_T)0;
+	if (get_Nl_value("kbase", Drive_Nl, &v) < 0 || !v
+	||  kread((KA_T)v, (char *)&Kb, sizeof(Kb))) {
+	    (void) fprintf(stderr,
+		"%s: can't read kernel base address from %s\n",
+		Pn, print_kptr(v, (char *)NULL));
+	    Exit(1);
 	}
 # endif	/* solaris >=20500 */
+
 /*
  * Get the Solaris clone major device number, if possible.
  */
-	if (Nl[X_CLONEMAJ].n_value
-	&&  kread((KA_T)Nl[X_CLONEMAJ].n_value, (char *)&CloneMaj,
-		  sizeof(CloneMaj)) == 0)
-	{
-		HaveCloneMaj = 1;
-	}
+	v = (KA_T)0;
+	if (get_Nl_value("clmaj", Drive_Nl, &v) >= 0 && v
+	&&  kread((KA_T)v, (char *)&CloneMaj, sizeof(CloneMaj)) == 0)
+	    HaveCloneMaj = 1;
 /*
  * The Solaris process table is a linked list whose head pointer is acquired
  * by open_kvm()'s call to kvm_open().  We call close_kvm() here so that the
@@ -521,7 +588,6 @@ void
 initialize()
 {
 	get_kernel_access();
-	iuidcache(UIDCACHEL);
 
 #if	defined(solaris)
 /*
@@ -532,7 +598,7 @@ initialize()
 	readfsinfo();
 
 # if	defined(HASDCACHE)
-	readdev();
+	readdev(0);
 # else	/* !defined(HASDCACHE) */
 	read_clone();
 # endif	/*defined(HASDCACHE) */
@@ -541,13 +607,13 @@ initialize()
  * Create current working and root directory buffers.
  */
 	if ((Cwd = (char *)malloc(MAXPATHLEN)) == NULL) {
-		(void) fprintf(stderr, "%s: no space for cwd buffer\n", Pn);
-		Exit(1);
+	    (void) fprintf(stderr, "%s: no space for cwd buffer\n", Pn);
+	    Exit(1);
 	}
 	if ((Rtd = (char *)malloc(MAXPATHLEN)) == NULL) {
-		(void) fprintf(stderr,
-			"%s: no space for root directory buffer\n", Pn);
-		Exit(1);
+	    (void) fprintf(stderr, "%s: no space for root directory buffer\n",
+		Pn);
+	    Exit(1);
 	}
 	*Cwd = *Rtd = '\0';
 #endif	/* solaris */
@@ -569,20 +635,22 @@ kread(addr, buf, len)
 /*
  * Because lsof reads kernel data and follows pointers found there at a
  * rate considerably slower than the kernel, lsof sometimes acquires
- * invalid pointers.  If the invalid pointers are fed to kvm_read(),
+ * invalid pointers.  If the invalid pointers are fed to kvm_[k]read(),
  * a segmentation violation may result, so legal kernel addresses are
  * limited by the value of the KERNELBASE symbol (Kb value from the
  * kernel's _kernelbase variable for Solaris 2.5 and above).
  */
 
 #if	defined(solaris) && solaris>=20500
-	if ((u_long)addr < Kb)
+#define	KVMREAD	kvm_kread
+	if (addr < Kb)
 #else	/* !defined(solaris) || solaris<20500 */
-	if ((u_long)addr < (u_long)KERNELBASE)
+#define	KVMREAD	kvm_read
+	if (addr < (KA_T)KERNELBASE)
 #endif	/* defined(solaris) && solaris>=20500 */
 
 		return(1);
-	br = kvm_read(Kd, (u_long)addr, buf, len);
+	br = KVMREAD(Kd, (u_long)addr, buf, len);
 	return((br == len) ? 0 : 1);
 }
 
@@ -594,8 +662,8 @@ kread(addr, buf, len)
 static void
 open_kvm()
 {
-	if (Kd != NULL)
-		return;
+	if (Kd)
+	    return;
 
 #if	defined(solaris) && defined(WILLDROPGID)
 /*
@@ -603,24 +671,23 @@ open_kvm()
  * surrendered, regain it.
  */
 	if (Switchgid == 2 && !Setgid) {
-		if (setgid(Savedgid) != 0) {
-			(void) fprintf(stderr,
-				"%s: can't set effective GID to %d: %s\n",
-				Pn, Savedgid, strerror(errno));
-			Exit(1);
-		}
-		Setgid = 1;
+	    if (setgid(Savedgid) != 0) {
+		(void) fprintf(stderr,
+		    "%s: can't set effective GID to %d: %s\n",
+		    Pn, (int)Savedgid, strerror(errno));
+		Exit(1);
+	    }
+	    Setgid = 1;
 	}
 #endif	/* defined(solaris) && defined(WILLDROPGID) */
 
-	if ((Kd = kvm_open(Nmlst, Memory, NULL, O_RDONLY, NULL)) == NULL) {
-		(void) fprintf(stderr,
-			"%s: kvm_open (namelist=%s, core=%s): %s\n",
-			Pn,
-			Nmlst ? Nmlst : "default",
-			Memory  ? Memory  : "default",
-			strerror(errno));
-		Exit(1);
+	if (!(Kd = kvm_open(Nmlst, Memory, NULL, O_RDONLY, NULL))) {
+	    (void) fprintf(stderr, "%s: kvm_open (namelist=%s, core=%s): %s\n",
+		Pn,
+		Nmlst ? Nmlst : "default",
+		Memory  ? Memory  : "default",
+		strerror(errno));
+	    Exit(1);
 	}
 
 #if	defined(WILLDROPGID)
@@ -636,7 +703,7 @@ open_kvm()
  * first call to this function.
  */
 	if (Switchgid == 1)
-		Switchgid = 2;
+	    Switchgid = 2;
 # endif	/* defined(solaris) */
 #endif	/* define(WILLDROPGID) */
 
@@ -655,27 +722,28 @@ open_kvm()
 
 static void
 process_text(pa)
-	struct as *pa;			/* address space description pointer */
+	KA_T pa;			/* address space description pointer */
 {
 	struct as as;
 	int i, j, k, l;
 	struct seg s;
 	union segdata {
-		struct segvn_data vn;
-		struct segdev_data dev;
-		struct segmap_data map;
+	    struct segvn_data vn;
+	    struct segdev_data dev;
+	    struct segmap_data map;
 	} sd;
-	struct vnode *v[MAXSEGS];
+	KA_T v[MAXSEGS];
 /*
  * Get address space description.
  */
 	if (kread((KA_T)pa, (char *)&as, sizeof(as))) {
-		alloc_lfile(" txt", -1);
-		(void) sprintf(Namech, "can't read text segment list (%#x)",
-			pa);
-		enter_nm(Namech);
+	    alloc_lfile(" txt", -1);
+	    (void) sprintf(Namech, "can't read text segment list (%s)",
+		print_kptr(pa, (char *)NULL));
+	    enter_nm(Namech);
+	    if (Lf->sf)
 		link_lfile();
-		return;
+	    return;
 	}
 /*
  * Loop through the segments.  The loop should stop when the segment
@@ -684,36 +752,40 @@ process_text(pa)
  */
 	s.s_next = as.a_segs;
 	for (i = j = k = 0; i < MAXSEGS && j < 2*MAXSEGS; j++) {
-		if (s.S_NEXT == NULL
-		||  kread((KA_T)s.S_NEXT, (char *)&s, sizeof(s)))
-			break;
-		if ((u_long)s.s_ops == Nl[X_SEGVN].n_value
-		&&  s.s_data != NULL) {
-			if (kread((KA_T)s.s_data, (char *)&sd, sizeof(sd)))
-				break;
-			if (sd.vn.vp) {
+	    if (!s.S_NEXT
+	    ||  kread((KA_T)s.S_NEXT, (char *)&s, sizeof(s)))
+		break;
+	    if ((KA_T)s.s_ops == Sgvops && s.s_data) {
+		if (kread((KA_T)s.s_data, (char *)&sd, sizeof(sd)))
+		    break;
+		if (sd.vn.vp) {
 			
-			/*
-			 * This is a virtual node segment.
-			 *
-			 * If its vnode pointer has not been seen already,
-			 * print its information.
-			 */
-				for (l = 0; l < k; l++) {
-					if (v[l] == sd.vn.vp)
-						break;
-				}
-				if (l >= k) {
-					alloc_lfile(" txt", -1);
-					process_node((caddr_t)sd.vn.vp);
-					if (Lf->sf) {
-						link_lfile();
-						i++;
-					}
-					v[k++] = sd.vn.vp;
-				}
+		/*
+		 * This is a virtual node segment.
+		 *
+		 * If its vnode pointer has not been seen already,
+		 * print its information.
+		 */
+		    for (l = 0; l < k; l++) {
+			if (v[l] == (KA_T)sd.vn.vp)
+			    break;
+		    }
+		    if (l >= k) {
+			alloc_lfile(" txt", -1);
+
+#if	defined(FILEPTR)
+			FILEPTR = (struct file *)NULL;
+#endif	/* defined(FILEPTR) */
+
+			process_node((KA_T)sd.vn.vp);
+			if (Lf->sf) {
+			    link_lfile();
+			    i++;
 			}
+			v[k++] = (KA_T)sd.vn.vp;
+		    }
 		}
+	    }
 	/*
 	 * Follow the segment link to the starting point in the address
 	 * space description.  (The i and j counters place an absolute
@@ -721,12 +793,12 @@ process_text(pa)
 	 */
 
 #if	!defined(solaris) || solaris<20400
-		if (s.s_next == as.a_segs)
+	    if (s.s_next == as.a_segs)
 #else	/* solaris>=20400 */
-		if (s.s_next.list == as.a_segs.list)
+	    if (s.s_next.list == as.a_segs.list)
 #endif	/* !defined(solaris) && solaris<20400 */
 
-			break;
+		break;
 	}
 }
 
@@ -743,41 +815,38 @@ readfsinfo()
 	int i;
 
 	if ((Fsinfomax = sysfs(GETNFSTYP)) == -1) {
-		(void) fprintf(stderr, "%s: sysfs(GETNFSTYP) error: %s\n",
-			Pn, strerror(errno));
-		Exit(1);
+	    (void) fprintf(stderr, "%s: sysfs(GETNFSTYP) error: %s\n",
+		Pn, strerror(errno));
+	    Exit(1);
 	} 
 	if (Fsinfomax == 0)
 		return;
-	if ((Fsinfo = (char **)malloc((MALLOC_S)(Fsinfomax * sizeof(char *))))
-	== NULL) {
-		(void) fprintf(stderr, "%s: no space for sysfs info\n", Pn);
-		Exit(1);
+	if (!(Fsinfo = (char **)malloc((MALLOC_S)(Fsinfomax * sizeof(char *)))))
+	{
+	    (void) fprintf(stderr, "%s: no space for sysfs info\n", Pn);
+	    Exit(1);
 	}
 	for (i = 1; i <= Fsinfomax; i++) {
-		if (sysfs(GETFSTYP, i, buf) == -1) {
-			(void) fprintf(stderr,
-				"%s: sysfs(GETFSTYP) error: %s\n",
-				Pn, strerror(errno));
-			Exit(1);
-		}
-		if (buf[0] == '\0') {
-			Fsinfo[i-1] = "";
-			continue;
-		}
-		buf[FSTYPSZ] = '\0';
-		if ((Fsinfo[i-1] = (char *)malloc((MALLOC_S)(strlen(buf) + 1)))
-		== NULL) {
-			(void) fprintf(stderr,
-				"%s: no space for file system entry %s\n",
-				Pn, buf);
-			Exit(1);
-		}
-		(void) strcpy(Fsinfo[i-1], buf);
+	    if (sysfs(GETFSTYP, i, buf) == -1) {
+		(void) fprintf(stderr, "%s: sysfs(GETFSTYP) error: %s\n",
+		    Pn, strerror(errno));
+		Exit(1);
+	    }
+	    if (buf[0] == '\0') {
+		Fsinfo[i-1] = "";
+		continue;
+	    }
+	    buf[FSTYPSZ] = '\0';
+	    if (!(Fsinfo[i-1] = (char *)malloc((MALLOC_S)(strlen(buf) + 1)))) {
+		(void) fprintf(stderr,
+		    "%s: no space for file system entry %s\n", Pn, buf);
+		Exit(1);
+	    }
+	    (void) strcpy(Fsinfo[i-1], buf);
 
 # if	defined(HAS_AFS)
-		if (strcasecmp(buf, "afs") == 0)
-			AFSfstype = i;
+	    if (strcasecmp(buf, "afs") == 0)
+		AFSfstype = i;
 # endif	/* defined(HAS_AFS) */
 
 	}
@@ -814,7 +883,7 @@ read_proc()
 	    if (sz == 0) {
 		sz = Knp + PROCDFLT/4;
 		len = (sz * sizeof(struct proc));
-		if ((P = (struct proc *)malloc(len)) == (struct proc *)NULL) {
+		if (!(P = (struct proc *)malloc(len))) {
 		    (void) fprintf(stderr, "%s: no proc table space\n", Pn);
 		    Exit(1);
 		}
@@ -825,19 +894,18 @@ read_proc()
 	 */
 		len = (MALLOC_S)(sz * sizeof(int));
 		if (Fpgrp) {
-		    if ((Pgrp = (int *)malloc(len)) == (int *)NULL) {
+		    if (!(Pgrp = (int *)malloc(len))) {
 			(void) fprintf(stderr, "%s: no PGRP table space\n", Pn);
 			Exit(1);
 		    }
 		}
-		if ((Pid = (int *)malloc(len)) == (int *)NULL) {
+		if (!(Pid = (int *)malloc(len))) {
 		    (void) fprintf(stderr, "%s: no PID table space\n", Pn);
 		    Exit(1);
 		}
 #endif	/* defined(solaris) */
 
 	    }
-
 	/*
 	 * Prepare for a proc table scan.
 	 */
@@ -851,20 +919,28 @@ read_proc()
 	 * Accumulate proc structures.
 	 */
 	    Np = 0;
-	    while ((p = kvm_nextproc(Kd)) != NULL) {
+	    while ((p = kvm_nextproc(Kd))) {
 		if (p->p_stat == 0 || p->p_stat == SZOMB)
 		    continue;
 
 #if	defined(solaris)
+# if	solaris >=20500
+		/*
+		 * Check Solaris 2.5 and above p_cred pointer.
+		 */
+	    	    if (!p->p_cred)
+			continue;
+# endif	/* solaris >=20500 */
+
 		/*
 		 * Read Solaris PGRP and PID numbers.
 		 */
 		if (Fpgrp) {
-		    if (p->p_pgidp == NULL
+		    if (!p->p_pgidp
 		    ||  kread((KA_T)p->p_pgidp, (char *)&pg, sizeof(pg)))
 			continue;
 		}
-		if (p->p_pidp == NULL
+		if (!p->p_pidp
 		||  kread((KA_T)p->p_pidp, (char *)&pids, sizeof(pids)))
 		    continue;
 #endif	/* defined(solaris) */
@@ -876,8 +952,7 @@ read_proc()
 		 */
 		    sz += PROCDFLT/2;
 		    len = (MALLOC_S)(sz * sizeof(struct proc));
-		    if ((P = (struct proc *)realloc((MALLOC_P *)P, len))
-		    == NULL) {
+		    if (!(P = (struct proc *)realloc((MALLOC_P *)P, len))) {
 			(void) fprintf(stderr,
 			    "%s: no more (%d) proc space\n", Pn, sz);
 			Exit(1);
@@ -889,15 +964,13 @@ read_proc()
 		 */
 		    len = (MALLOC_S)(sz * sizeof(int));
 		    if (Fpgrp) {
-			if ((Pgrp = (int *)realloc((MALLOC_P *)Pgrp, len))
-			== NULL) {
+			if (!(Pgrp = (int *)realloc((MALLOC_P *)Pgrp, len))) {
 			    (void) fprintf(stderr,
 				"%s: no more (%d) PGRP space\n", Pn, sz);
 			    Exit(1);
 			}
 		    }
-		    if ((Pid = (int *)realloc((MALLOC_P *)Pid, len))
-		    == NULL) {
+		    if (!(Pid = (int *)realloc((MALLOC_P *)Pid, len))) {
 			(void) fprintf(stderr,
 			    "%s: no more (%d) PID space\n", Pn, sz);
 			Exit(1);
@@ -912,8 +985,8 @@ read_proc()
 	     * local tables.
 	     */
 		if (Fpgrp)
-		    Pgrp[Np] = (int) pg.pid_id;
-		Pid[Np] = (int) pids.pid_id;
+		    Pgrp[Np] = (int)pg.pid_id;
+		Pid[Np] = (int)pids.pid_id;
 #endif	/* defined(solaris) */
 
 	    /*
@@ -946,7 +1019,7 @@ read_proc()
 	 * not in repeat mode.
 	 */
 	    len = (MALLOC_S)(Np * sizeof(struct proc));
-	    if ((P = (struct proc *)realloc((MALLOC_P *)P, len)) == NULL) {
+	    if (!(P = (struct proc *)realloc((MALLOC_P *)P, len))) {
 		(void) fprintf(stderr, "%s: can't reduce proc table to %d\n",
 		    Pn, Np);
 		Exit(1);
@@ -959,13 +1032,13 @@ read_proc()
 	 */
 	    len = (MALLOC_S)(Np * sizeof(int));
 	    if (Fpgrp) {
-		if ((Pgrp = (int *)realloc((MALLOC_P *)Pgrp, len)) == NULL) {
+		if (!(Pgrp = (int *)realloc((MALLOC_P *)Pgrp, len))) {
 		    (void) fprintf(stderr,
 			"%s: can't reduce PGRP table to %d\n", Pn, Np);
 		    Exit(1);
 		}
 	    }
-	    if ((Pid = (int *)realloc((MALLOC_P *)Pid, len)) == NULL) {
+	    if (!(Pid = (int *)realloc((MALLOC_P *)Pid, len))) {
 		(void) fprintf(stderr,
 		    "%s: can't reduce PID table to %d\n", Pn, Np);
 		Exit(1);
@@ -974,11 +1047,3 @@ read_proc()
 
 	}
 }
-
-
-/*
- * The ncache_addr(), ncache_load(), and ncache_lookup() functions are
- * obtained from ../common/rnch.frag.
- */
-
-
