@@ -33,7 +33,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.7 98/03/06 08:38:39 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.10 98/08/03 07:37:35 abe Exp $";
 #endif
 
 
@@ -244,18 +244,19 @@ process_socket(sa, ty)
 	KA_T sa;			/* stream's data address in kernel */
 	char *ty;			/* socket type name */
 {
-	struct in_addr *fa = (struct in_addr *)NULL;
+	unsigned char *fa = (unsigned char *)NULL;
 	int fp = 0;
 	int i, lp;
 	struct ipc_s ic;
 	int ics = 0;
-	struct in_addr *la = (struct in_addr *)NULL;
+	unsigned char *la = (unsigned char *)NULL;
 	struct module_info mi;
-	KA_T ka, qp;
+	KA_T ka;
 	u_short p;
 	KA_T pcb = (KA_T)NULL;
 	struct queue q;
 	struct qinit qi;
+	KA_T qp;
 	u_short *s;
 	struct stdata sd;
 	char tbuf[32];
@@ -320,7 +321,11 @@ process_socket(sa, ty)
 #   else	/* solaris>=20500 */
 	    u_long d25[8];
 	    u_long tcp_rack;	/* Seq # we have acked */
-	    u_long d26[58];
+#    if	solaris<20501
+	    u_long d26[29];
+#    else	/* solaris>=20501 */
+	    u_long d26[33];
+#    endif	/* solaris<20501 */
 #   endif	/* solaris<20500 */
 #  endif	/* solaris<20400 */
 
@@ -334,10 +339,10 @@ process_socket(sa, ty)
 	tcph_t th;
 	struct ud_s {			/* should come from kernel source
 					 * file ../uts/common/inet/udp.c */
-	    uint udp_state;
-	    u8 dummy2[2];
-	    u8 udp_port[2];		/* port bound to this stream */
-	    u8 udp_src[4];		/* source address of this stream */
+	    uint udp_state;		/* TPI state */
+	    unsigned char d1[2];
+	    unsigned char udp_port[2];	/* port bound to this stream */
+	    unsigned char udp_src[4];	/* source address of this stream */
 	} uc;
 	int ucs = 0;
 
@@ -409,23 +414,26 @@ process_socket(sa, ty)
 	 */
 
 # if	solaris<20600
-		la = (struct in_addr *)&ic.ipc_udp_addr;
+		la = (unsigned char *)&ic.ipc_udp_addr;
 		p = (u_short)ic.ipc_udp_port;
 # else	/* solaris>=20600 */
-		la = (struct in_addr *)&ic.ipc_faddr;
-		p = (u_short)ic.ipc_fport;
+		la = (unsigned char *)&ic.ipc_laddr;
+		p = (u_short)ic.ipc_lport;
 # endif	/* solaris<20600 */
 
-		if (la->s_addr == INADDR_ANY && p == 0 && ucs) {
+		if (((struct in_addr *)la)->s_addr == INADDR_ANY
+		&&  p == 0 && ucs)
+		{
 
 		/*
-		 * If the ipc_s structure has no local address, read
-		 * the ud_s structure for the stream one back.
+		 * If the ipc_s structure has no local address, use
+		 * the port in the ud_s structure.
 		 */
 		    s = (u_short *)&uc.udp_port[0];
 		    p = *s;
 		}
-		(void) ent_inaddr(la,(int)ntohs(p),(struct in_addr *)NULL,-1);
+		(void) ent_inaddr(la, (int)ntohs(p), (unsigned char *)NULL,
+				  -1, AF_INET);
 		if (!Fsize)
 		    Lf->off_def = 1;
 		if (ucs) {
@@ -439,19 +447,21 @@ process_socket(sa, ty)
 	     */
 
 # if	solaris<20400
-		la = (struct in_addr *)&ic.ipc_tcp_addr[0];
+		la = (unsigned char *)&ic.ipc_tcp_addr[0];
 		p = (u_short)ic.ipc_tcp_addr[5];
 # else	/* solaris>=20400 */
 #  if	solaris<20600
-		la = (struct in_addr *)&ic.ipc_tcp_laddr;
+		la = (unsigned char *)&ic.ipc_tcp_laddr;
 		p = (u_short)((short *)&ic.ipc_tcp_ports)[1];
 #  else	/* solaris>=20600 */
-		la = (struct in_addr *)&ic.ipc_laddr;
+		la = (unsigned char *)&ic.ipc_laddr;
 		p = (u_short)ic.ipc_lport;
 #  endif	/* solaris<20600 */
 # endif	/* solaris<20400 */
 
-		if (la->s_addr == INADDR_ANY && p == 0 && tcs) {
+		if (((struct in_addr *)la)->s_addr == INADDR_ANY
+		&&  p == 0 && tcs)
+		{
 
 		/*
 		 * If the ipc_s structure has no local address, use the local
@@ -460,7 +470,7 @@ process_socket(sa, ty)
 		 */
 
 # if	solaris!=20400
-		    la = (struct in_addr *)&tc.tcp_iph.iph_src[0];
+		    la = (unsigned char *)&tc.tcp_iph.iph_src[0];
 # endif	/* solaris!=20400 */
 
 		    if (tc.tcp_hdr_len && tc.tcp_tcph
@@ -474,7 +484,7 @@ process_socket(sa, ty)
 # if	solaris<20400
 		if ((int)ic.ipc_tcp_addr[2] != INADDR_ANY
 		|| ic.ipc_tcp_addr[4] != 0) {
-		    fa = (struct in_addr *)&ic.ipc_tcp_addr[2];
+		    fa = (unsigned char *)&ic.ipc_tcp_addr[2];
 		    fp = (int)ntohs(ic.ipc_tcp_addr[4]);
 		}
 # else	/* solaris>=20400 */
@@ -482,20 +492,20 @@ process_socket(sa, ty)
 		if ((int)ic.ipc_tcp_faddr != INADDR_ANY
 		|| ((u_short *) &ic.ipc_tcp_ports)[0] != 0)
 		{
-		    fa = (struct in_addr *)&ic.ipc_tcp_faddr;
+		    fa = (unsigned char *)&ic.ipc_tcp_faddr;
 		    fp = (int)ntohs(((u_short *)&ic.ipc_tcp_ports)[0]);
 		}
 #  else	/* solaris>=20600 */
 		if ((int)ic.ipc_faddr != INADDR_ANY || ((u_short)ic.ipc_fport))
 		{
-		    fa = (struct in_addr *)&ic.ipc_faddr;
+		    fa = (unsigned char *)&ic.ipc_faddr;
 		    fp = (int)ntohs(((u_short)ic.ipc_fport));
 		}
 #  endif	/* solaris<20600 */
 # endif	/* solaris <20400 */
 
 		if (fa || la)
-		    (void) ent_inaddr(la, lp, fa, fp);
+		    (void) ent_inaddr(la, lp, fa, fp, AF_INET);
 	    /*
 	     * Save TCP state information.
 	     */
@@ -568,11 +578,11 @@ process_socket(sa)
 	KA_T sa;			/* socket address in kernel */
 {
 	struct domain d;
-	struct in_addr *fa = (struct in_addr *)NULL;
+	unsigned char *fa = (unsigned char *)NULL;
 	int fam;
 	int fp, lp;
 	struct inpcb inp;
-	struct in_addr *la = (struct in_addr *)NULL;
+	unsigned char *la = (unsigned char *)NULL;
 	struct mbuf mb;
 	struct protosw p;
 	struct rawcb raw;
@@ -601,12 +611,15 @@ process_socket(sa)
 	    enter_nm("no socket type");
 	    return;
 	}
-	if (s.so_proto == NULL
-	||  kread((KA_T) s.so_proto, (char *) &p, sizeof(p))) {
-	    enter_nm("no protocol switch");
+	if (!s.so_proto
+	||  kread((KA_T)s.so_proto, (char *)&p, sizeof(p))) {
+	    (void) sprintf(Namech, "can't read protocol switch from %s",
+		print_kptr((KA_T)s.so_proto, (char *)NULL));
+	    enter_nm(Namech);
 	    return;
 	}
-	if (kread((KA_T) p.pr_domain, (char *) &d, sizeof(d))) {
+	if (!p.pr_domain
+	||  kread((KA_T)p.pr_domain, (char *)&d, sizeof(d))) {
 	    (void) sprintf(Namech, "can't read domain struct from %s",
 		print_kptr((KA_T)p.pr_domain, (char *)NULL));
 	    enter_nm(Namech);
@@ -667,24 +680,23 @@ process_socket(sa)
 							   : s.so_pcb),
 					(char *)NULL));
 		if (raw.rcb_laddr.sa_family == AF_INET)
-		    la = (struct in_addr *)&raw.rcb_laddr.sa_data[2];
+		    la = (unsigned char *)&raw.rcb_laddr.sa_data[2];
 		else if (raw.rcb_laddr.sa_family)
 		    printrawaddr(&raw.rcb_laddr);
 		if (raw.rcb_faddr.sa_family == AF_INET)
-		    fa = (struct in_addr *)&raw.rcb_faddr.sa_data[2];
+		    fa = (unsigned char *)&raw.rcb_faddr.sa_data[2];
 		else if (raw.rcb_faddr.sa_family) {
 		    (void) strcat(endnm(), "->");
 		    printrawaddr(&raw.rcb_faddr);
 		}
 		if (fa || la)
-		    (void) ent_inaddr(la, -1, fa, -1);
+		    (void) ent_inaddr(la, -1, fa, -1, AF_INET);
 	    } else {
 
 	    /*
 	     * Save Internet socket information.
 	     */
-		if (kread((KA_T)s.so_pcb, (char *) &inp, sizeof(inp))
-		||  (struct socket *)sa != inp.inp_socket) {
+		if (kread((KA_T)s.so_pcb, (char *)&inp, sizeof(inp))) {
 		    (void) sprintf(Namech, "can't read inpcb at %s",
 			print_kptr((KA_T)s.so_pcb, (char *)NULL));
 		    enter_nm(Namech);
@@ -693,14 +705,14 @@ process_socket(sa)
 		enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
 							    : s.so_pcb),
 					(char *)NULL));
-		la = &inp.inp_laddr;
+		la = (unsigned char *)&inp.inp_laddr;
 		lp = (int)ntohs(inp.inp_lport);
 		if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport != 0) {
-		    fa = &inp.inp_faddr;
+		    fa = (unsigned char *)&inp.inp_faddr;
 		    fp = (int)ntohs(inp.inp_fport);
 		}
 		if (fa || la)
-		    (void) ent_inaddr(la, lp, fa, fp);
+		    (void) ent_inaddr(la, lp, fa, fp, AF_INET);
 		if (p.pr_protocol == IPPROTO_TCP
 		&&  inp.inp_ppcb
 		&&  kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t)) == 0) {
