@@ -43,12 +43,16 @@
  */
 
 #define	_KMEMUSER	1
-#include	<sys/poll.h>
+#define	__BIT_TYPES_DEFINED__	1	/* work around to keep the BIND
+					 * <sys/bitypes.h> from colliding with
+					 * the Solaris <sys/int_types.h> */
+#include <sys/poll.h>
+
 # endif	/* defined(solaris) && solaris>=20600 */
 
 
 /*
- * $Id: machine.h,v 1.11 98/02/16 19:55:09 abe Exp $
+ * $Id: machine.h,v 1.15 98/06/29 15:49:44 abe Exp $
  */
 
 
@@ -165,6 +169,14 @@
 /* #define	HASINTSIGNAL	1 */
 
 
+/*
+ * HASKERNIDCK is defined for those dialects that support the comparison of
+ * the build to running kernel identity.
+ */
+
+#define	HASKERNIDCK	1
+
+
 /*
  * HASKOPT is defined for those dialects that support the -k option of
  * reading the kernel's name list from an optional file.
@@ -422,6 +434,7 @@
  * header files.
  */
 
+#define	USE_LIB_CKKV				1	/* ckkv.c */
 #define	USE_LIB_COMPLETEVFS			1	/* cvfs.c */
 #define	USE_LIB_FIND_CH_INO			1	/* fino.c */
 /* #define	USE_LIB_IS_FILE_NAMED		1	   isfn.c */
@@ -435,7 +448,6 @@
 
 /* #define	USE_LIB_READDEV			1	   rdev.c */
 /* #define	USE_LIB_READMNT			1	   rmnt.c */
-/* #define	USE_LIB_READVFS			1	   rvfs.c */
 /* #define	USE_LIB_RNAM			1	   rnam.c */
 #define	USE_LIB_RNCH				1	/* rnch.c */
 /* #define	USE_LIB_RNMH			1	   rnmh.c */
@@ -471,6 +483,10 @@
  * zeromem is a macro that uses bzero or memset.
  */
 
-#define	zeromem(a, l)		memset(a, 0, l)
+# if	defined(solaris)
+#define	zeromem(a, l)		memset((void *)a, 0, l)
+# else	/* !defined(solaris) */
+#define	zeromem(a, l)		memset((char *)a, 0, l)
+# endif	/* defined(solaris) */
 
 #endif	/* !defined(MACHINE_H) */
