@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.3 98/03/06 08:39:25 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.4 98/03/18 16:23:49 abe Exp $";
 #endif
 
 
@@ -268,7 +268,6 @@ process_node(ga)
 	static unsigned long fgops;
 	static int ft = 1;
 	struct gnode g;
-	MALLOC_S l;
 	struct mount m;
 	struct mntinfo *mi;
 	static KA_T nfsgops = (KA_T)0;
@@ -522,24 +521,22 @@ process_node(ga)
 /*
  * Save the file system names.
  */
-	if (f.fd_path[0] && (l = strlen(f.fd_path)) > 0) {
-	    if (!(Lf->fsdir = (char *)malloc(l + 1))) {
+	if (f.fd_path[0] && strlen(f.fd_path) > 0) {
+	    if (!(Lf->fsdir = mkstrcpy(f.fd_path, (MALLOC_S *)NULL))) {
 		(void) fprintf(stderr,
 		    "%s: no fs_data path name space, PID %d\n",
 		    Pn, Lp->pid);
 		Exit(1);
 	    }
-	    (void) strcpy(Lf->fsdir, f.fd_path);
 	} else
 	    Lf->fsdir = (char *)NULL;
-	if (f.fd_devname[0] && (l = strlen(f.fd_devname)) > 0) {
-	    if (!(Lf->fsdev = (char *)malloc(l + 1))) {
+	if (f.fd_devname[0] && strlen(f.fd_devname) > 0) {
+	    if (!(Lf->fsdev = mkstrcpy(f.fd_devname, (MALLOC_S *)NULL))) {
 		(void) fprintf(stderr,
 		    "%s: no fs_data path name space, PID %d\n",
 		    Pn, Lp->pid);
 		Exit(1);
 	    }
-	    (void) strcpy(Lf->fsdev, f.fd_devname);
 	} else
 	    Lf->fsdev = (char *)NULL;
 
