@@ -33,9 +33,10 @@
 
 
 /*
- * $Id: dproto.h,v 1.4 96/01/09 08:19:32 abe Exp $
+ * $Id: dproto.h,v 1.2 98/03/06 08:39:29 abe Exp $
  */
 
 
+_PROTOTYPE(extern void clr_flinfo,(void));
 _PROTOTYPE(extern int is_file_named,(char *p, enum vtype ty));
-_PROTOTYPE(extern int readgnode,(caddr_t ga, struct gnode *g));
+_PROTOTYPE(extern int readgnode,(KA_T ga, struct gnode *g));
