@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.5 98/03/06 08:39:30 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.6 98/05/22 07:16:49 abe Exp $";
 #endif
 
 
@@ -90,11 +90,11 @@ process_socket(sa)
 	KA_T sa;			/* socket address in kernel */
 {
 	struct domain d;
-	struct in_addr *fa = (struct in_addr *)NULL;
+	unsigned char *fa = (unsigned char *)NULL;
 	int fam;
 	int fp, lp;
 	struct inpcb inp;
-	struct in_addr *la = (struct in_addr *)NULL;
+	unsigned char *la = (unsigned char *)NULL;
 	struct mbuf mb;
 	struct protosw p;
 	struct rawcb raw;
@@ -235,17 +235,17 @@ process_socket(sa)
 							   : s.so_pcb),
 					(char *)NULL));
 		if (raw.rcb_laddr.sa_family == AF_INET)
-		    la = (struct in_addr *)&raw.rcb_laddr.sa_data[2];
+		    la = (unsigned char *)&raw.rcb_laddr.sa_data[2];
 		else if (raw.rcb_laddr.sa_family)
 		    printrawaddr(&raw.rcb_laddr);
 		if (raw.rcb_faddr.sa_family == AF_INET)
-		    fa = (struct in_addr *)&raw.rcb_faddr.sa_data[2];
+		    fa = (unsigned char *)&raw.rcb_faddr.sa_data[2];
 		else if (raw.rcb_faddr.sa_family) {
 		    (void) strcat(endnm(), "->");
 		    printrawaddr(&raw.rcb_faddr);
 		}
 		if (fa || la)
-		    (void) ent_inaddr(la, -1, fp, -1);
+		    (void) ent_inaddr(la, -1, fp, -1, AF_INET);
 	    } else {
 
 	    /*
@@ -261,14 +261,14 @@ process_socket(sa)
 		enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
 							    : s.so_pcb),
 					(char *)NULL));
-		la = &inp.inp_laddr;
+		la = (unsigned char *)&inp.inp_laddr;
 		lp = (int)ntohs(inp.inp_lport);
 		if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport != 0) {
-		    fa = &inp.inp_faddr;
+		    fa = (unsigned char *)&inp.inp_faddr;
 		    fp = (int)ntohs(inp.inp_fport);
 		}
 		if (fa || la)
-		    (void) ent_inaddr(la, lp, fa, fp);
+		    (void) ent_inaddr(la, lp, fa, fp, AF_INET);
 		if (p.pr_protocol == IPPROTO_TCP && inp.inp_ppcb
 		&&  !kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t))) {
 		    Lf->lts.type = 0;
