@@ -32,14 +32,24 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.6 98/03/06 08:40:01 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.9 98/09/14 19:55:47 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 #include <sys/fs/namenode.h>
+
+#if	UNIXWAREV>=70000
+#undef	IREAD
+#undef	IWRITE
+#undef	IEXEC
+#endif	/* UNIXWAREV>=70000 */
+
 #include <sys/fs/ufs_inode.h>
+
+#if	defined(HASXNAMNODE)
 #include <sys/fs/xnamnode.h>
+#endif	/* defined(HASXNAMNODE) */
 
 
 _PROTOTYPE(static void ent_fa,(KA_T *a1, KA_T *a2, char *d));
@@ -247,8 +257,12 @@ get_vty(v, va, kv, fx)
 	int nty = N_REGLR;
 	char tbuf[32];
 
-	if (!v->v_vfsp)
+	if (!v->v_vfsp) {
+	    *fx = 0;
+	    if (v->v_type == VFIFO && v->v_stream)
+		return(N_STREAM);
 	    return(N_REGLR);
+	}
 	if (!kread((KA_T)v->v_vfsp, (char *)kv, sizeof(struct vfs))) {
 
 	/*
@@ -345,6 +359,7 @@ process_node(na)
 {
 	char *cp;
 	dev_t dev;
+	int ds = 0;
 	struct l_dev *dp;
 	struct fifonode f;
 	int fx, rfx;
@@ -368,7 +383,6 @@ process_node(na)
 	char tbuf[32], *ty;
 	enum vtype type;
 	struct sockaddr_un ua;
-	struct l_vfs *lvfs;
 
 #if	defined(HASPROCFS)
 	struct as as;
@@ -412,20 +426,6 @@ process_node(na)
 get_lock_state:
 
 	Lf->lock = isvlocked(&v);
-/*
- * Establish the local virtual file system structure.
- */
-	if (!v.v_vfsp)
-	    lvfs = (struct l_vfs *)NULL;
-	else {
-	    if (!(lvfs = readvfs((KA_T)v.v_vfsp, &kv))) {
-		(void) sprintf(Namech, "bad vfs for %s at %s",
-		    print_kptr(na, tbuf),
-		    print_kptr((KA_T)v.v_vfsp, (char *)NULL));
-		enter_nm(Namech);
-		return;
-	    }
-	}
 /*
  * Read the fifonode, inode, namenode, prnode, rnode, snode, ...
  */
@@ -503,10 +503,6 @@ get_lock_state:
 	    fx = rfx;
 	    kv = rkv;
 	    v = rv;
-	    if (lvfs) {
-		(void) free((MALLOC_P *)lvfs);
-		lvfs = (struct l_vfs *)NULL;
-	    }
 	    goto get_lock_state;
 
 #if	defined(HASPROCFS)
@@ -599,7 +595,7 @@ get_lock_state:
 		enter_nm(Namech);
 		return;
 	    }
-	    pid = (long)pids.p_pid;
+	    pid = (long)pids.pid_id;
 	    (void) sprintf(Namech, "/%s/%ld", HASPROCFS, pid);
 	    i.number = pid + PR_INOBIAS;
 	    if (!p.p_as ||  kread((KA_T)p.p_as, (char *)&as, sizeof(as)))
@@ -612,7 +608,7 @@ get_lock_state:
 
 	case N_STREAM:
 	    if (v.v_stream) {
-		Lf->is_stream = 1;
+		Lf->is_stream = ni = 1;
 	    /*
 	     * Get the queue pointer and module name at the end of the stream.
 	     * The module name indentifies socket streams.
@@ -704,9 +700,13 @@ get_lock_state:
 		    }
 		    if (!rv.v_data || readlino(fx, &rv, &i)) {
 			(void) sprintf(Namech,
-			    "snode@%s: can't read inode (%s)",
+			    "snode@%s: unknown inode@%s; fx=",
 			     print_kptr((KA_T)v.v_data, tbuf),
 			     print_kptr((KA_T)rv.v_data, (char *)NULL));
+			    if (fx < 1 || fx > Fsinfomax)
+				(void) sprintf(endnm(), "%d", fx);
+			    else
+				(void) strcpy(endnm(), Fsinfo[fx - 1]);
 			enter_nm(Namech);
 			return;
 		     }
@@ -731,7 +731,7 @@ get_lock_state:
 			enter_nm(Namech);
 			return;
 		    }
-		    ni = 1;
+		    Lf->is_com = ni = 1;
 		} else
 		    ni = 1;
 		break;
@@ -747,9 +747,13 @@ get_lock_state:
 	 */
 	    if (readlino(fx, &v, &i)) {
 		(void) sprintf(Namech,
-		    "vnode@%s: can't read successor node from %s",
+		    "vnode@%s: unknown successor@%s; fx=",
 		    print_kptr(na, tbuf),
 		    print_kptr((KA_T)v.v_data, (char *)NULL));
+		    if (fx < 1 || fx > Fsinfomax)
+			(void) sprintf(endnm(), "%d", fx);
+		    else
+			(void) strcpy(endnm(), Fsinfo[fx - 1]);
 		enter_nm(Namech);
 		return;
 	    }
@@ -760,26 +764,38 @@ get_lock_state:
 	switch (Ntype) {
 	case N_NFS:
 	    dev = r.r_attr.va_fsid;
+	    ds = 1;
 	    break;
 
 #if	defined(HASPROCFS)
 	case N_PROC:
 	    dev = kv.vfs_dev;
+	    ds = 1;
 	    break;
 #endif	/* defined(HASPROCFS) */
 
 	case N_STREAM:
 	    if (sqp) {
-		dev = so.lux_dev.addr.tu_addr.dev;
+		if (so.lux_dev.size >= 8) {
+		    dev = so.lux_dev.addr.tu_addr.dev;
+		    ds = 1;
+		} else
+		    enter_dev_ch(print_kptr(sqp, (char *)NULL));
+		break;
+	    }
+	    if (v.v_type == VFIFO) {
+		KA_T ta;
+
+		if ((ta = (KA_T)(v.v_data ? v.v_data : v.v_stream)))
+		    enter_dev_ch(print_kptr(ta, (char *)NULL));
 		break;
 	    }
 	    /* fall through */
 	default:
 	    dev = (v.v_type == VCHR || ni) ? v.v_rdev : i.dev;
+	    ds = 1;
 	}
 	type = v.v_type;
-	if (lvfs && !lvfs->dir)
-	    (void) completevfs(lvfs, &dev);
 /*
  * Obtain the inode number.
  */
@@ -816,7 +832,7 @@ get_lock_state:
 	    }
 	    break;
 	case N_STREAM:
-	    if (sqp) {
+	    if (sqp && so.lux_dev.size >= 8) {
 		Lf->inode = (unsigned long)so.lux_dev.addr.tu_addr.ino;
 		Lf->inp_ty = 1;
 	    }
@@ -862,12 +878,9 @@ get_lock_state:
 	if (Ntype == N_NFS && Fnfs)
 	    Lf->sf |= SELNFS;
 /*
- * Save the file system names.
+ * Defer file system info lookup until printname().
  */
-	if (lvfs) {
-	    Lf->fsdir = lvfs->dir;
-	    Lf->fsdev = lvfs->fsname;
-	}
+	Lf->lmi_srch = 1;
 /*
  * Format the vnode type, and possibly the device name.
  */
@@ -878,18 +891,18 @@ get_lock_state:
 	case VREG:
 	case VDIR:
 	    ty = (type == VREG) ? "VREG" : "VDIR";
-	    Lf->dev_def = 1;
+	    Lf->dev_def = ds;
 	    Lf->dev = dev;
 	    break;
 	case VBLK:
 	    ty = "VBLK";
 	    Lf->dev = dev;
-	    Lf->dev_def = 1;
+	    Lf->dev_def = ds;
 	    Ntype = N_BLK;
 	    break;
 	case VCHR:
 	    Lf->dev = dev;
-	    Lf->dev_def = 1;
+	    Lf->dev_def = ds;
 	    ty = "VCHR";
 	    if (Lf->is_stream == 0)
 		Ntype = N_CHR;
@@ -910,7 +923,7 @@ get_lock_state:
 	case VFIFO:
 	    if (!Lf->dev_ch || Lf->dev_ch[0] == '\0') {
 		Lf->dev = dev;
-		Lf->dev_def = 1;
+		Lf->dev_def = ds;
 	    }
 	    ty = "FIFO";
 	    break;
@@ -954,7 +967,14 @@ get_lock_state:
 	    &&  !kread((KA_T)so.laddr.buf, (char *)&ua, sizeof(ua))) {
 		ua.sun_path[sizeof(ua.sun_path) - 1] = '\0';
 		(void) strcpy(Namech, ua.sun_path);
+		if (so.lux_dev.size >= 8) {
+		    Lf->inode = (unsigned long)so.lux_dev.addr.tu_addr.ino;
+		    Lf->inp_ty = 1;
+		}
 	    }
+	    if (!Namech[0] && so.so_conn)
+		(void) sprintf(Namech, "->%s",
+		    print_kptr((KA_T)so.so_conn, (char *)NULL));
 	    Lf->is_stream = 0;
 	}
 /*
@@ -1012,14 +1032,19 @@ readlino(fx, v, i)
 	struct l_ino *i;		/* local inode */
 {
 	struct vnode fa;
-	struct vfs kv;
 	struct mnode mn;
 	struct inode sn;
+
+#if	defined(HASXNAMNODE)
 	struct xnamnode xn;
+#endif	/* defined(HASXNAMNODE) */
+
 
 	if (fx < 1 || fx > Fsinfomax || !v->v_data)
 	    return(1);
-	if (!strcmp(Fsinfo[fx-1], "fifofs") || !strcmp(Fsinfo[fx-1], "ufs")) {
+	if (!strcmp(Fsinfo[fx-1], "fifofs")
+	||  !strcmp(Fsinfo[fx-1], "sfs")
+	||  !strcmp(Fsinfo[fx-1], "ufs")) {
 	    if (kread((KA_T)v->v_data, (char *)&sn, sizeof(sn)))
 		return(1);
 	    i->dev = sn.i_dev;
@@ -1031,6 +1056,10 @@ readlino(fx, v, i)
 	    return(reads5lino(v, i));
  	else if (!strcmp(Fsinfo[fx-1], "vxfs"))
 	    return(readvxfslino(v, i));
+ 	else if (!strcmp(Fsinfo[fx-1], "bfs"))
+	    return(readbfslino(v, i));
+
+#if	defined(HASXNAMNODE)
 	else if (!strcmp(Fsinfo[fx-1], "xnamfs")
 	     ||  !strcmp(Fsinfo[fx-1], "XENIX"))
 	{
@@ -1040,7 +1069,10 @@ readlino(fx, v, i)
 	    i->rdev = xn.x_fsid;
 	    i->size = xn.x_size;
 	    return(0);
-	} else if (!strcmp(Fsinfo[fx-1], "memfs")) {
+	}
+#endif	/* defined(HASXNAMNODE) */
+
+	else if (!strcmp(Fsinfo[fx-1], "memfs")) {
 	    if (kread((KA_T)v->v_data, (char *)&mn, sizeof(mn)))
 		return(1);
 	    i->dev = mn.mno_fsid;
