@@ -34,7 +34,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: main.c,v 1.17 98/03/09 10:49:35 abe Exp $";
+static char *rcsid = "$Id: main.c,v 1.19 98/04/27 06:51:48 abe Exp $";
 #endif
 
 
@@ -541,11 +541,6 @@ main(argc, argv)
 		Selinet = 1;
 	    Selall = 0;
 	}
-/*
- * Read the mount table, unless listing only network files (-i option).
- */
-	if (Selinet == 0 && readmnt() == 0)
-	    Exit(1);
 /*
  * Process the file arguments.
  */
@@ -632,8 +627,8 @@ main(argc, argv)
 		    for (i = n = 0; i < Nlproc; i++) {
 			Lp = (Nlproc > 1) ? slp[i] : &Lproc[i];
 			if (Lp->pss) {
-			    print_proc();
-			    n++;
+			    if (print_proc())
+				n++;
 			}
 			if (RptTm && PrPass)
 			    (void) free_lproc(Lp);
