@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: misc.c,v 1.11 98/03/09 15:59:46 abe Exp $";
+static char *rcsid = "$Id: misc.c,v 1.15 98/08/18 08:07:30 abe Exp $";
 #endif
 
 
@@ -178,13 +178,13 @@ compdev(a1, a2)
 	struct l_dev **p2 = (struct l_dev **)a2;
 
 	if ((dev_t)((*p1)->rdev) < (dev_t)((*p2)->rdev))
-		return(-1);
+	    return(-1);
 	if ((dev_t)((*p1)->rdev) > (dev_t)((*p2)->rdev))
-		return(1);
+	    return(1);
 	if ((ino_t)((*p1)->inode) < (ino_t)((*p2)->inode))
-		return(-1);
+	    return(-1);
 	if ((ino_t)((*p1)->inode) > (ino_t)((*p2)->inode))
-		return(1);
+	    return(1);
 	return(strcmp((*p1)->name, (*p2)->name));
 }
 
@@ -414,12 +414,12 @@ void
 dropgid()
 {
 	if (!Setuidroot && Setgid) {
-		if (setgid(Mygid) < 0) {
-			(void) fprintf(stderr, "%s: can't setgid(%d): %s\n",
-				Pn, (int)Mygid, strerror(errno));
-			Exit(1);
-		}
-		Setgid = 0;
+	    if (setgid(Mygid) < 0) {
+		(void) fprintf(stderr, "%s: can't setgid(%d): %s\n",
+		    Pn, (int)Mygid, strerror(errno));
+		Exit(1);
+	    }
+	    Setgid = 0;
 	}
 }
 #endif	/* defined(WILLDROPGID) */
@@ -548,32 +548,60 @@ handleint(sig)
  */
 
 int
-is_nw_addr(ia, p)
-	struct in_addr *ia;		/* Internet address */
+is_nw_addr(ia, p, af)
+	unsigned char *ia;		/* Internet address */
 	int p;				/* port */
+	int af;				/* address family -- e.g., AF_INET,
+					 * AF_INET6 */
 {
 	struct nwad *n;
-	unsigned char *u;
 
 	if (!(n = Nwad))
-		return(0);
-	u = (unsigned char *)ia;
+	    return(0);
 	for (; n; n = n->next) {
-		if (n->proto) {
-			if (strcasecmp(n->proto, Lf->iproto) != 0)
-				continue;
+	    if (n->proto) {
+		if (strcasecmp(n->proto, Lf->iproto) != 0)
+		    continue;
+	    }
+	    if (af && n->af && af != n->af)
+		continue;
+
+#if	defined(HASIPv6)
+	    if (af == AF_INET6) {
+		if (n->a[15] || n->a[14] || n->a[13] || n->a[12]
+		||  n->a[11] || n->a[10] || n->a[9]  || n->a[8]
+		||  n->a[7]  || n->a[6]  || n->a[5]  || n->a[4]
+		||  n->a[3]  || n->a[2]  || n->a[1]  || n->a[0]) {
+		    if (ia[15] != n->a[15] || ia[14] != n->a[14]
+		    ||  ia[13] != n->a[13] || ia[12] != n->a[12]
+		    ||  ia[11] != n->a[11] || ia[10] != n->a[10]
+		    ||  ia[9]  != n->a[9]  || ia[8]  != n->a[8]
+		    ||  ia[7]  != n->a[7]  || ia[6]  != n->a[6]
+		    ||  ia[5]  != n->a[5]  || ia[4]  != n->a[4]
+		    ||  ia[3]  != n->a[3]  || ia[2]  != n->a[2]
+		    ||  ia[1]  != n->a[1]  || ia[0]  != n->a[0])
+			continue;
 		}
-		if (n->a[0] != 0 || n->a[1] != 0
-		||  n->a[2] != 0 || n->a[3] != 0) {
-			if (u[3] != n->a[3] || u[2] != n->a[2]
-			||  u[1] != n->a[1] || u[0] != n->a[0])
-				continue;
-		}
-		if (n->sport == -1
-		||  (p >= n->sport && p <= n->eport)) {
-			n->f = 1;
-			return(1);
+	    } else if (af == AF_INET)
+#endif	/* defined(HASIPv6) */
+
+	    {
+		if (n->a[3] || n->a[2] || n->a[1] || n->a[0]) {
+		    if (ia[3] != n->a[3] || ia[2] != n->a[2]
+		    ||  ia[1] != n->a[1] || ia[0] != n->a[0])
+			continue;
 		}
+	    }
+
+#if	defined(HASIPv6)
+	    else
+		continue;
+#endif	/* defined(HASIPv6) */
+
+	    if (n->sport == -1 || (p >= n->sport && p <= n->eport)) {
+		n->f = 1;
+		return(1);
+	    }
 	}
 	return(0);
 }
@@ -590,7 +618,7 @@ char *
 mkstrcpy(src, rlp)
 	char *src;			/* source */
 	MALLOC_S *rlp;			/* returned length pointer (optional)
-					 * The reurend length is an strlen()
+					 * The returned length is an strlen()
 					 * equivalent */
 {
 	MALLOC_S len;
@@ -1149,24 +1177,23 @@ x2dev(s, d)
 	if  (strncasecmp(s, "0x", 2) == 0)
 		s += 2;
 	for (cp = s, n = 0; *cp; cp++, n++) {
-		if (isdigit((unsigned char)*cp)
-		||  (isalpha((unsigned char)*cp)
-		&&   (c = toupper((unsigned char)*cp)) >= 'A' && c <= 'F'))
-			continue;
-		if (*cp == ' ' || *cp == ',')
-			break;
-		return((char *)NULL);
+	    if (isdigit((unsigned char)*cp) || (isalpha((unsigned char)*cp)
+	    &&  (c = toupper((unsigned char)*cp)) >= 'A' && c <= 'F'))
+		continue;
+	    if (*cp == ' ' || *cp == ',')
+		break;
+	    return((char *)NULL);
 	}
 	if (!n)
-		return((char *)NULL);
+	    return((char *)NULL);
 	if (n > (2 * (int)sizeof(dev_t))) {
-		cp1 = s;
-		s += (n - (2 * sizeof(dev_t)));
-		while (cp1 < s) {
-			if (*cp1 != 'f' && *cp1 != 'F')
-				return((char *)NULL);
-			cp1++;
-		}
+	    cp1 = s;
+	    s += (n - (2 * sizeof(dev_t)));
+	    while (cp1 < s) {
+		if (*cp1 != 'f' && *cp1 != 'F')
+		    return((char *)NULL);
+		cp1++;
+	    }
 	}
 /*
  * Assemble the validated hex digits of the device number, starting at a point
