@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: print.c,v 1.16 98/03/09 12:59:35 abe Exp $";
+static char *rcsid = "$Id: print.c,v 1.24 98/08/03 15:43:04 abe Exp $";
 #endif
 
 
@@ -56,8 +56,10 @@ struct utmp dutmp;			/* dummy entry to get login name size */
 					 * and fill_porttab() */
 
 struct hostcache {
-	unsigned char addr[4];
-	char *name;
+	unsigned char a[MAX_AF_ADDR];	/* numeric address */
+	int af;				/* address family -- e.g., AF_INET
+					 * or AF_INET6 */
+	char *name;			/* name */
 };
 
 struct porttab {
@@ -157,7 +159,7 @@ fill_portmap()
 	struct porttab *pt;
 	struct rpcent *r;
 	int s = RPC_ANYSOCK;
-	struct timeval tm;
+	struct TIMEVAL_LSOF tm;
 /*
  * Construct a TCP socket address for communicating with the portmapper.
  */
@@ -331,38 +333,67 @@ fill_porttab()
  */
 
 char *
-gethostnm(ia)
-	struct in_addr *ia;		/* pointer to Internet address */
+gethostnm(ia, af)
+	unsigned char *ia;		/* Internet address */
+	int af;				/* address family -- e.g., AF_INET
+					 * or AF_INET6 */
 {
-	char hbuf[32];
+	int al = MIN_AF_ADDR;
+	char hbuf[256];
 	static struct hostcache *hc = (struct hostcache *)NULL;
 	static int hcx = 0;
 	char *hn, *np;
-	int i;
 	struct hostent *he = (struct hostent *)NULL;
+	int i, j;
 	MALLOC_S len;
 	static int nhc = 0;
-	unsigned char *u;
-
-	u = (unsigned char *)ia;
 /*
  * Search cache.
  */
+
+#if	defined(HASIPv6)
+	if (af == AF_INET6)
+	    al = MAX_AF_ADDR;
+#endif	/* defined(HASIPv6) */
+
 	for (i = 0; i < hcx; i++) {
-	    if (u[3] == hc[i].addr[3] && u[2] == hc[i].addr[2]
-	    &&  u[1] == hc[i].addr[1] && u[0] == hc[i].addr[0])
+	    if (af != hc[i].af)
+		continue;
+	    for (j = 0; j < al; j++) {
+		if (ia[j] != hc[i].a[j])
+		    break;
+	    }
+	    if (j >= al)
 		return(hc[i].name);
 	}
 /*
- * If -n has been specified, construct a dot-form address.  Otherwise,
- * look up host name by address.  If that fails, or if there is no name
- * in the returned hostent structure, construct a dot-form version of the
- * address.
+ * If -n has been specified, construct a numeric address.  Otherwise, look up
+ * host name by address.  If that fails, or if there is no name in the returned
+ * hostent structure, construct a numeric version of the address.
  */
 	if (Fhost)
-	    he = gethostbyaddr((char *)ia, sizeof(struct in_addr), AF_INET);
+	    he = gethostbyaddr((char *)ia, al, af);
 	if (!he || !he->h_name) {
-	    (void) sprintf(hbuf, "%u.%u.%u.%u", u[0], u[1], u[2], u[3]);
+
+#if	defined(HASIPv6)
+	    if (af == AF_INET6) {
+
+	    /*
+	     * Since IPv6 numeric addresses use `:' as a separator, enclose
+	     * them in brackets.
+	     */
+		hbuf[0] = '[';
+		if (!inet_ntop(af, ia, hbuf + 1, sizeof(hbuf) - 3))
+		    (void) strcpy(&hbuf[1], "can't format IPv6 address]");
+		else
+		    (void) strcat(hbuf, "]");
+	    } else
+#endif	/* defined(HASIPv6) */
+
+	    if (af == AF_INET)
+		(void) sprintf(hbuf, "%u.%u.%u.%u", ia[0], ia[1], ia[2], ia[3]);
+	    else
+		(void) sprintf(hbuf, "(unknown AF value: %d)", af);
 	    hn = hbuf;
 	} else
 	    hn = (char *)he->h_name;
@@ -389,10 +420,10 @@ gethostnm(ia)
 		Exit(1);
 	    }
 	}
-	hc[hcx].addr[0] = u[0];
-	hc[hcx].addr[1] = u[1];
-	hc[hcx].addr[2] = u[2];
-	hc[hcx].addr[3] = u[3];
+	hc[hcx].af = af;
+	for (i = 0; i < al; i++) {
+	    hc[hcx].a[i] = ia[i];
+	}
 	hc[hcx++].name = np;
 	return(np);
 }
@@ -617,7 +648,7 @@ print_file()
 		(void) printf(" %*s", SzOffColW, SZTTL);
 	    else
 		(void) printf(" %*s", SzOffColW, SZOFFTTL);
-	    (void) printf(" %*s %s\n", InoColW, INOTTL, NMTTL);
+	    (void) printf(" %*s %s\n", NodeColW, NODETTL, NMTTL);
 	    Hdr++;
 	}
 /*
@@ -691,7 +722,10 @@ print_file()
 		FdColW = len;
 	} else
 	    (void) printf(" %*.*s%c%c", FdColW - 2, FdColW - 2, Lf->fd,
-		Lf->access, Lf->lock);
+		(Lf->lock == ' ') ? Lf->access
+				  : (Lf->access == ' ') ? '-'
+							: Lf->access,
+		Lf->lock);
 /*
  * Size or print the type.
  */
@@ -821,19 +855,23 @@ print_file()
 
 	    break;
 	case 2:
-	    if (Lf->iproto[0]) {
+	    if (Lf->iproto[0])
 		cp = Lf->iproto;
-		break;
-	    }
-	    /* else fall through */
+	    else
+		cp = "";
+	    break;
+	case 3:
+	    (void) sprintf(buf, "%#lx", Lf->inode);
+	    cp = buf;
+	    break;
 	default:
 	    cp = "";
 	}
 	if (!PrPass) {
-	    if ((len = strlen(cp)) > InoColW)
-		InoColW = len;
+	    if ((len = strlen(cp)) > NodeColW)
+		NodeColW = len;
 	} else {
-	    (void) printf(" %*.*s", InoColW, InoColW, cp);
+	    (void) printf(" %*.*s", NodeColW, NodeColW, cp);
 	}
 /*
  * If this is the second pass, print the name column.  (It doesn't need
@@ -869,7 +907,7 @@ printinaddr()
  * separate it from the local address with "->".
  */
 	for (i = 0, *np = '\0'; i < 2; i++) {
-	    if (!Lf->li[i].as)
+	    if (!Lf->li[i].af)
 		continue;
 	    host = port = (char *)NULL;
 	    if (i) {
@@ -892,10 +930,22 @@ addr_too_long:
 	/*
 	 * Convert the address to a host name.
 	 */
-	    if (Lf->li[i].ia.s_addr == INADDR_ANY)
+
+#if	defined(HASIPv6)
+	    if ((Lf->li[i].af == AF_INET6
+	    &&   IN6_IS_ADDR_UNSPECIFIED(&Lf->li[i].ia.a6))
+	    ||  (Lf->li[i].af == AF_INET
+	    &&    Lf->li[i].ia.a4.s_addr == INADDR_ANY))
 		host ="*";
 	    else
-		host = gethostnm(&Lf->li[i].ia);
+		host = gethostnm((unsigned char *)&Lf->li[i].ia, Lf->li[i].af);
+#else /* !defined(HASIPv6) */
+	    if (Lf->li[i].ia.a4.s_addr == INADDR_ANY)
+		host ="*";
+	    else
+		host = gethostnm((unsigned char *)&Lf->li[i].ia, Lf->li[i].af);
+#endif	/* defined(HASIPv6) */
+
 	/*
 	 * Process the port number.
 	 */
@@ -913,12 +963,24 @@ addr_too_long:
 		 * loopback address 127.0.0.1.  (Test 2 may not always work
 		 * -- e.g., on hosts with multiple interfaces.)
 		 */
-		    if ((src = i) && FportMap
-		    &&  (Lf->li[i].ia.s_addr == INADDR_LOOPBACK
-		    ||   (Lf->li[0].as &&
-		          Lf->li[0].ia.s_addr == Lf->li[1].ia.s_addr)))
-		    {
-			src = 0;
+		    if ((src = i) && FportMap) {
+
+#if	defined(HASIPv6)
+			if (Lf->li[0].af == AF_INET6) {
+			    if (IN6_IS_ADDR_LOOPBACK(&Lf->li[i].ia.a6)
+			    ||  IN6_ARE_ADDR_EQUAL(&Lf->li[0].ia.a6,
+						   &Lf->li[1].ia.a6)
+			    )
+				src = 0;
+			} else
+#endif	/* defined(HASIPv6) */
+
+			if (Lf->li[0].af == AF_INET) {
+			    if (Lf->li[i].ia.a4.s_addr == INADDR_LOOPBACK
+			    ||  Lf->li[0].ia.a4.s_addr == Lf->li[1].ia.a4.s_addr
+			    )
+				src = 0;
+			}
 		    }
 		    if (strcasecmp(Lf->iproto, "TCP") == 0)
 			port = lkup_port(Lf->li[i].p, 0, src);
@@ -974,8 +1036,8 @@ print_init()
 	CmdColW = strlen(CMDTTL);
 	DevColW = strlen(DEVTTL);
 	FdColW = strlen(FDTTL);
-	InoColW = strlen(INOTTL);
 	NmColW = strlen(NMTTL);
+	NodeColW = strlen(NODETTL);
 	PgrpColW = strlen(PGRPTTL);
 	PidColW = strlen(PIDTTL);
 	PpidColW = strlen(PPIDTTL);
@@ -1017,9 +1079,11 @@ printiproto(p)
 #endif	/* defined(IPPROTO_UDP) */
 
 #if	defined(IPPROTO_IP)
+# if	!defined(IPPROTO_HOPOPTS) || IPPROTO_IP!=IPPROTO_HOPOPTS
 	case IPPROTO_IP:
 	    s = "IP";
 	    break;
+# endif	/* !defined(IPPROTO_HOPOPTS) || IPPROTO_IP!=IPPROTO_HOPOPTS */
 #endif	/* defined(IPPROTO_IP) */
 
 #if	defined(IPPROTO_ICMP)
@@ -1028,6 +1092,12 @@ printiproto(p)
 	    break;
 #endif	/* defined(IPPROTO_ICMP) */
 
+#if	defined(IPPROTO_ICMPV6)
+	case IPPROTO_ICMPV6:
+	    s = "ICMPV6";
+	    break;
+#endif	/* defined(IPPROTO_ICMPV6) */
+
 #if	defined(IPPROTO_IGMP)
 	case IPPROTO_IGMP:
 	    s = "IGMP";
@@ -1094,6 +1164,530 @@ printiproto(p)
 	    break;
 #endif	/* defined(IPPROTO_RAWIF) */
 
+#if	defined(IPPROTO_HOPOPTS)
+	case IPPROTO_HOPOPTS:
+	    s = "HOPOPTS";
+	    break;
+#endif	/* defined(IPPROTO_HOPOPTS) */
+
+#if	defined(IPPROTO_IPIP)
+	case IPPROTO_IPIP:
+	    s = "IPIP";
+	    break;
+#endif	/* defined(IPPROTO_IPIP) */
+
+#if	defined(IPPROTO_ST)
+	case IPPROTO_ST:
+	    s = "ST";
+	    break;
+#endif	/* defined(IPPROTO_ST) */
+
+#if	defined(IPPROTO_PIGP)
+	case IPPROTO_PIGP:
+	    s = "PIGP";
+	    break;
+#endif	/* defined(IPPROTO_PIGP) */
+
+#if	defined(IPPROTO_RCCMON)
+	case IPPROTO_RCCMON:
+	    s = "RCCMON";
+	    break;
+#endif	/* defined(IPPROTO_RCCMON) */
+
+#if	defined(IPPROTO_NVPII)
+	case IPPROTO_NVPII:
+	    s = "NVPII";
+	    break;
+#endif	/* defined(IPPROTO_NVPII) */
+
+#if	defined(IPPROTO_ARGUS)
+	case IPPROTO_ARGUS:
+	    s = "ARGUS";
+	    break;
+#endif	/* defined(IPPROTO_ARGUS) */
+
+#if	defined(IPPROTO_EMCON)
+	case IPPROTO_EMCON:
+	    s = "EMCON";
+	    break;
+#endif	/* defined(IPPROTO_EMCON) */
+
+#if	defined(IPPROTO_XNET)
+	case IPPROTO_XNET:
+	    s = "XNET";
+	    break;
+#endif	/* defined(IPPROTO_XNET) */
+
+#if	defined(IPPROTO_CHAOS)
+	case IPPROTO_CHAOS:
+	    s = "CHAOS";
+	    break;
+#endif	/* defined(IPPROTO_CHAOS) */
+
+#if	defined(IPPROTO_MUX)
+	case IPPROTO_MUX:
+	    s = "MUX";
+	    break;
+#endif	/* defined(IPPROTO_MUX) */
+
+#if	defined(IPPROTO_MEAS)
+	case IPPROTO_MEAS:
+	    s = "MEAS";
+	    break;
+#endif	/* defined(IPPROTO_MEAS) */
+
+#if	defined(IPPROTO_HMP)
+	case IPPROTO_HMP:
+	    s = "HMP";
+	    break;
+#endif	/* defined(IPPROTO_HMP) */
+
+#if	defined(IPPROTO_PRM)
+	case IPPROTO_PRM:
+	    s = "PRM";
+	    break;
+#endif	/* defined(IPPROTO_PRM) */
+
+#if	defined(IPPROTO_TRUNK1)
+	case IPPROTO_TRUNK1:
+	    s = "TRUNK1";
+	    break;
+#endif	/* defined(IPPROTO_TRUNK1) */
+
+#if	defined(IPPROTO_TRUNK2)
+	case IPPROTO_TRUNK2:
+	    s = "TRUNK2";
+	    break;
+#endif	/* defined(IPPROTO_TRUNK2) */
+
+#if	defined(IPPROTO_LEAF1)
+	case IPPROTO_LEAF1:
+	    s = "LEAF1";
+	    break;
+#endif	/* defined(IPPROTO_LEAF1) */
+
+#if	defined(IPPROTO_LEAF2)
+	case IPPROTO_LEAF2:
+	    s = "LEAF2";
+	    break;
+#endif	/* defined(IPPROTO_LEAF2) */
+
+#if	defined(IPPROTO_RDP)
+	case IPPROTO_RDP:
+	    s = "RDP";
+	    break;
+#endif	/* defined(IPPROTO_RDP) */
+
+#if	defined(IPPROTO_IRTP)
+	case IPPROTO_IRTP:
+	    s = "IRTP";
+	    break;
+#endif	/* defined(IPPROTO_IRTP) */
+
+#if	defined(IPPROTO_TP)
+	case IPPROTO_TP:
+	    s = "TP";
+	    break;
+#endif	/* defined(IPPROTO_TP) */
+
+#if	defined(IPPROTO_BLT)
+	case IPPROTO_BLT:
+	    s = "BLT";
+	    break;
+#endif	/* defined(IPPROTO_BLT) */
+
+#if	defined(IPPROTO_NSP)
+	case IPPROTO_NSP:
+	    s = "NSP";
+	    break;
+#endif	/* defined(IPPROTO_NSP) */
+
+#if	defined(IPPROTO_INP)
+	case IPPROTO_INP:
+	    s = "INP";
+	    break;
+#endif	/* defined(IPPROTO_INP) */
+
+#if	defined(IPPROTO_SEP)
+	case IPPROTO_SEP:
+	    s = "SEP";
+	    break;
+#endif	/* defined(IPPROTO_SEP) */
+
+#if	defined(IPPROTO_3PC)
+	case IPPROTO_3PC:
+	    s = "3PC";
+	    break;
+#endif	/* defined(IPPROTO_3PC) */
+
+#if	defined(IPPROTO_IDPR)
+	case IPPROTO_IDPR:
+	    s = "IDPR";
+	    break;
+#endif	/* defined(IPPROTO_IDPR) */
+
+#if	defined(IPPROTO_XTP)
+	case IPPROTO_XTP:
+	    s = "XTP";
+	    break;
+#endif	/* defined(IPPROTO_XTP) */
+
+#if	defined(IPPROTO_DDP)
+	case IPPROTO_DDP:
+	    s = "DDP";
+	    break;
+#endif	/* defined(IPPROTO_DDP) */
+
+#if	defined(IPPROTO_CMTP)
+	case IPPROTO_CMTP:
+	    s = "CMTP";
+	    break;
+#endif	/* defined(IPPROTO_CMTP) */
+
+#if	defined(IPPROTO_TPXX)
+	case IPPROTO_TPXX:
+	    s = "TPXX";
+	    break;
+#endif	/* defined(IPPROTO_TPXX) */
+
+#if	defined(IPPROTO_IL)
+	case IPPROTO_IL:
+	    s = "IL";
+	    break;
+#endif	/* defined(IPPROTO_IL) */
+
+#if	defined(IPPROTO_IPV6)
+	case IPPROTO_IPV6:
+	    s = "IPV6";
+	    break;
+#endif	/* defined(IPPROTO_IPV6) */
+
+#if	defined(IPPROTO_SDRP)
+	case IPPROTO_SDRP:
+	    s = "SDRP";
+	    break;
+#endif	/* defined(IPPROTO_SDRP) */
+
+#if	defined(IPPROTO_ROUTING)
+	case IPPROTO_ROUTING:
+	    s = "ROUTING";
+	    break;
+#endif	/* defined(IPPROTO_ROUTING) */
+
+#if	defined(IPPROTO_FRAGMENT)
+	case IPPROTO_FRAGMENT:
+	    s = "FRAGMNT";
+	    break;
+#endif	/* defined(IPPROTO_FRAGMENT) */
+
+#if	defined(IPPROTO_IDRP)
+	case IPPROTO_IDRP:
+	    s = "IDRP";
+	    break;
+#endif	/* defined(IPPROTO_IDRP) */
+
+#if	defined(IPPROTO_RSVP)
+	case IPPROTO_RSVP:
+	    s = "RSVP";
+	    break;
+#endif	/* defined(IPPROTO_RSVP) */
+
+#if	defined(IPPROTO_GRE)
+	case IPPROTO_GRE:
+	    s = "GRE";
+	    break;
+#endif	/* defined(IPPROTO_GRE) */
+
+#if	defined(IPPROTO_MHRP)
+	case IPPROTO_MHRP:
+	    s = "MHRP";
+	    break;
+#endif	/* defined(IPPROTO_MHRP) */
+
+#if	defined(IPPROTO_BHA)
+	case IPPROTO_BHA:
+	    s = "BHA";
+	    break;
+#endif	/* defined(IPPROTO_BHA) */
+
+#if	defined(IPPROTO_ESP)
+	case IPPROTO_ESP:
+	    s = "ESP";
+	    break;
+#endif	/* defined(IPPROTO_ESP) */
+
+#if	defined(IPPROTO_AH)
+	case IPPROTO_AH:
+	    s = "AH";
+	    break;
+#endif	/* defined(IPPROTO_AH) */
+
+#if	defined(IPPROTO_INLSP)
+	case IPPROTO_INLSP:
+	    s = "INLSP";
+	    break;
+#endif	/* defined(IPPROTO_INLSP) */
+
+#if	defined(IPPROTO_SWIPE)
+	case IPPROTO_SWIPE:
+	    s = "SWIPE";
+	    break;
+#endif	/* defined(IPPROTO_SWIPE) */
+
+#if	defined(IPPROTO_NHRP)
+	case IPPROTO_NHRP:
+	    s = "NHRP";
+	    break;
+#endif	/* defined(IPPROTO_NHRP) */
+
+#if	defined(IPPROTO_NONE)
+	case IPPROTO_NONE:
+	    s = "NONE";
+	    break;
+#endif	/* defined(IPPROTO_NONE) */
+
+#if	defined(IPPROTO_DSTOPTS)
+	case IPPROTO_DSTOPTS:
+	    s = "DSTOPTS";
+	    break;
+#endif	/* defined(IPPROTO_DSTOPTS) */
+
+#if	defined(IPPROTO_AHIP)
+	case IPPROTO_AHIP:
+	    s = "AHIP";
+	    break;
+#endif	/* defined(IPPROTO_AHIP) */
+
+#if	defined(IPPROTO_CFTP)
+	case IPPROTO_CFTP:
+	    s = "CFTP";
+	    break;
+#endif	/* defined(IPPROTO_CFTP) */
+
+#if	defined(IPPROTO_SATEXPAK)
+	case IPPROTO_SATEXPAK:
+	    s = "SATEXPK";
+	    break;
+#endif	/* defined(IPPROTO_SATEXPAK) */
+
+#if	defined(IPPROTO_KRYPTOLAN)
+	case IPPROTO_KRYPTOLAN:
+	    s = "KRYPTOL";
+	    break;
+#endif	/* defined(IPPROTO_KRYPTOLAN) */
+
+#if	defined(IPPROTO_RVD)
+	case IPPROTO_RVD:
+	    s = "RVD";
+	    break;
+#endif	/* defined(IPPROTO_RVD) */
+
+#if	defined(IPPROTO_IPPC)
+	case IPPROTO_IPPC:
+	    s = "IPPC";
+	    break;
+#endif	/* defined(IPPROTO_IPPC) */
+
+#if	defined(IPPROTO_ADFS)
+	case IPPROTO_ADFS:
+	    s = "ADFS";
+	    break;
+#endif	/* defined(IPPROTO_ADFS) */
+
+#if	defined(IPPROTO_SATMON)
+	case IPPROTO_SATMON:
+	    s = "SATMON";
+	    break;
+#endif	/* defined(IPPROTO_SATMON) */
+
+#if	defined(IPPROTO_VISA)
+	case IPPROTO_VISA:
+	    s = "VISA";
+	    break;
+#endif	/* defined(IPPROTO_VISA) */
+
+#if	defined(IPPROTO_IPCV)
+	case IPPROTO_IPCV:
+	    s = "IPCV";
+	    break;
+#endif	/* defined(IPPROTO_IPCV) */
+
+#if	defined(IPPROTO_CPNX)
+	case IPPROTO_CPNX:
+	    s = "CPNX";
+	    break;
+#endif	/* defined(IPPROTO_CPNX) */
+
+#if	defined(IPPROTO_CPHB)
+	case IPPROTO_CPHB:
+	    s = "CPHB";
+	    break;
+#endif	/* defined(IPPROTO_CPHB) */
+
+#if	defined(IPPROTO_WSN)
+	case IPPROTO_WSN:
+	    s = "WSN";
+	    break;
+#endif	/* defined(IPPROTO_WSN) */
+
+#if	defined(IPPROTO_PVP)
+	case IPPROTO_PVP:
+	    s = "PVP";
+	    break;
+#endif	/* defined(IPPROTO_PVP) */
+
+#if	defined(IPPROTO_BRSATMON)
+	case IPPROTO_BRSATMON:
+	    s = "BRSATMN";
+	    break;
+#endif	/* defined(IPPROTO_BRSATMON) */
+
+#if	defined(IPPROTO_WBMON)
+	case IPPROTO_WBMON:
+	    s = "WBMON";
+	    break;
+#endif	/* defined(IPPROTO_WBMON) */
+
+#if	defined(IPPROTO_WBEXPAK)
+	case IPPROTO_WBEXPAK:
+	    s = "WBEXPAK";
+	    break;
+#endif	/* defined(IPPROTO_WBEXPAK) */
+
+#if	defined(IPPROTO_EON)
+	case IPPROTO_EON:
+	    s = "EON";
+	    break;
+#endif	/* defined(IPPROTO_EON) */
+
+#if	defined(IPPROTO_VMTP)
+	case IPPROTO_VMTP:
+	    s = "VMTP";
+	    break;
+#endif	/* defined(IPPROTO_VMTP) */
+
+#if	defined(IPPROTO_SVMTP)
+	case IPPROTO_SVMTP:
+	    s = "SVMTP";
+	    break;
+#endif	/* defined(IPPROTO_SVMTP) */
+
+#if	defined(IPPROTO_VINES)
+	case IPPROTO_VINES:
+	    s = "VINES";
+	    break;
+#endif	/* defined(IPPROTO_VINES) */
+
+#if	defined(IPPROTO_TTP)
+	case IPPROTO_TTP:
+	    s = "TTP";
+	    break;
+#endif	/* defined(IPPROTO_TTP) */
+
+#if	defined(IPPROTO_IGP)
+	case IPPROTO_IGP:
+	    s = "IGP";
+	    break;
+#endif	/* defined(IPPROTO_IGP) */
+
+#if	defined(IPPROTO_DGP)
+	case IPPROTO_DGP:
+	    s = "DGP";
+	    break;
+#endif	/* defined(IPPROTO_DGP) */
+
+#if	defined(IPPROTO_TCF)
+	case IPPROTO_TCF:
+	    s = "TCF";
+	    break;
+#endif	/* defined(IPPROTO_TCF) */
+
+#if	defined(IPPROTO_IGRP)
+	case IPPROTO_IGRP:
+	    s = "IGRP";
+	    break;
+#endif	/* defined(IPPROTO_IGRP) */
+
+#if	defined(IPPROTO_OSPFIGP)
+	case IPPROTO_OSPFIGP:
+	    s = "OSPFIGP";
+	    break;
+#endif	/* defined(IPPROTO_OSPFIGP) */
+
+#if	defined(IPPROTO_SRPC)
+	case IPPROTO_SRPC:
+	    s = "SRPC";
+	    break;
+#endif	/* defined(IPPROTO_SRPC) */
+
+#if	defined(IPPROTO_LARP)
+	case IPPROTO_LARP:
+	    s = "LARP";
+	    break;
+#endif	/* defined(IPPROTO_LARP) */
+
+#if	defined(IPPROTO_MTP)
+	case IPPROTO_MTP:
+	    s = "MTP";
+	    break;
+#endif	/* defined(IPPROTO_MTP) */
+
+#if	defined(IPPROTO_AX25)
+	case IPPROTO_AX25:
+	    s = "AX25";
+	    break;
+#endif	/* defined(IPPROTO_AX25) */
+
+#if	defined(IPPROTO_IPEIP)
+	case IPPROTO_IPEIP:
+	    s = "IPEIP";
+	    break;
+#endif	/* defined(IPPROTO_IPEIP) */
+
+#if	defined(IPPROTO_MICP)
+	case IPPROTO_MICP:
+	    s = "MICP";
+	    break;
+#endif	/* defined(IPPROTO_MICP) */
+
+#if	defined(IPPROTO_SCCSP)
+	case IPPROTO_SCCSP:
+	    s = "SCCSP";
+	    break;
+#endif	/* defined(IPPROTO_SCCSP) */
+
+#if	defined(IPPROTO_ETHERIP)
+	case IPPROTO_ETHERIP:
+	    s = "ETHERIP";
+	    break;
+#endif	/* defined(IPPROTO_ETHERIP) */
+
+#if	defined(IPPROTO_ENCAP)
+# if	!defined(IPPROTO_IPIP) || IPPROTO_IPIP!=IPPROTO_ENCAP
+	case IPPROTO_ENCAP:
+	    s = "ENCAP";
+	    break;
+# endif	/* !defined(IPPROTO_IPIP) || IPPROTO_IPIP!=IPPROTO_ENCAP */
+#endif	/* defined(IPPROTO_ENCAP) */
+
+#if	defined(IPPROTO_APES)
+	case IPPROTO_APES:
+	    s = "APES";
+	    break;
+#endif	/* defined(IPPROTO_APES) */
+
+#if	defined(IPPROTO_GMTP)
+	case IPPROTO_GMTP:
+	    s = "GMTP";
+	    break;
+#endif	/* defined(IPPROTO_GMTP) */
+
+#if	defined(IPPROTO_DIVERT)
+	case IPPROTO_DIVERT:
+	    s = "DIVERT";
+	    break;
+#endif	/* defined(IPPROTO_DIVERT) */
+
 	default:
 	    s = (char *)NULL;
 	}
@@ -1136,10 +1730,10 @@ printname(nl)
 	 */
 	    safestrprt(Lf->nm, stdout, 0);
 	    ps++;
-	    if (!Lf->li[0].as && !Lf->li[1].as)
+	    if (!Lf->li[0].af && !Lf->li[1].af)
 		goto print_nma;
 	}
-	if (Lf->li[0].as || Lf->li[1].as) {
+	if (Lf->li[0].af || Lf->li[1].af) {
 	    if (ps)
 		putchar(' ');
 	/*
@@ -1184,6 +1778,26 @@ printname(nl)
 	    goto print_nma;
 #endif	/* defined(HASPRIVNMCACHE) */
 
+	if (Lf->lmi_srch) {
+	    struct mounts *mp;
+	/*
+	 * Do a deferred local mount info table search for the file system
+	 * (mounted) directory name and inode number, and mounted device name.
+	 */
+	    for (mp = readmnt(); mp; mp = mp->next) {
+		if (Lf->dev == mp->dev) {
+		    Lf->fsdir = mp->dir;
+		    Lf->fsdev = mp->fsname;
+
+#if	defined(HASFSINO)
+		    Lf->fs_ino = mp->inode;
+#endif	/* defined(HASFSINO) */
+
+		    break;
+		}
+	    }
+	    Lf->lmi_srch = 0;
+	}
 	if (Lf->fsdir || Lf->fsdev) {
 
 	/*
@@ -1200,7 +1814,7 @@ printname(nl)
 	    &&  (cp = ncache_lookup(buf, sizeof(buf), &fp))) {
 		if (*cp == '\0')
 		    goto print_nma;
-		if (fp) {
+		if (fp && Lf->fsdir) {
 		    if (*cp != '/') {
 			cp1 = strrchr(Lf->fsdir, '/');
 			if (cp1 == (char *)NULL ||  *(cp1 + 1) != '\0')
@@ -1446,6 +2060,12 @@ printunkaf(fam)
 	    break;
 #endif	/* defined(AF_INET) */
 
+#if	defined(AF_INET6)
+	case AF_INET6:
+	    s = "INET6";
+	    break;
+#endif	/* defined(AF_INET6) */
+
 #if	defined(AF_IMPLINK)
 	case AF_IMPLINK:
 	    s = "IMPLINK";
