@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: proc.c,v 1.11 98/03/09 12:59:04 abe Exp $";
+static char *rcsid = "$Id: proc.c,v 1.16 98/08/18 08:07:33 abe Exp $";
 #endif
 
 
@@ -67,8 +67,8 @@ alloc_lfile(nm, num)
 	char *nm;
 	int num;
 {
-	char buf[32], *cp;
-	struct str_lst *sp;
+	char *cp;
+	struct fd_lst *fp;
 
 	if (Lf) {
 /*
@@ -94,8 +94,9 @@ alloc_lfile(nm, num)
  */
 	Lf->access = Lf->lock = ' ';
 	Lf->dev_def = Lf->inp_ty = Lf->is_com = Lf->is_nfs = Lf->is_stream
-		    = Lf->off_def = Lf->sz_def = (unsigned char)0;
-	Lf->li[0].as = Lf->li[1].as = 0;
+		    = Lf->lmi_srch = Lf->off_def = Lf->sz_def
+		    = (unsigned char)0;
+	Lf->li[0].af = Lf->li[1].af = 0;
 	Lf->lts.type = -1;
 
 #if	defined(HASTCPTPIQ)
@@ -157,12 +158,16 @@ alloc_lfile(nm, num)
 	if ((cp = nm)) {
 	    while (*cp && *cp == ' ')
 		cp++;
-	} else {
-	    (void) sprintf(buf, "%d", num);
-	    cp = buf;
 	}
-	for (sp = Fdl; sp; sp = sp->next) {
-	    if (strcmp(sp->str, cp) == 0) {
+	for (fp = Fdl; fp; fp = fp->next) {
+	    if (cp) {
+		if (fp->nm && strcmp(fp->nm, cp) == 0) {
+		    Lf->sf |= SELFD;
+		    return;
+		}
+		continue;
+	    }
+	    if (num >= fp->lo && num <= fp->hi) {
 		Lf->sf |= SELFD;
 		return;
 	    }
@@ -232,32 +237,52 @@ alloc_lproc(pid, pgrp, ppid, uid, cmd, pss, sf)
  */
 
 void
-ent_inaddr(la, lp, fa, fp)
-	struct in_addr *la;		/* local Internet address */
+ent_inaddr(la, lp, fa, fp, af)
+	unsigned char *la;		/* local Internet address */
 	int lp;				/* local port */
-	struct in_addr *fa;		/* foreign Internet address -- may
+	unsigned char *fa;		/* foreign Internet address -- may
 					 * be NULL to indicate no foreign
 					 * address is known */
 	int fp;				/* foreign port */
+	int af;				/* address family -- e.g, AF_INET,
+					 * AF_INET */
 {
+	int m;
+
 	if (la) {
-	    Lf->li[0].as = 1;
-	    Lf->li[0].ia = *la;
+	    Lf->li[0].af = af;
+
+#if	defined(HASIPv6)
+	    if (af == AF_INET6)
+		Lf->li[0].ia.a6 = *(struct in6_addr *)la;
+	    else
+#endif	/* defined(HASIPv6) */
+
+	        Lf->li[0].ia.a4 = *(struct in_addr *)la;
 	    Lf->li[0].p = lp;
 	} else
-	    Lf->li[0].as = 0;
+	    Lf->li[0].af = 0;
 	if (fa) {
-	    Lf->li[1].as = 1;
-	    Lf->li[1].ia = *fa;
+	    Lf->li[1].af = af;
+
+#if	defined(HASIPv6)
+	    if (af == AF_INET6)
+		Lf->li[1].ia.a6 = *(struct in6_addr *)fa;
+	    else
+#endif	/* defined(HASIPv6) */
+
+		Lf->li[1].ia.a4 = *(struct in_addr *)fa;
 	    Lf->li[1].p = fp;
 	} else
-	    Lf->li[1].as = 0;
+	    Lf->li[1].af = 0;
 /*
  * If network address matching has been selected, check both addresses.
  */
-	if ((Selflags & SELNA) && Nwad
-	&&  ((fa && is_nw_addr(fa, fp)) || (la && is_nw_addr(la, lp))))
-	    Lf->sf |= SELNA;
+	if ((Selflags & SELNA) && Nwad) {
+	    m = (fa && is_nw_addr(fa, fp, af)) ? 1 : 0;
+	    if ((m |= (la && is_nw_addr(la, lp, af)) ? 1 : 0))
+		Lf->sf |= SELNA;
+	}
 }
 
 
@@ -291,10 +316,10 @@ examine_lproc()
 	}
 	if (Lp->pss && Npid == 1 && sbp) {
 	    print_init();
-	    print_proc();
+	    (void) print_proc();
 	    PrPass++;
 	    if (PrPass < 2)
-		print_proc();
+		(void) print_proc();
 	    Lp->pss = 0;
 	}
 /*
@@ -571,17 +596,17 @@ link_lfile()
  * print_proc() - print process
  */
 
-void
+int
 print_proc()
 {
 	char *cp;
 	int lc, st, ty;
-	int sel = 0;
+	int rv = 0;
 /*
  * If nothing in the process has been selected, skip it.
  */
 	if (!Lp->pss)
-		return;
+	    return(0);
 	if (Fterse) {
 
 	/*
@@ -592,15 +617,15 @@ print_proc()
 	    if (Fand) {
 		for (Lf = Lp->file; Lf; Lf = Lf->next) {
 		    if (is_file_sel(Lf)) {
-			sel++;
+			rv = 1;
 			break;
 		    }
 		}
 	    } else
-		sel = 1;
-	    if (sel)
+		rv = 1;
+	    if (rv)
 		(void) printf("%d\n", Lp->pid);
-	    return;
+	    return(rv);
 	}
 /*
  * If fields have been selected, output the process-only ones, provided
@@ -612,7 +637,8 @@ print_proc()
 		    break;
 	    }
 	    if (!Lf)
-		return;
+		return(rv);
+	    rv = 1;
 	    (void) printf("%c%d%c", LSOF_FID_PID, Lp->pid, Terminator);
 	    if (FieldSel[LSOF_FIX_PGRP].st && Fpgrp)
 		(void) printf("%c%d%c", LSOF_FID_PGRP, Lp->pgrp, Terminator);
@@ -643,6 +669,7 @@ print_proc()
 	for (Lf = Lp->file; Lf; Lf = Lf->next) {
 	    if (!is_file_sel(Lf))
 		continue;
+	    rv = 1;
 	/*
 	 * If no field output selected, print dialects-specific formatted
 	 * output.
@@ -777,4 +804,5 @@ print_proc()
 	    if (Terminator == '\0' && lc)
 		putchar('\n');
 	}
+	return(rv);
 }
