@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: store.c,v 1.21 96/04/18 10:26:25 abe Exp $";
+static char *rcsid = "$Id: store.c,v 1.14 98/08/18 08:07:35 abe Exp $";
 #endif
 
 
@@ -43,16 +43,28 @@ static char *rcsid = "$Id: store.c,v 1.21 96/04/18 10:26:25 abe Exp $";
  * Global storage definitions
  */
 
-struct str_lst *Cmdl = NULL;	/* command names selected with -c */
+#if	defined(HASBLKDEV)
+struct l_dev *BDevtp = (struct l_dev *)NULL;
+				/* block device table pointer */
+int BNdev = 0;			/* number of entries in BDevtp[] */
+struct l_dev **BSdev = (struct l_dev **)NULL;
+				/* pointer to BDevtp[] pointers, sorted
+				 * by device */
+#endif	/* defined(HASBLKDEV) */
+
+int CkPasswd = 0;		/* time to check /etc/passwd for change */
+struct str_lst *Cmdl = (struct str_lst *)NULL;
+				/* command names selected with -c */
+int CmdColW;			/* COMMAND column width */
 
 #if	defined(HASDCACHE)
 unsigned DCcksum;		/* device cache file checksum */
 int DCfd = -1;			/* device cache file descriptor */
-FILE *DCfs = NULL;		/* stream pointer for DCfd */
-char *DCpathArg = NULL;		/* device cache path from -D[b|r|u]<path> */
+FILE *DCfs = (FILE *)NULL;	/* stream pointer for DCfd */
+char *DCpathArg = (char *)NULL;	/* device cache path from -D[b|r|u]<path> */
 char *DCpath[] = {		/* device cache paths, indexed by DCpathX
 				 *when it's >= 0 */
-	NULL, NULL, NULL, NULL
+	(char *)NULL, (char *)NULL, (char *)NULL, (char *)NULL
 };
 int DCpathX = -1;		/* device cache path index:
 				 *	-1 = path not defined
@@ -61,6 +73,8 @@ int DCpathX = -1;		/* device cache path index:
 				 *	 2 = defined via HASSYSDC
 				 *	 3 = defined via HASPERSDC and
 				 *	     HASPERSDCPATH */
+int DCrebuilt = 0;		/* an unsafe device cache file has been
+				 * rebuilt */
 int DCstate = 3;		/* device cache state:
 				 *	0 = ignore (-Di)
 				 *	1 = build (-Db[path])
@@ -68,13 +82,22 @@ int DCstate = 3;		/* device cache state:
 				 *	3 = update; read and rebuild if
 				 *	    necessary (-Du[path])
 				 */
+int DCunsafe = 0;		/* device cache file is potentially unsafe,
+				 * (The [cm]time check failed.) */
 #endif	/* defined(HASDCACHE) */
 
 int DChelp = 0;			/* -D? status */
 
-struct l_dev *Devtp = NULL;	/* device table pointer */
+int DevColW;			/* DEVICE column width */
+struct l_dev *Devtp = (struct l_dev *)NULL;
+				/* device table pointer */
 int Fand = 0;			/* -a option status */
 int Fblock = 0;			/* -b option status */
+int FdColW;			/* FD column width */
+int Ffilesys = 0;		/* -f option status:
+				 *    0 = paths may be file systems
+				 *    1 = paths are just files
+				 *    2 = paths must be file systems */
 
 #if	defined(HASNCACHE)
 int Fncache = 1;		/* -C option status */
@@ -83,23 +106,45 @@ int Fncache = 1;		/* -C option status */
 int Ffield = 0;			/* -f and -F status */
 int Fhelp = 0;			/* -h option status */
 int Fhost = 1;			/* -H option status */
-int Fnet = 0;			/* -n option status */
-int Fnfs = 0;			/* -N option status */
+int Fnet = 0;			/* -i option status: 0==none, 1==find all,
+				 * 2==some found*/
+int Fnfs = 0;			/* -N option status: 0==none, 1==find all,
+				 * 2==some found*/
 int Foffset = 0;		/* -o option status */
 int Fovhd = 0;			/* -O option status */
 int Fport = 1;			/* -P option status */
+
+#if	defined(HASPMAPENABLED)
+int FportMap = 1;		/* +|-M option status */
+#else	/* !defined(HASPMAPENABLED) */
+int FportMap = 0;		/* +|-M option status */
+#endif	/* defined(HASPMAPENABLED) */
+
 int Fpgrp = 0;			/* -g option status */
+
+#if	defined(HASPPID)
+int Fppid = 0;			/* -R option status */
+#endif	/* defined(HASPPID) */
+
 int Fsize = 0;			/* -s option status */
+int Ftcptpi = TCPTPI_STATE;	/* -T option status */
 int Fterse = 0;			/* -t option status */
 int Funix = 0;			/* -U option status */
 int Futol = 1;			/* -l option status */
-int Fwarn = 0;			/* -w option status */
+int Fverbose = 0;		/* -V option status */
+
+#if	defined(WARNINGSTATE)
+int Fwarn = 1;			/* +|-w option status */
+#else	/* !defined(WARNINGSTATE) */
+int Fwarn = 0;			/* +|-w option status */
+#endif	/* defined(WARNINGSTATE) */
 
 #if	defined(HASXOPT_VALUE)
 int Fxopt = HASXOPT_VALUE;	/* -X option status */
 #endif	/* defined(HASXOPT_VALUE) */
 
-struct str_lst *Fdl = NULL;	/* file descriptors selected with -d */
+struct fd_lst *Fdl = (struct fd_lst *)NULL;
+				/* file descriptors selected with -d */
 
 struct fieldsel FieldSel[] = {
 	{ LSOF_FID_ACCESS,	0,	LSOF_FNM_ACCESS	},	/*  0 */
@@ -116,11 +161,13 @@ struct fieldsel FieldSel[] = {
 	{ LSOF_FID_PID,		1,	LSOF_FNM_PID	},	/* 11 */
 	{ LSOF_FID_PGRP,	0,	LSOF_FNM_PGRP	},	/* 12 */
 	{ LSOF_FID_PROTO,	0,	LSOF_FNM_PROTO	},	/* 13 */
-	{ LSOF_FID_SIZE,	0,	LSOF_FNM_SIZE	},	/* 14 */
-	{ LSOF_FID_STREAM,	0,	LSOF_FNM_STREAM	},	/* 15 */
-	{ LSOF_FID_TYPE,	0,	LSOF_FNM_TYPE	},	/* 16 */
-	{ LSOF_FID_UID,		0,	LSOF_FNM_UID	},	/* 17 */
-	{ LSOF_FID_TERM,	0,	LSOF_FNM_TERM	},	/* 18 */
+	{ LSOF_FID_PPID,	0,	LSOF_FNM_PPID	},	/* 14 */
+	{ LSOF_FID_SIZE,	0,	LSOF_FNM_SIZE	},	/* 15 */
+	{ LSOF_FID_STREAM,	0,	LSOF_FNM_STREAM	},	/* 16 */
+	{ LSOF_FID_TYPE,	0,	LSOF_FNM_TYPE	},	/* 17 */
+	{ LSOF_FID_TCPTPI,	0,	LSOF_FNM_TCPTPI	},	/* 18 */
+	{ LSOF_FID_UID,		0,	LSOF_FNM_UID	},	/* 19 */
+	{ LSOF_FID_TERM,	0,	LSOF_FNM_TERM	},	/* 20 */
 
 #if	defined(HASFIELDAP1)
 	{ '1',			0,	HASFIELDAP1},		/* TERM + 1 */
@@ -162,15 +209,18 @@ struct fieldsel FieldSel[] = {
 };
 
 int Hdr = 0;			/* header print status */
-struct lfile *Lf = NULL;	/* current local file structure */
-struct lproc *Lp = NULL;	/* current local process table entry */
-struct lproc *Lproc = NULL;	/* local process table */
-char *Memory = NULL;		/* core file path */
-struct mounts *Mtab = NULL;	/* mounted devices */
+struct lfile *Lf = (struct lfile *)NULL;
+				/* current local file structure */
+struct lproc *Lp = (struct lproc *)NULL;
+				/* current local process table entry */
+struct lproc *Lproc = (struct lproc *)NULL;
+				/* local process table */
+char *Memory = (char *)NULL;	/* core file path */
 
 #if	defined(HASPROCFS)
-struct mounts *Mtprocfs = NULL;	/* /proc mount entry */
-#endif
+struct mounts *Mtprocfs = (struct mounts *)NULL;
+				/* /proc mount entry */
+#endif	/* defined(HASPROCFS) */
 
 int Mxpgrp = 0;			/* maximum process groip ID table entries */
 int Mxpid = 0;			/* maximum PID table entries */
@@ -178,10 +228,19 @@ int Mxuid = 0;			/* maximum UID table entries */
 gid_t Mygid;			/* real GID of this lsof process */
 int Mypid;			/* lsof's process ID */
 uid_t Myuid;			/* real UID of this lsof process */
-char *Namech = NULL;		/* name characters for printing */
-int Ndev = 0;			/* number of device table entries */
+char *Namech = (char *)NULL;	/* name characters for printing */
+int Ndev = 0;			/* number of entries in Devtp[] */
+
+#if	defined(HASNLIST)
+struct NLIST_TYPE *Nl = (struct NLIST_TYPE *)NULL;
+				/* kernel name list */
+int Nll = 0;			/* Nl calloc'd length */
+#endif	/* defined(HASNLIST) */
+
 int Nlproc = 0;			/* number of entries in Lproc[] */
-char *Nmlst = NULL;		/* namelist file path */
+int NmColW;			/* NAME column width */
+char *Nmlst = (char *)NULL;	/* namelist file path */
+int NodeColW;			/* NODE column width */
 int Npgrp = 0;			/* -g option count */
 int Npid = 0;			/* -p option count */
 int Npuns;			/* number of unselected PIDs (starts at Npid) */
@@ -189,27 +248,56 @@ int Ntype;			/* node type (see N_* symbols) */
 int Nuid = 0;			/* -u option count */
 int Nuidexcl = 0;		/* -u option count of UIDs excluded */
 int Nuidincl = 0;		/* -u option count of UIDs included */
-struct nwad *Nwad = NULL;	/* list of network addresses */
-struct lfile *Plf = NULL;	/* previous local file structure */
+struct nwad *Nwad = (struct nwad *)NULL;
+				/* list of network addresses */
+int OffDecDig = OFFDECDIG;	/* offset decimal form (0t...) digit limit */
+int OffColW;			/* OFFSET column width */
+int PgrpColW;			/* PGRP column width */
+int PidColW;			/* PID column width */
+struct lfile *Plf = (struct lfile *)NULL;
+				/* previous local file structure */
 char *Pn;			/* program name */
+int PpidColW;			/* PPID column width */
 
 #if	defined(HASPROCFS)
-struct procfsid *Procfsid = NULL;
+int Procfind = 0;		/* 1 when searching for an proc file system
+				 * file and one was found */
+struct procfsid *Procfsid = (struct procfsid *)NULL;
 				/* proc file system PID search table */
 int Procsrch = 0;		/* 1 if searching for any proc file system
 				 * file */
-#endif
+#endif	/* defined(HASPROCFS) */
 
+int PrPass = 0;			/* print pass: 0 = compute column widths
+				 *	       1 = print */
 int RptTm = 0;			/* repeat time -- set by -r */
-struct l_dev **Sdev = NULL;	/* pointer to device table pointers, sorted
+struct l_dev **Sdev = (struct l_dev **)NULL;
+				/* pointer to Devtp[] pointers, sorted
 				 * by device */
 int Selall = 1;			/* all processes are selected (default) */
 int Selflags = 0;		/* selection flags -- see SEL* in lsof.h */
 int Setgid = 0;			/* setgid state */
+int Selinet = 0;		/* select only Internet socket files */
 int Setuidroot = 0;		/* setuid-root state */
-struct sfile *Sfile = NULL;	/* chain of files to search for */
-int *Spgrp = NULL;		/* process group IDs to search for */
-int *Spid = NULL;		/* Process IDs to search for */
-struct seluid *Suid = NULL;	/* User IDs to include or exclude */
+struct sfile *Sfile = (struct sfile *)NULL;
+				/* chain of files to search for */
+struct int_lst *Spgrp = (struct int_lst *)NULL;
+				/* process group IDs to search for */
+struct int_lst *Spid = (struct int_lst *)NULL;
+				/* Process IDs to search for */
+struct seluid *Suid = (struct seluid *)NULL;
+				/* User IDs to include or exclude */
+int SzColW;			/* SIZE column width */
+int SzOffColW;			/* SIZE/OFF column width */
+char *SzOffFmt_0t = (char *)NULL;
+				/* SZOFFTYPE 0t%u printf specification */
+char *SzOffFmt_d = (char *)NULL;
+				/* SZOFFTYPE %d printf  specification */
+char *SzOffFmt_dv = (char *)NULL;
+				/* SZOFFTYPE %*d printf  specification */
+char *SzOffFmt_x = (char *)NULL;
+				/* SZOFFTYPE %#x printf  specification */
 char Terminator = '\n';		/* output field terminator */
 int TmLimit = TMLIMIT;		/* Readlink() and stat() timeout (seconds) */
+int TypeColW;			/* TYPE column width */
+int UserColW;			/* USER column width */
